#include "FEModelIndexRangeDlg.h"
#include <qlabel.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qpushbutton.h>

CFEModelIndexRangeDlg::CFEModelIndexRangeDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CFEModelIndexRangeDlg_base( parent, name, modal, f )
{
	// min spinbox & slider
	connect( IDC_SPIN_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_MIN, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_MIN, SLOT( setValue( int ) ) );

	// max spinbox & slider
	connect( IDC_SPIN_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_MAX, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_MAX, SLOT( setValue( int ) ) );

	// connect signal-slot between IDC_SPIN_MIN and slotMinChanged
	connect( IDC_SPIN_MIN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotMinChanged( int ) ) );

	// connect signal-slot between IDC_SPIN_MAX and slotMaxChanged
	connect( IDC_SPIN_MAX, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotMaxChanged( int ) ) );

	// ok & cancel buttons
	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ),
		this, SLOT( slotCancel() ) );
} // CFEModelIndexRangeDlg

CFEModelIndexRangeDlg::~CFEModelIndexRangeDlg()
{
} // ~CFEModelIndexRangeDlg

bool CFEModelIndexRangeDlg::init( int min, int max )
{
	IDC_SPIN_MIN->setRange( 0, 255 );
	IDC_SLIDER_MIN->setRange( 0, 255 );

	IDC_SPIN_MAX->setRange( 0, 255 );
	IDC_SLIDER_MAX->setRange( 0, 255 );

	IDC_SPIN_MIN->setValue( min );
	IDC_SPIN_MAX->setValue( max );

	IDC_LBL_DESCRIPTION->setText(
		tr( 
"  Value which has less than Min value will be shown as [GRAY] color.\n\
  Value which has greater than Max value will be shown as [WHITE] color."
		) );

	return true;
} // init

// slot
void CFEModelIndexRangeDlg::slotMinChanged( int value )
{
	int nMax = IDC_SPIN_MAX->value();

	if ( value >= nMax )
		IDC_SPIN_MAX->setValue( value );
} // slotMinChanged

// slot
void CFEModelIndexRangeDlg::slotMaxChanged( int value )
{
	int nMin = IDC_SPIN_MIN->value();

	if ( value <= nMin )
		IDC_SPIN_MIN->setValue( value );
} // slotMaxChanged

// slot
void CFEModelIndexRangeDlg::slotOk()
{
	QDialog::accept();
} // slotOk

// slot
void CFEModelIndexRangeDlg::slotCancel()
{
	QDialog::reject();
} // slotCancel
