#include "GLView.h"
#include "VolumeFile.h"
#include "MainWindow.h"
#include "FEModelIndexRangeDlg.h"

#include <math.h>
#include <qpainter.h>
#include <qbrush.h>
#include <qstatusbar.h>
#include <qcolor.h>
#include <qcolordialog.h>
#include <qspinbox.h>
#include <qpixmap.h>
#include <qcursor.h>
#include <qtoolbutton.h>

//XXX
#include <qdatetime.h>
//XXX

#include "cursor_xpm.h"


// int CGLView::m_nListCount = 1;

CGLView::CGLView( QWidget *parent, const char *name, const QGLWidget *shareWidget, WFlags f )
	: QGLWidget( parent, name, shareWidget, f )
{
	setBackgroundMode( NoBackground );
	setFocusPolicy( QWidget::StrongFocus );

	m_pVolume = NULL;

	m_bResizeFlag = false;
	m_bIsTracking = false;

	m_bControlKey = false;

	m_bDefaultView = true;
	m_bZoomAll = true;

	m_bWhiteBlack = true;
	m_bSingleColor = false;
	m_bIndexColor = false;

	m_bShowTitle = true;
	m_bShowGridLine = true;
	m_bShowBoundingBox = true;

	m_bShowFilteredVoxel = true;
	
	m_fTrackingAngle = 0.0f;

	m_fViewingDistance = 10.0f;
	m_fFovy = 30.0f;
	m_fNearFactor = 0.1f; 
	m_fFarFactor = 10.0f;

	m_fZoomScale = m_fViewingDistance * m_fNearFactor 
					* (float)tan( m_fFovy * PI / 360.0 );
	m_fPanOffsetX = 0.0f;
	m_fPanOffsetY = 0.0f;

	m_vPrevVec[0] = m_vPrevVec[1] = m_vPrevVec[2] = 0.f;
	m_vCurVec[0] = m_vCurVec[1] = m_vCurVec[2] = 0.f;

	m_vAxis[0] = m_vAxis[1] = m_vAxis[2] = 0.f;

	// default view transform
	m_mxTransform[0][0] = 0.544677f; m_mxTransform[0][1] = -0.362249f;
	m_mxTransform[0][2] = 0.756375f; m_mxTransform[0][3] = 0.f;
    m_mxTransform[1][0] = 0.838644f; m_mxTransform[1][1] = 0.233635f;
	m_mxTransform[1][2] = -0.492026f; m_mxTransform[1][3] = 0.f;
    m_mxTransform[2][0] = 0.001520f; m_mxTransform[2][1] = 0.902324f;
	m_mxTransform[2][2] = 0.431054f; m_mxTransform[2][3] = 0.f;
    m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	// signal slot
	connect( this, SIGNAL( imageChanged( bool ) ), this, SLOT( slotImageChanged( bool ) ) );
} // CGLView

CGLView::~CGLView()
{
//	glDeleteLists( m_nObject, m_nListCount );
} // ~CGLView

bool CGLView::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	// initialize single color
	m_rgbSingleColor.rgbRed = 127;
	m_rgbSingleColor.rgbGreen = 127;
	m_rgbSingleColor.rgbBlue = 127;

	// initialize index color
	if ( m_pVolume->isWhiteIsBone() )
		initIndexColor1();
	else
		initIndexColor2();

	double fX, fY, fZ;
	fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

	float fTmp = (float) sqrt( fX*fX + fY*fY + fZ*fZ );
	setViewingDistance( fTmp * 2.2f );

	// m_nObject = drawVolume();		// Generate an OpenGL display list

	return true;
} // init

void CGLView::setVolume( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );
} // setVolume

void CGLView::updateView( bool bUpdate )
{
	m_Pixmap = renderPixmap( width(), height() );

	if ( bUpdate )
		repaint( false );
	/*
	else
		drawTitle();
	*/
} // updateView

// private
void CGLView::setupViewingFrustum()
{
	double aspectRatio = (double)m_vViewport[3] / (double)m_vViewport[2];
	float nearPlane = m_fNearFactor * m_fViewingDistance;
	float farPlane = m_fFarFactor * m_fViewingDistance;

	float left = -(float)(m_fZoomScale / aspectRatio) + m_fPanOffsetX;
	float right = (float)(m_fZoomScale / aspectRatio) + m_fPanOffsetX;
	float bottom = -m_fZoomScale + m_fPanOffsetY;
	float top = m_fZoomScale + m_fPanOffsetY;

	glFrustum( left, right, bottom, top, nearPlane, farPlane );
} // setupViewingFrustum

// private
void CGLView::setupViewingTransform()
{
	glClear( GL_COLOR_BUFFER_BIT );
	glLoadIdentity();
	glTranslatef( 0.0f, 0.0f, -m_fViewingDistance );
	if ( m_eManipulationMode == TRACKBALL && m_bIsTracking == true )
	{
		glPushMatrix();
		glLoadIdentity();
		glRotatef( m_fTrackingAngle, m_vAxis[0], m_vAxis[1], m_vAxis[2] );
		glMultMatrixf( (GLfloat *)m_mxTransform );
		glGetFloatv( GL_MODELVIEW_MATRIX, (GLfloat *)m_mxTransform ) ;
		glPopMatrix();
	}
	glMultMatrixf( (GLfloat *)m_mxTransform );
} // setupViewingTransform

// private
void CGLView::setViewingDistance( float dist )
{
	m_fViewingDistance = dist;
	m_fZoomScale = m_fViewingDistance * m_fNearFactor 
					* (float)tan(m_fFovy * PI / 360.0);
} // setViewingDistance

// private
void CGLView::setupOpenGLState()
{	
	float fX, fY, fZ;
	fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

	float dist = fX + fY + fZ;

	GLfloat ambientLight[]  = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat diffuseLight[]  = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat specularLight[] = { 0.7f, 0.7f, 0.7f, 1.0f};
	GLfloat specref[]       = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLint   shininess = 128;
	GLfloat	lightPos[4];	// = { 0.0f, 0.0f, 0.0f, 1.0f };

	lightPos[0] = 0.0f;
	lightPos[1] = 0.0f;
	lightPos[2] = dist;
	lightPos[3] = 1.0f;
		
	glLightfv( GL_LIGHT0, GL_AMBIENT, ambientLight );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuseLight );
	glLightfv( GL_LIGHT0, GL_SPECULAR, specularLight );
	glLightfv( GL_LIGHT0, GL_POSITION, lightPos );
	
	glEnable( GL_LIGHT0 );

	// enable drawing features that we need...
	glEnable( GL_DEPTH_TEST );	// hidden surface removal
	glFrontFace( GL_CCW );		// counter clock-wise polygons face out
	glEnable( GL_CULL_FACE );	// do not calculate inside

	// enable lighting
	glEnable( GL_LIGHTING );

	// setup and enable light 0
	glLightfv( GL_LIGHT0, GL_AMBIENT, ambientLight );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuseLight );
	glLightfv( GL_LIGHT0, GL_SPECULAR, specularLight );
	glEnable( GL_LIGHT0 );

	// enable color tracking
	glEnable( GL_COLOR_MATERIAL );
	
	// set material properties to follow glColor values
	glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );

	// all materials hereafter have full specular reflectivity
	// with a high shine
	glMaterialfv( GL_FRONT, GL_SPECULAR, specref );
	glMateriali( GL_FRONT, GL_SHININESS, shininess );

	// light black background
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	// glClearColor( m_vBackground[0], m_vBackground[1], m_vBackground[2], m_vBackground[3] );

	glShadeModel( GL_SMOOTH );
//	glShadeModel( GL_FLAT );
} // setupHeadLight

// private
void CGLView::renderScene()
{
	float fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	float fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	float fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

	float fSliceSize = (float) sqrt( fX*fX + fY*fY );

	if ( m_bShowGridLine )
		drawWorkspace( 20, fSliceSize / 20.0f, -fZ/2 );

	glTranslatef( -fX/2, -fY/2, -fZ/2 );

	if ( m_bShowBoundingBox )
		drawBoundingBox();

	if ( !m_bIsTracking )
	{
		setCursor( WaitCursor );
		drawVolume();
		setCursor( ArrowCursor );
	}
} // renderScene

// private
void CGLView::drawVolume()
{
	DWORD total = m_pVolume->totalNumVoxel();
	float fDx, fDy, fDz;
//	float fX, fY, fZ;
	float fPoint1[3], fPoint2[3];
	POSITION_THREED posPosition;
//	WORD wVoxelValue;
	BYTE byteFlag;
	BYTE byteVisible;
	RGB rgbRgb;
	
	fDx = m_pVolume->voxelWidth();
	fDy = m_pVolume->voxelHeight();
	fDz = m_pVolume->voxelDepth();

	/*
	// generate OpenGL display list
	GLuint list;
	list = glGenLists( 1 );
	glNewList( list, GL_COMPILE );
	*/

	/*
	fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

	glTranslatef( -fX/2, -fY/2, -fZ/2 );
	*/

// QTime time;

//	byteVisible = checkVisibleSurface();
	byteVisible = 0x3f;

	for ( DWORD i=1 ; i<=total ; i++ )
	{
	//	wVoxelValue = m_pVolume->voxelValue( i );
		byteFlag = m_pVolume->voxelFlag( i );

		if ( ( byteFlag & 0x40 ) == 0x40 )			// if visible
		{
			if ( ( byteFlag & 0x80 ) == 0x80 )		// filtered voxel
			{
				if ( !m_bShowFilteredVoxel )
					goto _NEXT_RET;

				if ( m_bIndexColor )
					glColor3ub( 255, 255, 255 );	// white
				else
					glColor3ub( 255, 85, 0 );		// orange
			}
			else
			{
				rgbRgb = getRenderingColor( i-1 );
				glColor3ub( rgbRgb.rgbRed, rgbRgb.rgbGreen, rgbRgb.rgbBlue );
			}

/*
time = QTime::currentTime();
qDebug( "1 : %d %d %d %d", time.hour(), time.minute(), time.second(), time.msec() );
*/

			if ( !m_pVolume->voxelPosition( i, posPosition ) )
				goto _NEXT_RET;

/*
time = QTime::currentTime();
qDebug( "2 : %d %d %d %d", time.hour(), time.minute(), time.second(), time.msec() );
*/
			
			fPoint1[0] = fDx * (posPosition.x-1);
			fPoint1[1] = fDy * (posPosition.y-1);
			fPoint1[2] = fDz * (posPosition.z-1);

			fPoint2[0] = fDx * posPosition.x;
			fPoint2[1] = fDy * posPosition.y;
			fPoint2[2] = fDz * posPosition.z;

			glBegin( GL_QUADS );

			if ( (byteFlag & SURFACE1) == SURFACE1 &&
				 (byteVisible & SURFACE1) == SURFACE1 )
			{
				glNormal3f( 0, 0, -1 );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint1[2] );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint1[2] );
			}

			if ( (byteFlag & SURFACE2) == SURFACE2 &&
				 (byteVisible & SURFACE2) == SURFACE2 )
			{
				glNormal3f( 0, 0, 1 );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint2[2] );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint2[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint2[2] );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint2[2] );
			}

			if ( (byteFlag & SURFACE3) == SURFACE3 &&
				 (byteVisible & SURFACE3) == SURFACE3 )
			{
				glNormal3f( 0, -1, 0 );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint2[2] );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint2[2] );
			}

			if ( (byteFlag & SURFACE4) == SURFACE4 &&
				 (byteVisible & SURFACE4) == SURFACE4 )
			{
				glNormal3f( 0, 1, 0 );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint1[2] );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint2[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint2[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint1[2] );
			}

			if ( (byteFlag & SURFACE5) == SURFACE5 &&
				 (byteVisible & SURFACE5) == SURFACE5 )
			{
				glNormal3f( -1, 0, 0 );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint1[2] );
				glVertex3f( fPoint2[0], fPoint2[1], fPoint2[2] );
				glVertex3f( fPoint2[0], fPoint1[1], fPoint2[2] );
			}

			if ( (byteFlag & SURFACE6) == SURFACE6 &&
				 (byteVisible & SURFACE6) == SURFACE6 )
			{
				glNormal3f( 1, 0, 0 );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint1[2] );
				glVertex3f( fPoint1[0], fPoint1[1], fPoint2[2] );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint2[2] );
				glVertex3f( fPoint1[0], fPoint2[1], fPoint1[2] );
			}

			glEnd();

_NEXT_RET:;

/*
time = QTime::currentTime();
qDebug( "3 : %d %d %d %d", time.hour(), time.minute(), time.second(), time.msec() );
*/
		} // if
	} // for
} // drawVolume

// private
BYTE CGLView::checkVisibleSurface()
{
	BYTE byteVisible = 0;
	float x, y, z;
	float matrix[4][4];
	float f[6][3] = {
		{  0,  0, -1 },
		{  0,  0,  1 },
		{  0, -1,  0 },
		{  0,  1,  0 },
		{ -1,  0,  0 },
		{  1,  0,  0 }
	};

//	glPushMatrix();

	// get current model view matrix
	glGetFloatv( GL_MODELVIEW_MATRIX, (GLfloat *)matrix );

qDebug( "-start" );

	// check visible surface for current model view matrix
	x = f[0][0] * matrix[0][0] + f[0][1] * matrix[0][1] + f[0][2] * matrix[0][2];
	y = f[0][0] * matrix[1][0] + f[0][1] * matrix[1][1] + f[0][2] * matrix[1][2];
	z = f[0][0] * matrix[2][0] + f[0][1] * matrix[2][1] + f[0][2] * matrix[2][2];

	if ( z < 0 )
		byteVisible |= SURFACE1;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

	x = f[1][0] * matrix[0][0] + f[1][1] * matrix[0][1] + f[1][2] * matrix[0][2];
	y = f[1][0] * matrix[1][0] + f[1][1] * matrix[1][1] + f[1][2] * matrix[1][2];
	z = f[1][0] * matrix[2][0] + f[1][1] * matrix[2][1] + f[1][2] * matrix[2][2];

	if ( z > 0 )
		byteVisible |= SURFACE2;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

	x = f[2][0] * matrix[0][0] + f[2][1] * matrix[0][1] + f[2][2] * matrix[0][2];
	y = f[2][0] * matrix[1][0] + f[2][1] * matrix[1][1] + f[2][2] * matrix[1][2];
	z = f[2][0] * matrix[2][0] + f[2][1] * matrix[2][1] + f[2][2] * matrix[2][2];

	if ( y < 0 )
		byteVisible |= SURFACE3;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

	x = f[3][0] * matrix[0][0] + f[3][1] * matrix[0][1] + f[3][2] * matrix[0][2];
	y = f[3][0] * matrix[1][0] + f[3][1] * matrix[1][1] + f[3][2] * matrix[1][2];
	z = f[3][0] * matrix[2][0] + f[3][1] * matrix[2][1] + f[3][2] * matrix[2][2];

	if ( y > 0 )
		byteVisible |= SURFACE4;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

	x = f[4][0] * matrix[0][0] + f[4][1] * matrix[0][1] + f[4][2] * matrix[0][2];
	y = f[4][0] * matrix[1][0] + f[4][1] * matrix[1][1] + f[4][2] * matrix[1][2];
	z = f[4][0] * matrix[2][0] + f[4][1] * matrix[2][1] + f[4][2] * matrix[2][2];

	if ( x < 0 )
		byteVisible |= SURFACE5;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

	x = f[5][0] * matrix[0][0] + f[5][1] * matrix[0][1] + f[5][2] * matrix[0][2];
	y = f[5][0] * matrix[1][0] + f[5][1] * matrix[1][1] + f[5][2] * matrix[1][2];
	z = f[5][0] * matrix[2][0] + f[5][1] * matrix[2][1] + f[5][2] * matrix[2][2];

	if ( x > 0 )
		byteVisible |= SURFACE6;

qDebug( " %15.5f  %15.5f  %15.5f", x, y, z );

qDebug( "-end" );

//	glPophMatrix();

	return byteVisible;
} // checkVisibleSurface


// private
void CGLView::drawBoundingBox()
{	
	float fX, fY, fZ;
	
	fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

//	glTranslatef( -fX/2, -fY/2, -fZ/2 );

	glColor3f( 0.0, 1.0, 0.0 );

//	glLineWidth( 2 );
	glBegin( GL_LINES );
		// bottom quadrangle
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( fX, 0.0, 0.0 );

		glVertex3f( fX, 0.0, 0.0 );
		glVertex3f( fX, fY, 0.0 );

		glVertex3f( fX, fY, 0.0 );
		glVertex3f( 0.0, fY, 0.0 );

		glVertex3f( 0.0, fY, 0.0 );
		glVertex3f( 0.0, 0.0, 0.0 );

		// upper quadrangle
		glVertex3f( 0.0, 0.0, fZ );
		glVertex3f( fX, 0.0, fZ );

		glVertex3f( fX, 0.0, fZ );
		glVertex3f( fX, fY, fZ );

		glVertex3f( fX, fY, fZ );
		glVertex3f( 0.0, fY, fZ );

		glVertex3f( 0.0, fY, fZ );
		glVertex3f( 0.0, 0.0, fZ );

		// side edges
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( 0.0, 0.0, fZ );

		glVertex3f( fX, 0.0, 0.0 );
		glVertex3f( fX, 0.0, fZ );

		glVertex3f( fX, fY, 0.0 );
		glVertex3f( fX, fY, fZ );

		glVertex3f( 0.0, fY, 0.0 );
		glVertex3f( 0.0, fY, fZ );
	glEnd();

//	glLineWidth( 1 );
} // drawBoundingBox

// private
void CGLView::drawWorkspace( int gridNum, float gridSpace, float fZ )
{
	float gridColor[4];
	gridColor[0] = 1.0f; 
	gridColor[1] = 0.0f; 
	gridColor[2] = 0.0f; 
	gridColor[3] = 1.0f;
	
	int halfNum = gridNum / 2;
	float width = gridSpace * halfNum;
	
	glColor3fv( gridColor );
	glBegin( GL_LINES );
	for( int i=-halfNum ; i<=halfNum ; i++ ) 
	{
		glVertex3f( -width, i * gridSpace, fZ );
		glVertex3f( width, i * gridSpace, fZ );
		glVertex3f( i * gridSpace, -width, fZ );
		glVertex3f( i * gridSpace, width, fZ );
	}
	glEnd();
} // drawWorkspace

// private
void CGLView::drawTitle()
{
	QPen cyanPen( QColor( 0, 255, 255 ), 2 );

	QPainter p;
	QFontMetrics fm( p.font() );
	int height = fm.height();
	p.begin( &m_Pixmap );
		// draw text to pixmap
		if ( m_bShowTitle )
		{
			p.setPen( cyanPen );
#ifdef _DEMO_VERSION_
			QString str;
			str = PROGRAM_NAME;
			str += " ";
			str += PROGRAM_VERSION;
			p.drawText( 5, height+5, str );
			p.drawText( 5, height*2+10, PROGRAM_COPYRIGHT );
			p.drawText( 5, height*3+15, PROGRAM_EMAIL );
#else
			p.drawText( 5, height+5, "Finite Element Model" );
#endif // _DEMO_VERSION_
		}
		if ( m_bIndexColor )
		{
			int w = fm.width( "C" );
			p.setPen( QPen( QColor( 255, 255, 255 ) ) );
			p.drawRect( width()-w*8, 5, w*3, height );
			p.drawRect( width()-w*8, height+10, w*3, height );
			p.drawRect( width()-w*8, height*2+15, w*3, height );
			QString str;
			str.sprintf( "%d", m_nIndexRange[0] );
			p.drawText( width()-w*4, height+5, str );
			str.sprintf( "%d", (int) ((m_nIndexRange[0]+m_nIndexRange[1]) / 2.0) );
			p.drawText( width()-w*4, height*2+10, str );
			str.sprintf( "%d", m_nIndexRange[1] );
			p.drawText( width()-w*4, height*3+15, str );

			QBrush redBrush( QColor( 0, 0, 255 ) );
			p.fillRect( width()-w*8+1, 6, w*3-2, height-2, redBrush );
			QBrush greenBrush( QColor( 0, 255, 0 ) );
			p.fillRect( width()-w*8+1, height+11, w*3-2, height-2, greenBrush );
			QBrush blueBrush( QColor( 255, 0, 0 ) );
			p.fillRect( width()-w*8+1, height*2+16, w*3-2, height-2, blueBrush );
		}
	p.end();
}

// slot
void CGLView::slotImageChanged( bool flag )
{
	if ( m_pVolume )
	{
		if ( flag )
		{
			m_Pixmap = renderPixmap( width(), height() );
			repaint( false );
		}
		else
			updateGL();
	}
} // slotImageChanged

// slot
void CGLView::slotZoomAll( bool bDefault ) 
{
	if ( !bDefault && m_bZoomAll )
		return;
	if ( bDefault && m_bDefaultView )
		return;

	float fTmp;

	float fX, fY, fZ;
	
	fX = m_pVolume->voxelWidth() * m_pVolume->volumeWidth();
	fY = m_pVolume->voxelHeight() * m_pVolume->volumeHeight();
	fZ = m_pVolume->voxelDepth() * m_pVolume->volumeDepth();

	fTmp = (float) sqrt( fX*fX + fY*fY + fZ*fZ );

	m_fPanOffsetX = m_fPanOffsetY = 0.0f;

	// set viewing distance
	setViewingDistance( fTmp*2.2f );

	// set model matrix to default view
	if ( bDefault )
	{
		m_mxTransform[0][0] = 0.544677f; m_mxTransform[0][1] = -0.362249f;
		m_mxTransform[0][2] = 0.756375f; m_mxTransform[0][3] = 0.f;
		m_mxTransform[1][0] = 0.838644f; m_mxTransform[1][1] = 0.233635f;
		m_mxTransform[1][2] = -0.492026f; m_mxTransform[1][3] = 0.f;
		m_mxTransform[2][0] = 0.001520f; m_mxTransform[2][1] = 0.902324f;
		m_mxTransform[2][2] = 0.431054f; m_mxTransform[2][3] = 0.f;
		m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
		m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

		m_bDefaultView = true;
	}

	m_bZoomAll = true;

	updateView();
} // slotZoomAll

// slot
void CGLView::slotTopView()
{
	m_mxTransform[0][0] = 1.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = 0.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 1.0f;
	m_mxTransform[1][2] = 0.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = 0.0f; m_mxTransform[2][1] = 0.0f;
	m_mxTransform[2][2] = 1.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotTopView

void CGLView::slotBottomView()
{
	m_mxTransform[0][0] = -1.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = 0.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 1.0f;
	m_mxTransform[1][2] = 0.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = 0.0f; m_mxTransform[2][1] = 0.0f;
	m_mxTransform[2][2] = -1.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotBottomView

void CGLView::slotLeftView()
{
	m_mxTransform[0][0] = 0.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = -1.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 1.0f;
	m_mxTransform[1][2] = 0.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = 1.0f; m_mxTransform[2][1] = 0.0f;
	m_mxTransform[2][2] = 0.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotLeftView

void CGLView::slotRightView()
{
	m_mxTransform[0][0] = 0.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = 1.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 1.0f;
	m_mxTransform[1][2] = 0.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = -1.0f; m_mxTransform[2][1] = 0.0f;
	m_mxTransform[2][2] = 0.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotRightView

void CGLView::slotFrontView()
{
	m_mxTransform[0][0] = 1.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = 0.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 0.0f;
	m_mxTransform[1][2] = -1.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = 0.0f; m_mxTransform[2][1] = 1.0f;
	m_mxTransform[2][2] = 0.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotFrontView

void CGLView::slotBackView()
{
	m_mxTransform[0][0] = -1.0f; m_mxTransform[0][1] = 0.0f;
	m_mxTransform[0][2] = 0.0f; m_mxTransform[0][3] = 0.0f;

	m_mxTransform[1][0] = 0.0f; m_mxTransform[1][1] = 0.0f;
	m_mxTransform[1][2] = 1.0f; m_mxTransform[1][3] = 0.0f;

	m_mxTransform[2][0] = 0.0f; m_mxTransform[2][1] = 1.0f;
	m_mxTransform[2][2] = 0.0f; m_mxTransform[2][3] = 0.0f;

	m_mxTransform[3][0] = 0.f; m_mxTransform[3][1] = 0.f;
	m_mxTransform[3][2] = 0.f; m_mxTransform[3][3] = 1.f;

	m_bDefaultView = false;

	updateView();
} // slotMinusZView

// slot
void CGLView::slotWhiteToBlackColor()
{
	m_bWhiteBlack = true;
	m_bSingleColor = false;
	m_bIndexColor = false;

	updateView();
} // slotWhiteToBlackColor

// slot
void CGLView::slotBlackToWhiteColor()
{
	m_bWhiteBlack = false;
	m_bSingleColor = false;
	m_bIndexColor = false;

	updateView();
} // slotBlackToWhiteColor

// slot
void CGLView::slotSingleColor()
{
	m_bWhiteBlack = false;
	m_bSingleColor = true;
	m_bIndexColor = false;

	updateView();
} // slotSingleColor

// slot
void CGLView::slotIndexColor()
{
	m_bWhiteBlack = false;
	m_bSingleColor = false;
	m_bIndexColor = true;

	updateView();
} // slotIndexColor

bool CGLView::isTitleShow()
{
	if ( m_bShowTitle )
		return true;
	else
		return false;
} // isTitleShow

bool CGLView::isGridLineShow()
{
	if ( m_bShowGridLine )
		return true;
	else
		return false;
} // isGridLineShow

bool CGLView::isBoundingBoxShow()
{
	if ( m_bShowBoundingBox )
		return true;
	else
		return false;
} // isBoundingBoxShow

bool CGLView::isFilteredShow()
{
	if ( m_bShowFilteredVoxel )
		return true;
	else
		return false;
} // isFilteredShow

void CGLView::setShowFilteredVoxel( bool flag, bool update )
{
	m_bShowFilteredVoxel = flag;

	if ( update )
		updateView();
} // slotShowFilteredVoxel

void CGLView::setShowTitle( bool flag, bool update )
{
	m_bShowTitle = flag;

	if ( update )
		updateView();
} // setShowTitle

void CGLView::setShowGridLine( bool flag, bool update )
{
	m_bShowGridLine = flag;

	if ( update )
		updateView();
} // setShowGridLine

void CGLView::setShowBoundingBox( bool flag, bool update )
{
	m_bShowBoundingBox = flag;

	if ( update )
		updateView();
} // setShowBoundingBox

int CGLView::imagePalette()
{
	if ( m_bSingleColor )
		return SINGLE_COLOR;
	if ( m_bIndexColor )
		return INDEX_COLOR;
	if ( m_bWhiteBlack )
		return WHITE_BLACK_COLOR;
	else
		return BLACK_WHITE_COLOR;
} // imagePalette

// slot
void CGLView::slotSelectSingleColor()
{
	QColor oldColor;
	oldColor.setRgb(
		(int) m_rgbSingleColor.rgbRed,
		(int) m_rgbSingleColor.rgbGreen,
		(int) m_rgbSingleColor.rgbBlue );

	QColor newColor = QColorDialog::getColor( oldColor );

	if ( newColor.isValid() )
	{
		// check tool bar
		g_pMainWindow->m_pSingleColor->setOn( true );
		g_pMainWindow->m_pIndexColor->setOn( false );
		g_pMainWindow->m_pWhiteToBlackColor->setOn( false );
		g_pMainWindow->m_pBlackToWhiteColor->setOn( false );

		m_rgbSingleColor.rgbRed = (BYTE) newColor.red();
		m_rgbSingleColor.rgbGreen = (BYTE) newColor.green();
		m_rgbSingleColor.rgbBlue = (BYTE) newColor.blue();

		slotSingleColor();
		updateView();
	}
} // slotSelectSingleColor

// slot
void CGLView::slotSelectIndexRange()
{
	static bool bInit = true;

	CFEModelIndexRangeDlg dlg( this, NULL, true );
	dlg.init( m_nIndexRange[0], m_nIndexRange[1] );
	if ( dlg.exec() != QDialog::Accepted )
		return;

	// check tool bar
	g_pMainWindow->m_pIndexColor->setOn( true );
	g_pMainWindow->m_pSingleColor->setOn( false );
	g_pMainWindow->m_pWhiteToBlackColor->setOn( false );
	g_pMainWindow->m_pBlackToWhiteColor->setOn( false );

	int nIndexRange[2];
	nIndexRange[0] = dlg.IDC_SPIN_MIN->value();
	nIndexRange[1] = dlg.IDC_SPIN_MAX->value();
	if ( bInit )
	{
		// check tool bar
		g_pMainWindow->m_pIndexColor->setOn( true );
		g_pMainWindow->m_pSingleColor->setOn( false );
		g_pMainWindow->m_pWhiteToBlackColor->setOn( false );
		g_pMainWindow->m_pBlackToWhiteColor->setOn( false );

		m_nIndexRange[0] = nIndexRange[0];
		m_nIndexRange[1] = nIndexRange[1];

		slotIndexColor();
		updateView();

		bInit = false;

		return;
	}

	if ( nIndexRange[0] != m_nIndexRange[0] || nIndexRange[1] != m_nIndexRange[1] )
	{
		m_nIndexRange[0] = nIndexRange[0];
		m_nIndexRange[1] = nIndexRange[1];

		slotIndexColor();
		updateView();
	}
} // slotSelectIndexRange

// virtual
void CGLView::paintGL()
{
	g_pMainWindow->statusBar()->message( "Render Scene..." );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	setupViewingFrustum();

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	setupViewingTransform();
	setupOpenGLState();

	// render view
	glPushMatrix();
	renderScene();
	glPopMatrix();

	g_pMainWindow->statusBar()->message( "Ready" );

	// glCallList( m_nObject );
} // paintGL

// virtual
void CGLView::resizeGL( int w, int h )
{
	m_vViewport[2] = w;
	m_vViewport[3] = h;

	glViewport( 0, 0, (GLint)w, (GLint)h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum( -1.0, 1.0, -1.0, 1.0, 5.0, 15.0 );
	glMatrixMode( GL_MODELVIEW );

	m_bResizeFlag = true;
} // resizeGL

// virtual
void CGLView::paintEvent( QPaintEvent * /*e*/ )
{
	if ( m_bResizeFlag )
	{
		if ( m_Pixmap.width() != width() || m_Pixmap.height() != height() )
			m_Pixmap = renderPixmap( width(), height() );

		drawTitle();

		QPainter p;
		p.begin( this );
			// draw pixmap to screen
			p.drawPixmap( 0, 0, m_Pixmap );
		p.end();

		m_bResizeFlag = false;
		return;
	}

	QPainter p;
	p.begin( this );
		// draw pixmap to screen
		p.drawPixmap( 0, 0, m_Pixmap );
	p.end();
} // paintEvent

// virtual
void CGLView::keyPressEvent( QKeyEvent *e )
{
	int key = e->key();

	if ( key == Qt::Key_Control )
	{
		m_bControlKey = true;
	}
	else if ( key == Qt::Key_A )
	{
		slotZoomAll();
	}
	else if ( key == Qt::Key_D )
	{
		slotZoomAll( true );	// default
	}
	else if ( key == Qt::Key_X )
	{
		if ( e->state() == Qt::ShiftButton )
			slotBackView();
		else
			slotFrontView();

		m_bDefaultView = false;		// not default view any more
	}
	else if ( key == Qt::Key_Y )
	{
		if ( e->state() == Qt::ShiftButton )
			slotRightView();
		else
			slotLeftView();

		m_bDefaultView = false;		// not default view any more
	}
	else if ( key == Qt::Key_Z )
	{
		if ( e->state() == Qt::ShiftButton )
			slotBottomView();
		else
			slotTopView();

		m_bDefaultView = false;		// not default view any more
	}
} // keyPressEvent

// virtual
void CGLView::keyReleaseEvent( QKeyEvent * /*e*/ )
{
	m_bControlKey = false;
} // keyReleaseEvent

// virtual
void CGLView::mousePressEvent( QMouseEvent *e )
{
	m_ptPrev = e->pos();

	pointTo3DVector( m_ptPrev, m_vPrevVec );

	if ( m_bControlKey )
	{
		if ( e->button() == LeftButton )
		{
			QPixmap zoom( zoomCursor_xpm );
			QCursor zoomCursor( zoom );
			setCursor( zoomCursor );

			m_eManipulationMode = ZOOMING;
		}
		else if ( e->button() == RightButton )
		{
			QPixmap rotate( rotateCursor_xpm );
			QCursor rotateCursor( rotate );
			setCursor( rotateCursor );

			m_eManipulationMode = TRACKBALL;
		}
		else
		{
			QPixmap pan( panCursor_xpm );
			QCursor panCursor( pan );
			setCursor( panCursor );

			m_eManipulationMode = PANNING;
		}
	}

	m_bIsTracking = true;

	grabMouse();
} // mousePressEvent

// virtual
void CGLView::mouseReleaseEvent( QMouseEvent * /*e*/ )
{
	m_bIsTracking = false;

	// generate pixmap for display purpose
	if ( m_eManipulationMode != NONE )
	{
		m_eManipulationMode = NONE;

		emit imageChanged( true );
	}

	setCursor( ArrowCursor );

	releaseMouse();

	//debug
	/*
glGetFloatv( GL_MODELVIEW_MATRIX, (GLfloat *)m_mxTransform );
qDebug( "viewing matrix\n" );
qDebug( "  %f %f %f %f\n  %f %f %f %f\n  %f %f %f %f\n  %f %f %f %f\n",
		m_mxTransform[0][0], m_mxTransform[0][1], m_mxTransform[0][2], m_mxTransform[0][3],
		m_mxTransform[1][0], m_mxTransform[1][1], m_mxTransform[1][2], m_mxTransform[1][3],
		m_mxTransform[2][0], m_mxTransform[2][1], m_mxTransform[2][2], m_mxTransform[2][3],
		m_mxTransform[3][0], m_mxTransform[3][1], m_mxTransform[3][2], m_mxTransform[3][3] );
	*/

	/*
	// debug
double aspectRatio = (double)m_vViewport[3] / (double)m_vViewport[2];
float left = -(float)(m_fZoomScale / aspectRatio) + m_fPanOffsetX;
float right = (float)(m_fZoomScale / aspectRatio) + m_fPanOffsetX;
float bottom = -m_fZoomScale + m_fPanOffsetY;
float top = m_fZoomScale + m_fPanOffsetY;
float nearPlane = m_fNearFactor * m_fViewingDistance;
float farPlane = m_fFarFactor * m_fViewingDistance;

qDebug( "viewport
qDebug( "  (%f %f)  (%f %f)  (%f %f)  %f",
		left, right, bottom, top, nearPlane, farPlane, m_fViewingDistance );
	*/
} // mouseReleaseEvent

// virtual
void CGLView::mouseMoveEvent( QMouseEvent *e )
{
	GLfloat dx, dy, dz;
	QPoint point = e->pos();

	pointTo3DVector( point, m_vCurVec );

	if ( m_bControlKey )
	{
		if ( m_eManipulationMode == TRACKBALL )
		{
			dx = m_vCurVec[0] - m_vPrevVec[0];
			dy = m_vCurVec[1] - m_vPrevVec[1];
			dz = m_vCurVec[2] - m_vPrevVec[2];
			m_fTrackingAngle = 45.0f * (float) sqrt( dx*dx + dy*dy + dz*dz );

			m_vAxis[0] = m_vPrevVec[1]*m_vCurVec[2] - m_vPrevVec[2]*m_vCurVec[1];
			m_vAxis[1] = m_vPrevVec[2]*m_vCurVec[0] - m_vPrevVec[0]*m_vCurVec[2];
			m_vAxis[2] = m_vPrevVec[0]*m_vCurVec[1] - m_vPrevVec[1]*m_vCurVec[0];

			m_vPrevVec[0] = m_vCurVec[0];
			m_vPrevVec[1] = m_vCurVec[1];
			m_vPrevVec[2] = m_vCurVec[2];

			m_bDefaultView = false;		// not default view any more
		}
		else if ( m_eManipulationMode == ZOOMING )
		{
			m_fZoomScale *= 1.f + (float)( m_ptPrev.y() - point.y() ) * 2 / m_vViewport[2];
			m_ptPrev = point;

			m_bDefaultView = false;		// not default view any more
			m_bZoomAll = false;			// not zoomed all view any more
		}
		else if( m_eManipulationMode == PANNING )
		{
			m_fPanOffsetX += (float)( m_ptPrev.x() - point.x() ) * m_fZoomScale / m_vViewport[2] * 2;
			m_fPanOffsetY -= (float)( m_ptPrev.y() - point.y() ) * m_fZoomScale / m_vViewport[3] * 2;
			m_ptPrev = point;

			m_bDefaultView = false;		// not default view any more
			m_bZoomAll = false;			// not zoomed all view any more
		}

		emit imageChanged( false );
	}
} // mouseMoveEvent


////////////////////////////////////////////////////////////////////////////
//
// etc
//

// private
void CGLView::pointTo3DVector( QPoint point, GLfloat *pVec )
{
	float dis, a;

	/* project x,y onto a hemi-sphere centered within width, height */
	pVec[0] = ( 2.0f * point.x() - m_vViewport[2] ) / m_vViewport[2];
	pVec[1] = ( m_vViewport[3] - 2.0f * point.y() ) / m_vViewport[3];
	dis = (float) sqrt( pVec[0]*pVec[0] + pVec[1]*pVec[1] );
	pVec[2] = (float) cos( (PI/2.0f) * ( (dis < 1.0f) ? dis : 1.0f ) );
	a = (float) sqrt( pVec[0]*pVec[0] + pVec[1]*pVec[1] + pVec[2]*pVec[2] );
	pVec[0] /= a;
	pVec[1] /= a;
	pVec[2] /= a;
} // pointTo3DVector

// private
RGB CGLView::getRenderingColor( DWORD id )
{
	BYTE voxelColor = m_pVolume->voxelColor( id );
	int nColorIndex;
	RGB rgbRgb;
	int nIndexRange = m_nIndexRange[1] - m_nIndexRange[0];

	if ( m_bIndexColor )
	{
		nColorIndex = 
			(int)( ( (int)voxelColor - m_nIndexRange[0] ) / (float) nIndexRange * 24.0 );

		if ( nColorIndex > 24 )
		{
			rgbRgb.rgbRed = 255;
			rgbRgb.rgbGreen = 255;
			rgbRgb.rgbBlue = 255;
		}
		else if ( nColorIndex < 0 )
		{
			rgbRgb.rgbRed = 127;
			rgbRgb.rgbGreen = 127;
			rgbRgb.rgbBlue = 127;
		}
		else
		{
			rgbRgb.rgbRed = m_rgbIndexColor[nColorIndex].rgbRed;
			rgbRgb.rgbGreen = m_rgbIndexColor[nColorIndex].rgbGreen;
			rgbRgb.rgbBlue = m_rgbIndexColor[nColorIndex].rgbBlue;
		}
	}
	else
	{
		if ( m_bSingleColor )
		{
			rgbRgb.rgbRed = m_rgbSingleColor.rgbRed;
			rgbRgb.rgbGreen = m_rgbSingleColor.rgbGreen;
			rgbRgb.rgbBlue = m_rgbSingleColor.rgbBlue;
		}
		else
		{
			if ( m_bWhiteBlack )
			{
				rgbRgb.rgbRed = voxelColor;
				rgbRgb.rgbGreen = voxelColor;
				rgbRgb.rgbBlue = voxelColor;
			}
			else
			{
				rgbRgb.rgbRed = -1 * ( voxelColor - 255 );
				rgbRgb.rgbGreen = -1 * ( voxelColor - 255 );
				rgbRgb.rgbBlue = -1 * ( voxelColor - 255 );
			}
		}
	}
	
	return rgbRgb;
} // getRenderingColor

// private
// for white color is bone
void CGLView::initIndexColor1()
{
	// initialize index range
	m_nIndexRange[0] = 127;
	m_nIndexRange[1] = 255;

	// set index
	m_rgbIndexColor[24].rgbRed = 255;
	m_rgbIndexColor[24].rgbGreen = 0;
	m_rgbIndexColor[24].rgbBlue = 0;

	m_rgbIndexColor[23].rgbRed = 255;
	m_rgbIndexColor[23].rgbGreen = 43;
	m_rgbIndexColor[23].rgbBlue = 0;

	m_rgbIndexColor[22].rgbRed = 255;
	m_rgbIndexColor[22].rgbGreen = 85;
	m_rgbIndexColor[22].rgbBlue = 0;

	m_rgbIndexColor[21].rgbRed = 255;
	m_rgbIndexColor[21].rgbGreen = 128;
	m_rgbIndexColor[21].rgbBlue = 0;

	m_rgbIndexColor[20].rgbRed = 255;
	m_rgbIndexColor[20].rgbGreen = 170;
	m_rgbIndexColor[20].rgbBlue = 0;

	m_rgbIndexColor[19].rgbRed = 255;
	m_rgbIndexColor[19].rgbGreen = 213;
	m_rgbIndexColor[19].rgbBlue = 0;

	m_rgbIndexColor[18].rgbRed = 255;
	m_rgbIndexColor[18].rgbGreen = 255;
	m_rgbIndexColor[18].rgbBlue = 0;

	m_rgbIndexColor[17].rgbRed = 213;
	m_rgbIndexColor[17].rgbGreen = 255;
	m_rgbIndexColor[17].rgbBlue = 0;

	m_rgbIndexColor[16].rgbRed = 170;
	m_rgbIndexColor[16].rgbGreen = 255;
	m_rgbIndexColor[16].rgbBlue = 0;

	m_rgbIndexColor[15].rgbRed = 128;
	m_rgbIndexColor[15].rgbGreen = 255;
	m_rgbIndexColor[15].rgbBlue = 0;

	m_rgbIndexColor[14].rgbRed = 85;
	m_rgbIndexColor[14].rgbGreen = 255;
	m_rgbIndexColor[14].rgbBlue = 0;

	m_rgbIndexColor[13].rgbRed = 43;
	m_rgbIndexColor[13].rgbGreen = 255;
	m_rgbIndexColor[13].rgbBlue = 0;

	m_rgbIndexColor[12].rgbRed = 0;
	m_rgbIndexColor[12].rgbGreen = 255;
	m_rgbIndexColor[12].rgbBlue = 0;

	m_rgbIndexColor[11].rgbRed = 0;
	m_rgbIndexColor[11].rgbGreen = 255;
	m_rgbIndexColor[11].rgbBlue = 43;

	m_rgbIndexColor[10].rgbRed = 0;
	m_rgbIndexColor[10].rgbGreen = 255;
	m_rgbIndexColor[10].rgbBlue = 85;

	m_rgbIndexColor[9].rgbRed = 0;
	m_rgbIndexColor[9].rgbGreen = 255;
	m_rgbIndexColor[9].rgbBlue = 128;

	m_rgbIndexColor[8].rgbRed = 0;
	m_rgbIndexColor[8].rgbGreen = 255;
	m_rgbIndexColor[8].rgbBlue = 170;

	m_rgbIndexColor[7].rgbRed = 0;
	m_rgbIndexColor[7].rgbGreen = 255;
	m_rgbIndexColor[7].rgbBlue = 213;

	m_rgbIndexColor[6].rgbRed = 0;
	m_rgbIndexColor[6].rgbGreen = 255;
	m_rgbIndexColor[6].rgbBlue = 255;

	m_rgbIndexColor[5].rgbRed = 0;
	m_rgbIndexColor[5].rgbGreen = 213;
	m_rgbIndexColor[5].rgbBlue = 255;

	m_rgbIndexColor[4].rgbRed = 0;
	m_rgbIndexColor[4].rgbGreen = 170;
	m_rgbIndexColor[4].rgbBlue = 255;

	m_rgbIndexColor[3].rgbRed = 0;
	m_rgbIndexColor[3].rgbGreen = 128;
	m_rgbIndexColor[3].rgbBlue = 255;

	m_rgbIndexColor[2].rgbRed = 0;
	m_rgbIndexColor[2].rgbGreen = 85;
	m_rgbIndexColor[2].rgbBlue = 255;

	m_rgbIndexColor[1].rgbRed = 0;
	m_rgbIndexColor[1].rgbGreen = 43;
	m_rgbIndexColor[1].rgbBlue = 255;

	m_rgbIndexColor[0].rgbRed = 0;
	m_rgbIndexColor[0].rgbGreen = 0;
	m_rgbIndexColor[0].rgbBlue = 255;
} // initIndexColor1

// private
// for black color is bone
void CGLView::initIndexColor2()
{
	// initialize index range
	m_nIndexRange[0] = 0;
	m_nIndexRange[1] = 127;

	// set index
	m_rgbIndexColor[0].rgbRed = 255;
	m_rgbIndexColor[0].rgbGreen = 0;
	m_rgbIndexColor[0].rgbBlue = 0;

	m_rgbIndexColor[1].rgbRed = 255;
	m_rgbIndexColor[1].rgbGreen = 43;
	m_rgbIndexColor[1].rgbBlue = 0;

	m_rgbIndexColor[2].rgbRed = 255;
	m_rgbIndexColor[2].rgbGreen = 85;
	m_rgbIndexColor[2].rgbBlue = 0;

	m_rgbIndexColor[3].rgbRed = 255;
	m_rgbIndexColor[3].rgbGreen = 128;
	m_rgbIndexColor[3].rgbBlue = 0;

	m_rgbIndexColor[4].rgbRed = 255;
	m_rgbIndexColor[4].rgbGreen = 170;
	m_rgbIndexColor[4].rgbBlue = 0;

	m_rgbIndexColor[5].rgbRed = 255;
	m_rgbIndexColor[5].rgbGreen = 213;
	m_rgbIndexColor[5].rgbBlue = 0;

	m_rgbIndexColor[6].rgbRed = 255;
	m_rgbIndexColor[6].rgbGreen = 255;
	m_rgbIndexColor[6].rgbBlue = 0;

	m_rgbIndexColor[7].rgbRed = 213;
	m_rgbIndexColor[7].rgbGreen = 255;
	m_rgbIndexColor[7].rgbBlue = 0;

	m_rgbIndexColor[8].rgbRed = 170;
	m_rgbIndexColor[8].rgbGreen = 255;
	m_rgbIndexColor[8].rgbBlue = 0;

	m_rgbIndexColor[9].rgbRed = 128;
	m_rgbIndexColor[9].rgbGreen = 255;
	m_rgbIndexColor[9].rgbBlue = 0;

	m_rgbIndexColor[10].rgbRed = 85;
	m_rgbIndexColor[10].rgbGreen = 255;
	m_rgbIndexColor[10].rgbBlue = 0;

	m_rgbIndexColor[11].rgbRed = 43;
	m_rgbIndexColor[11].rgbGreen = 255;
	m_rgbIndexColor[11].rgbBlue = 0;

	m_rgbIndexColor[12].rgbRed = 0;
	m_rgbIndexColor[12].rgbGreen = 255;
	m_rgbIndexColor[12].rgbBlue = 0;

	m_rgbIndexColor[13].rgbRed = 0;
	m_rgbIndexColor[13].rgbGreen = 255;
	m_rgbIndexColor[13].rgbBlue = 43;

	m_rgbIndexColor[14].rgbRed = 0;
	m_rgbIndexColor[14].rgbGreen = 255;
	m_rgbIndexColor[14].rgbBlue = 85;

	m_rgbIndexColor[15].rgbRed = 0;
	m_rgbIndexColor[15].rgbGreen = 255;
	m_rgbIndexColor[15].rgbBlue = 128;

	m_rgbIndexColor[16].rgbRed = 0;
	m_rgbIndexColor[16].rgbGreen = 255;
	m_rgbIndexColor[16].rgbBlue = 170;

	m_rgbIndexColor[17].rgbRed = 0;
	m_rgbIndexColor[17].rgbGreen = 255;
	m_rgbIndexColor[17].rgbBlue = 213;

	m_rgbIndexColor[18].rgbRed = 0;
	m_rgbIndexColor[18].rgbGreen = 255;
	m_rgbIndexColor[18].rgbBlue = 255;

	m_rgbIndexColor[19].rgbRed = 0;
	m_rgbIndexColor[19].rgbGreen = 213;
	m_rgbIndexColor[19].rgbBlue = 255;

	m_rgbIndexColor[20].rgbRed = 0;
	m_rgbIndexColor[20].rgbGreen = 170;
	m_rgbIndexColor[20].rgbBlue = 255;

	m_rgbIndexColor[21].rgbRed = 0;
	m_rgbIndexColor[21].rgbGreen = 128;
	m_rgbIndexColor[21].rgbBlue = 255;

	m_rgbIndexColor[22].rgbRed = 0;
	m_rgbIndexColor[22].rgbGreen = 85;
	m_rgbIndexColor[22].rgbBlue = 255;

	m_rgbIndexColor[23].rgbRed = 0;
	m_rgbIndexColor[23].rgbGreen = 43;
	m_rgbIndexColor[23].rgbBlue = 255;

	m_rgbIndexColor[24].rgbRed = 0;
	m_rgbIndexColor[24].rgbGreen = 0;
	m_rgbIndexColor[24].rgbBlue = 255;
} // initIndexColor2
