#ifndef _MONTAGEVIEW_H_
#define _MONTAGEVIEW_H_

#include "global_define.h"
#include <qwidget.h>
#include <qimage.h>


#define NUM_UNIT_MONTAGE	5
#define UNIT_MONTAGE_SIZE	100


class MontageMDIWindow;
class CMontageViewArea;
class CVolume;
class QPixmap;
class QScrollView;


class CMontageView : public QWidget
{
	Q_OBJECT

public:
	CMontageView( QWidget *parent=0, const char *name=0, WFlags f=0 );
	~CMontageView();
	
	bool init( CVolume *pVolume, int nSlice );

	MontageMDIWindow *montageMDIWindow();
	CVolume *volume();
	CMontageViewArea *montageViewArea();
	
private:
	bool makeImage( QPixmap &pixmap );
	DWORD voxelIndex( WORD x, WORD y, int slice );

signals:
	void viewResized();

public slots:
	void slotSetSlice( int slice );
	void slotTypeChanged( int type );

private slots:
	void slotMoveImageToCenter();

protected:
	virtual void resizeEvent( QResizeEvent * );

private:
	MontageMDIWindow *m_pMontageMDIWindow;
	CVolume *m_pVolume;
	CMontageViewArea *m_pMontageViewArea;

	QScrollView *m_pScrollView;

	int m_nType;	// 1: x-view  2: y-view  3: z-view (default)

	WORD m_nWidth;
	WORD m_nHeight;
	WORD m_nDepth;

	int m_nUnitHeight;
	int m_nSlice;
};

class CMontageViewArea : public QWidget
{
	Q_OBJECT
	
public:
	CMontageViewArea( QWidget *parent=0, const char *name=0, WFlags f=0 );
	~CMontageViewArea();

	bool init( QPixmap pixmap, int unitHeight, int nSlice );

	int currentSlice() { return m_nSlice; }

public slots:
	void slotSetSlice( int slice );

protected:
	virtual void paintEvent( QPaintEvent * );
	virtual void resizeEvent( QResizeEvent * );
	virtual void mousePressEvent( QMouseEvent * );
	virtual void mouseReleaseEvent( QMouseEvent * );
	virtual void mouseMoveEvent( QMouseEvent * );
	virtual void keyPressEvent( QKeyEvent * );

private:
	void repaintMontageView();
	void calculateRowCol( int slice );
	void calculateRowCol( int x, int y );

private:
	CMontageView *m_pMontageView;

	QPixmap m_OriPixmap;
	QPixmap m_Pixmap;

	int m_nUnitHeight;

	int m_nMaxSlice;
	int m_nSlice;
	int m_nRow;
	int m_nCol;
};

#endif // _MONTAGEVIEW_H_
