#include "RenderInfoDlg.h"
#include <qpushbutton.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qslider.h>

CRenderInfoDlg::CRenderInfoDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CRenderInfoDlg_base( parent, name, modal, f )
{
	// voxel opacity
	connect( IDC_SLIDER_OPACITY_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_OPACITY_MIN, SLOT( setValue( int ) ) );
	connect( IDC_SPIN_OPACITY_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_OPACITY_MIN, SLOT( setValue( int ) ) );
	
	connect( IDC_SLIDER_OPACITY_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_OPACITY_MAX, SLOT( setValue( int ) ) );
	connect( IDC_SPIN_OPACITY_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_OPACITY_MAX, SLOT( setValue( int ) ) );

	IDC_SPIN_OPACITY_MIN->setMinValue( 0 );
	IDC_SPIN_OPACITY_MIN->setMaxValue( 100 );
	IDC_SPIN_OPACITY_MAX->setMinValue( 0 );
	IDC_SPIN_OPACITY_MAX->setMaxValue( 100 );

	IDC_SLIDER_OPACITY_MIN->setMinValue( 0 );
	IDC_SLIDER_OPACITY_MIN->setMaxValue( 100 );
	IDC_SLIDER_OPACITY_MAX->setMinValue( 0 );
	IDC_SLIDER_OPACITY_MAX->setMaxValue( 100 );


	connect( IDC_SPIN_OPACITY_MIN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotOpacityMinChanged( int ) ) );
	connect( IDC_SPIN_OPACITY_MAX, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotOpacityMaxChanged( int ) ) );

	// material ambient
	IDC_SPIN_AMBIENT_X->setMinValue( 0 );
	IDC_SPIN_AMBIENT_X->setMaxValue( 100 );

	IDC_SPIN_AMBIENT_Y->setMinValue( 0 );
	IDC_SPIN_AMBIENT_Y->setMaxValue( 100 );

	IDC_SPIN_AMBIENT_Z->setMinValue( 0 );
	IDC_SPIN_AMBIENT_Z->setMaxValue( 100 );

	// material diffuse
	IDC_SPIN_DIFFUSE_X->setMinValue( 0 );
	IDC_SPIN_DIFFUSE_X->setMaxValue( 100 );

	IDC_SPIN_DIFFUSE_Y->setMinValue( 0 );
	IDC_SPIN_DIFFUSE_Y->setMaxValue( 100 );

	IDC_SPIN_DIFFUSE_Z->setMinValue( 0 );
	IDC_SPIN_DIFFUSE_Z->setMaxValue( 100 );

	// material specular
	IDC_SPIN_SPECULAR_X->setMinValue( 0 );
	IDC_SPIN_SPECULAR_X->setMaxValue( 100 );

	IDC_SPIN_SPECULAR_Y->setMinValue( 0 );
	IDC_SPIN_SPECULAR_Y->setMaxValue( 100 );

	IDC_SPIN_SPECULAR_Z->setMinValue( 0 );
	IDC_SPIN_SPECULAR_Z->setMaxValue( 100 );

	// material shinyness
	connect( IDC_SLIDER_SHINYNESS, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_SHINYNESS, SLOT( setValue( int ) ) );
	connect( IDC_SPIN_SHINYNESS, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_SHINYNESS, SLOT( setValue( int ) ) );

	IDC_SPIN_SHINYNESS->setMinValue( 0 );
	IDC_SPIN_SHINYNESS->setMaxValue( 255 );
	IDC_SLIDER_SHINYNESS->setMinValue( 0 );
	IDC_SLIDER_SHINYNESS->setMaxValue( 255 );

	// light color
	IDC_SPIN_LIGHT_COLOR_R->setMinValue( 0 );
	IDC_SPIN_LIGHT_COLOR_R->setMaxValue( 100 );

	IDC_SPIN_LIGHT_COLOR_G->setMinValue( 0 );
	IDC_SPIN_LIGHT_COLOR_G->setMaxValue( 100 );

	IDC_SPIN_LIGHT_COLOR_B->setMinValue( 0 );
	IDC_SPIN_LIGHT_COLOR_B->setMaxValue( 100 );

	// light direction
	IDC_SPIN_LIGHT_DIRECTION_X->setMinValue( -100 );
	IDC_SPIN_LIGHT_DIRECTION_X->setMaxValue( 100 );

	IDC_SPIN_LIGHT_DIRECTION_Y->setMinValue( -100 );
	IDC_SPIN_LIGHT_DIRECTION_Y->setMaxValue( 100 );

	IDC_SPIN_LIGHT_DIRECTION_Z->setMinValue( -100 );
	IDC_SPIN_LIGHT_DIRECTION_Z->setMaxValue( 100 );

	// buttons
	connect( IDC_BTN_OK, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
	connect( IDC_BTN_DEFAULT, SIGNAL( clicked() ), this, SLOT( slotDefault() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
} // CRenderInfoDlg

CRenderInfoDlg::~CRenderInfoDlg()
{
} // ~CRenderInfoDlg

bool CRenderInfoDlg::init( RENDERINFO ri )
{
	// voxel opacity
	IDC_SPIN_OPACITY_MIN->setValue( (int) (ri.voxel_opacity[0] * 100.0) );
	IDC_SPIN_OPACITY_MAX->setValue( (int) (ri.voxel_opacity[1] * 100.0) );

	// material ambient
	IDC_SPIN_AMBIENT_X->setValue( (int) (ri.mat_ambient[0] * 100.0) );
	IDC_SPIN_AMBIENT_Y->setValue( (int) (ri.mat_ambient[1] * 100.0) );
	IDC_SPIN_AMBIENT_Z->setValue( (int) (ri.mat_ambient[2] * 100.0) );

	// material diffuse
	IDC_SPIN_DIFFUSE_X->setValue( (int) (ri.mat_diffuse[0] * 100.0) );
	IDC_SPIN_DIFFUSE_Y->setValue( (int) (ri.mat_diffuse[1] * 100.0) );
	IDC_SPIN_DIFFUSE_Z->setValue( (int) (ri.mat_diffuse[2] * 100.0) );

	// material specular
	IDC_SPIN_SPECULAR_X->setValue( (int) (ri.mat_specular[0] * 100.0) );
	IDC_SPIN_SPECULAR_Y->setValue( (int) (ri.mat_specular[1] * 100.0) );
	IDC_SPIN_SPECULAR_Z->setValue( (int) (ri.mat_specular[2] * 100.0) );

	// material shinyness
	IDC_SPIN_SHINYNESS->setValue( (int) ri.mat_shinyness );

	// light color
	IDC_SPIN_LIGHT_COLOR_R->setValue( (int) (ri.light_color[0] * 100.0) );
	IDC_SPIN_LIGHT_COLOR_G->setValue( (int) (ri.light_color[1] * 100.0) );
	IDC_SPIN_LIGHT_COLOR_B->setValue( (int) (ri.light_color[2] * 100.0) );

	// light direction
	IDC_SPIN_LIGHT_DIRECTION_X->setValue( (int) (ri.light_direction[0] * 100.0) );
	IDC_SPIN_LIGHT_DIRECTION_Y->setValue( (int) (ri.light_direction[1] * 100.0) );
	IDC_SPIN_LIGHT_DIRECTION_Z->setValue( (int) (ri.light_direction[2] * 100.0) );

	m_ri = ri;
 
	return true;
} // init

// slot
void CRenderInfoDlg::slotOpacityMinChanged( int value )
{
	int nMax = IDC_SPIN_OPACITY_MAX->value();

	if ( value >= nMax )
		IDC_SPIN_OPACITY_MAX->setValue( value );
} // slotOpacityMinChanged

// slot
void CRenderInfoDlg::slotOpacityMaxChanged( int value )
{
	int nMin = IDC_SPIN_OPACITY_MIN->value();

	if ( value <= nMin )
		IDC_SPIN_OPACITY_MIN->setValue( value );
} // slotOpacityMaxChanged

// slot
void CRenderInfoDlg::slotOk()
{
	m_ri.voxel_opacity[0] = IDC_SPIN_OPACITY_MIN->value() / 100.0;
	m_ri.voxel_opacity[1] = IDC_SPIN_OPACITY_MAX->value() / 100.0;

	m_ri.mat_ambient[0] = IDC_SPIN_AMBIENT_X->value() / 100.0;
	m_ri.mat_ambient[1] = IDC_SPIN_AMBIENT_Y->value() / 100.0;
	m_ri.mat_ambient[2] = IDC_SPIN_AMBIENT_Z->value() / 100.0;

	m_ri.mat_diffuse[0] = IDC_SPIN_DIFFUSE_X->value() / 100.0;
	m_ri.mat_diffuse[1] = IDC_SPIN_DIFFUSE_Y->value() / 100.0;
	m_ri.mat_diffuse[2] = IDC_SPIN_DIFFUSE_Z->value() / 100.0;

	m_ri.mat_specular[0] = IDC_SPIN_SPECULAR_X->value() / 100.0;
	m_ri.mat_specular[1] = IDC_SPIN_SPECULAR_Y->value() / 100.0;
	m_ri.mat_specular[2] = IDC_SPIN_SPECULAR_Z->value() / 100.0;

	m_ri.mat_shinyness = IDC_SPIN_SHINYNESS->value();

	m_ri.light_color[0] = IDC_SPIN_LIGHT_COLOR_R->value() / 100.0;
	m_ri.light_color[1] = IDC_SPIN_LIGHT_COLOR_G->value() / 100.0;
	m_ri.light_color[2] = IDC_SPIN_LIGHT_COLOR_B->value() / 100.0;

	m_ri.light_direction[0] = IDC_SPIN_LIGHT_DIRECTION_X->value() / 100.0;
	m_ri.light_direction[1] = IDC_SPIN_LIGHT_DIRECTION_Y->value() / 100.0;
	m_ri.light_direction[2] = IDC_SPIN_LIGHT_DIRECTION_Z->value() / 100.0;

	QDialog::accept();
} // slotOky

// slot
void CRenderInfoDlg::slotDefault()
{
	// voxel opacity
	IDC_SPIN_OPACITY_MIN->setValue( 5 );
	IDC_SPIN_OPACITY_MAX->setValue( 95 );

	// material ambient
	IDC_SPIN_AMBIENT_X->setValue( 18 );
	IDC_SPIN_AMBIENT_Y->setValue( 18 );
	IDC_SPIN_AMBIENT_Z->setValue( 18 );

	// material diffuse
	IDC_SPIN_DIFFUSE_X->setValue( 35 );
	IDC_SPIN_DIFFUSE_Y->setValue( 35 );
	IDC_SPIN_DIFFUSE_Z->setValue( 35 );

	// material specular
	IDC_SPIN_SPECULAR_X->setValue( 39 );
	IDC_SPIN_SPECULAR_Y->setValue( 39 );
	IDC_SPIN_SPECULAR_Z->setValue( 39 );

	// material shinyness
	IDC_SPIN_SHINYNESS->setValue( 10 );

	// light color
	IDC_SPIN_LIGHT_COLOR_R->setValue( 100 );
	IDC_SPIN_LIGHT_COLOR_G->setValue( 100 );
	IDC_SPIN_LIGHT_COLOR_B->setValue( 100 );

	// light direction
	IDC_SPIN_LIGHT_DIRECTION_X->setValue( 30 );
	IDC_SPIN_LIGHT_DIRECTION_Y->setValue( 30 );
	IDC_SPIN_LIGHT_DIRECTION_Z->setValue( 100 );
} // slotDefault

// slot
void CRenderInfoDlg::slotCancel()
{
	QDialog::reject();
} // slotCancel
