#include "RenderMDIWindow.h"
#include "RenderView.h"

#include <qimage.h>
#include <qpainter.h>
#include <qpen.h>
#include <qsize.h>
#include <qscrollview.h>

////////////////////////////////////////////////////////////////////////////
//
// CRenderView
//

CRenderView::CRenderView( QWidget *parent, const char *name, WFlags fl )
	: QWidget( parent, name, fl )
{
	m_pRenderMDIWindow = NULL;
	m_pScrollView = NULL;
	m_pRenderViewArea = NULL;

	connect( this, SIGNAL( viewResized() ), 
		this, SLOT( slotMoveImageToCenter() ) );
} // CRenderView

CRenderView::~CRenderView()
{
	if ( m_pRenderViewArea )
	{
		delete m_pRenderViewArea;
		m_pRenderViewArea = NULL;
	}

	if ( m_pScrollView )
	{
		delete m_pScrollView;
		m_pScrollView = NULL;
	}
} // ~CRenderView

void CRenderView::init( RenderMDIWindow *pRenderMDIWindow )
{
	m_pRenderMDIWindow = pRenderMDIWindow;
	ASSERT( m_pRenderMDIWindow );

	// 1. scroll view
	m_pScrollView = new QScrollView( this );
	ASSERT( m_pScrollView );

	// 2. scroll area
	m_pRenderViewArea = new CRenderViewArea( this );
	BYTE *image = m_pRenderMDIWindow->renderedImage();
	m_pRenderViewArea->init( image, m_pRenderMDIWindow->renderedImageSizeRatio() );

	m_pScrollView->addChild( m_pRenderViewArea );
} // init

RenderMDIWindow *CRenderView::renderMDIWindow()
{
	if ( m_pRenderMDIWindow )
		return m_pRenderMDIWindow;
	else
		return NULL;
} // renderMDIWindow

CRenderViewArea *CRenderView::renderViewArea()
{
	if ( m_pRenderViewArea )
		return m_pRenderViewArea;
	else
		return NULL;
} // renderViewArea

QPixmap CRenderView::imagePixmap()
{
	return m_pRenderViewArea->imagePixmap();
} // imagePixmap

// slot
void CRenderView::slotMoveImageToCenter()
{
	QScrollBar *vb = m_pScrollView->verticalScrollBar();
	QScrollBar *hb = m_pScrollView->horizontalScrollBar();

	int x = 0;
	int y = 0;
	int w = m_pScrollView->width();
	int h = m_pScrollView->height();

	if ( !hb->isVisible() && m_pRenderViewArea->width() < w )
		x = (int) ( ( w - m_pRenderViewArea->width() ) / 2 ) - 1;

	if ( !vb->isVisible() && m_pRenderViewArea->height() < h )
		y = (int) ( ( h - m_pRenderViewArea->height() ) / 2 ) - 1;

	m_pScrollView->moveChild( m_pRenderViewArea, x, y );
} // slotMoveImageToCenter

void CRenderView::resizeEvent( QResizeEvent * /*e*/ )
{
	m_pScrollView->setGeometry( 0, 0, width(), height() );

	emit viewResized();
} // resizeEvent


////////////////////////////////////////////////////////////////////////////
//
// CRenderViewArea
//

CRenderViewArea::CRenderViewArea( QWidget *parent, const char *name, WFlags f )
	: QWidget( parent, name, f )
{
	m_pParent = (CRenderView *) parent;
	ASSERT( m_pParent );

	setBackgroundMode( NoBackground );

	setMouseTracking( true );	// enable mouseMoveEvent
	setFocusPolicy( QWidget::StrongFocus );	// enable keyPressEvent
} // CRenderViewArea

CRenderViewArea::~CRenderViewArea()
{
} // ~CRenderViewArea

bool CRenderViewArea::init( BYTE *image, int ratio )
{
	int width, height;

	width = 128 * ratio;
	height = 128 * ratio;

	QImage img( width, height, 32 );

	int t;
	int gray;
	for ( int i=0 ; i<height ; i++ )
	{
		for ( int j=0 ; j<width ; j++ )
		{
			t = i*width + j;
			gray = (int) image[t];
			img.setPixel( i, j, qRgb( gray, gray, gray ) );
		}
	}

	m_Pixmap.convertFromImage( img );

	m_szImage.setWidth( width );
	m_szImage.setHeight( height );

	resize( m_szImage );

	return true;
} // init

QPixmap CRenderViewArea::imagePixmap()
{
	return m_Pixmap;
} // imagePixmap

// virtual
void CRenderViewArea::resizeEvent( QResizeEvent * /*e*/ )
{
	resize( m_szImage );
} // resizeEvent

// virtual
void CRenderViewArea::paintEvent( QPaintEvent * /*e*/ )
{
#ifdef _DEMO_VERSION_
	/*************/
	/* draw text */
	/*************/
	QPen yellowPen( QColor( 255, 255, 0 ), 2 );

	QPainter p;
	QFontMetrics fm( p.font() );
	int height = fm.height();
	p.begin( &m_Pixmap );
		p.setPen( yellowPen );
		QString str;
		str = PROGRAM_NAME;
		str += " ";
		str += PROGRAM_VERSION;
		p.drawText( 5, height+5, str );
		p.drawText( 5, height*2+10, PROGRAM_COPYRIGHT );
		p.drawText( 5, height*3+15, PROGRAM_EMAIL );
	p.end();
#endif // _DEMO_VERSION_

	bitBlt( this, 0, 0, &m_Pixmap );
} // paintEvent
