#include "ResamplingDlg.h"
#include "MDIWindow.h"
#include "VolumeFile.h"
#include <qimage.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qmessagebox.h>

CResamplingDlg::CResamplingDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CResamplingDlg_base( parent, name, modal, f )
{
	m_pMDIWindow = NULL;
	m_pVolume = NULL;

	// file info
	connect( IDC_EDIT_DIRECTORY, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotDirectoryChanged( const QString & ) ) );
	connect( IDC_BTN_DIRECTORY, SIGNAL( clicked() ),
		this, SLOT( slotDirectory() ) );
	connect( IDC_EDIT_KEYWORD, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotKeywordChanged( const QString & ) ) );

	// view type
	connect( IDC_COMBO_VIEWTYPE, SIGNAL( activated( int ) ),
		this, SLOT( slotTypeChanged( int ) ) );

	// start spinbox & slider
	connect( IDC_SPIN_START, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_START, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_START, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_START, SLOT( setValue( int ) ) );

	// end spinbox & slider
	connect( IDC_SPIN_END, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_END, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_END, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_END, SLOT( setValue( int ) ) );

	// ok button & cancel button
	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ),
		this, SLOT( slotCancel() ) );
} // CResamplingDlg

CResamplingDlg::~CResamplingDlg()
{
} // ~CResamplingDlg

bool CResamplingDlg::init( MDIWindow *pMDIWindow )
{
	m_pMDIWindow = pMDIWindow;
	ASSERT( m_pMDIWindow );

	m_pVolume = m_pMDIWindow->volume();
	ASSERT( m_pVolume );

	IDC_EDIT_KEYWORD->setEnabled( false );
	IDC_BTN_OK->setEnabled( false );

	// for file type combobox
	for ( WORD i = 0; i < QImageIO::outputFormats().count(); i++ )
	{
		QString str = QString( QImageIO::outputFormats().at( i ) );
		IDC_COMBO_FILEFORMAT->insertItem( str ); 
	}

	// for view type
	m_nDepth = m_pVolume->volumeDepth();

	IDC_SPIN_START->setRange( 1, m_nDepth );
	IDC_SLIDER_START->setRange( 1, m_nDepth );
	IDC_SPIN_END->setRange( 1, m_nDepth );
	IDC_SLIDER_END->setRange( 1, m_nDepth );

	IDC_SPIN_START->setValue( 1 );
	IDC_SPIN_END->setValue( m_nDepth );

	// connect signal-slot between IDC_SPIN_START and slotMinChanged
	connect( IDC_SPIN_START, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotStartChanged( int ) ) );

	// connect signal-slot between IDC_SPIN_END and slotMaxChanged
	connect( IDC_SPIN_END, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotEndChanged( int ) ) );

	IDC_LBL_DESCRIPTION->setText(
		tr( 
"You can resample and then use [Import Stack] to generate new Volume."
		) );

	return true;
} // init

// slot
void CResamplingDlg::slotDirectory()
{
	// file extension
	QString fn = QFileDialog::getExistingDirectory( NULL, this );
    if ( !fn.isEmpty() )
		IDC_EDIT_DIRECTORY->setText( fn );
} // slotDirectory

// slot
void CResamplingDlg::slotDirectoryChanged( const QString &str )
{
	if ( str.isEmpty() )
	{
		IDC_EDIT_KEYWORD->setText( "" );
		IDC_EDIT_KEYWORD->setEnabled( false );
		IDC_BTN_OK->setEnabled( false );
	}
	else
	{
		IDC_EDIT_KEYWORD->setEnabled( true );
	}

} // slotDirecoryChanged

// slot
void CResamplingDlg::slotKeywordChanged( const QString &str )
{
	if ( str.isEmpty() )
		IDC_BTN_OK->setEnabled( false );
	else
		IDC_BTN_OK->setEnabled( true );
} // slotKeywordChanged

// slot
void CResamplingDlg::slotTypeChanged( int type )
{
	// if current view type is not changed, just return
	if ( type == 0 && m_nType == 3 )
		return;
	if ( type == 1 && m_nType == 1 )
		return;
	if ( type == 2 && m_nType == 2 )
		return;

	// set current view type
	if ( type == 0 )
		m_nType = 3;
	else if ( type == 1 )
		m_nType = 1;
	else if ( type == 2 )
		m_nType = 2;

	// set geometry of virtual volume according to view type
	if ( m_nType == 1 )			// x-view
		m_nDepth  = m_pVolume->volumeWidth();
	else if ( m_nType == 2 )	// y-view
		m_nDepth  = m_pVolume->volumeHeight();
	else if ( m_nType == 3 )	// z-view
		m_nDepth  = m_pVolume->volumeDepth();

	IDC_SPIN_START->setRange( 1, m_nDepth );
	IDC_SLIDER_START->setRange( 1, m_nDepth );
	IDC_SPIN_END->setRange( 1, m_nDepth );
	IDC_SLIDER_END->setRange( 1, m_nDepth );

	IDC_SPIN_START->setValue( 1 );
	IDC_SPIN_END->setValue( m_nDepth );
} // slotChangeType

// slot
void CResamplingDlg::slotStartChanged( int value )
{
	int nEnd = IDC_SPIN_END->value();

	if ( value >= nEnd )
		IDC_SPIN_END->setValue( value );
} // slotStartChanged

// slot
void CResamplingDlg::slotEndChanged( int value )
{
	int nStart = IDC_SPIN_START->value();

	if ( value <= nStart )
		IDC_SPIN_START->setValue( value );
} // slotEndChanged

// slot
void CResamplingDlg::slotOk()
{
	// check directory
	QString strDir = IDC_EDIT_DIRECTORY->text();

	QDir dir( strDir );
	if ( !dir.exists() )
		dir.mkdir( strDir );

	QFileInfo fi( strDir, NULL );
	if ( !fi.isDir() )
	{
		QMessageBox::critical( this, "Error",
			tr( "Can't make directory...\nSelect again..!!" ) );
		IDC_EDIT_DIRECTORY->setText( "" );
		return;
	}

	QDialog::accept();
} // slotOk

// slot
void CResamplingDlg::slotCancel()
{
	QDialog::reject();
} // slotCancel
