#include <math.h>
#include <sys/stat.h>

#include <qapplication.h>
#include <qstring.h>
#include <qdir.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qcolor.h>
#include <qimage.h>
#include <qprogressdialog.h>
#include <qlist.h>
#include <qregion.h>
#include <qstatusbar.h>

#include "define.h"
#include "VolumeFile.h"
#include "DicomFile.h"
#include "MDIWindow.h"
#include "MainWindow.h"
#include "PlanView.h"

#include "MultiLineEditDlg.h"	//XXX
#include <qmultilineedit.h>


// global
extern CMainWindow *g_pMainWindow;


CVolume::CVolume( MDIWindow *mdi )
{
	m_pMDIWindow = mdi;
	ASSERT( m_pMDIWindow );

	// file is closed
	m_bOpenFlag = false;

	// not dicom file
	m_nDicomFlag = false;

	m_dwNumCurrentFeaVoxel = 0;

	m_dwCurrentFeaSurface[0] = 0;
	m_dwCurrentFeaSurface[1] = 0;
	m_dwCurrentFeaSurface[2] = 0;

	m_nNumFilterVoxel = 0;

	m_dwNumCurrentBone = 0;
	m_dwNumFilteredBone = 0;

	for( int i=0 ; i<256 ; i++ )
	{
		m_dwNumVoxel[i] = 0;
		m_dwNumFeaVoxel[i] = 0;

		m_fBoneSurfaceArea[i] = 0.0;
		m_fBoneVolume[i] = 0.0;

		m_dwNumBone[i] = 0;
	}

	/*
	m_vpc = NULL;
	m_pRawVolume = NULL;
	*/

	m_bMeshFiltered = false;

	// contact option
	m_bSurfaceContact = true;
} // CVolume

CVolume::~CVolume()
{
	if( m_bOpenFlag == true )
		close();
} // ~CVolume

MDIWindow *CVolume::mdiWindow()
{
	return m_pMDIWindow;
} // mdiWindow


/************************/
/* file I/O information */
/************************/

void CVolume::close()
{
	if ( m_bOpenFlag == true )
	{
		if ( m_Volume.wDensity )
		{
			delete m_Volume.wDensity;
			m_Volume.wDensity = NULL;
		}
		if ( m_Volume.byteFlag )
		{
			delete m_Volume.byteFlag;
			m_Volume.byteFlag = NULL;
		}
		if ( m_pMinThreshold )
		{
			delete [] m_pMinThreshold;
			m_pMinThreshold = NULL;
		}
		if ( m_pMaxThreshold )
		{
			delete [] m_pMaxThreshold;
			m_pMaxThreshold = NULL;
		}

		m_bOpenFlag = false;
	}
} // close

bool CVolume::open( QString fileName )
{
	// set volume file name
	m_strFileName = fileName;

	// open the volume file
	QFile volumeFile ( m_strFileName );
	if ( !volumeFile.open( IO_ReadOnly ) )	// file open error..!!
	{
		QMessageBox::critical( g_pMainWindow, "File Open Error",
			"Can't open volume file..." );

		g_pMainWindow->checkOutRecentFile( fileName );

		return false;
	}

	// file opened successfully
	QDataStream ds( &volumeFile );	// use a data stream
    
	// voluem version
	ds >> m_Volume.VolumeFileHeader.vfVersion;

	if ( m_Volume.VolumeFileHeader.vfVersion != MGIBA_100 )
	{
		QMessageBox::critical( g_pMainWindow, "Volume File Error",
			"Not supported volume file..." );
		return false;
	}

	// read volume file header into memory
	ds >> m_Volume.VolumeFileHeader.vfType;

	if ( m_Volume.VolumeFileHeader.vfType != ISVOLUME )
	{
		QMessageBox::critical( g_pMainWindow, "Volume File Error",
			"Not Volume File..!!" );
		return false;
	}
	ds >> m_Volume.VolumeFileHeader.vfSize;

	// read volume info header into memory
	ds >> m_Volume.VolumeInfoHeader.viSize;
	ds >> m_Volume.VolumeInfoHeader.viWidth;
	ds >> m_Volume.VolumeInfoHeader.viHeight;
	ds >> m_Volume.VolumeInfoHeader.viDepth;

	// voxel size = element size
	ds >> m_Volume.VolumeInfoHeader.fDx;
	ds >> m_Volume.VolumeInfoHeader.fDy;
	ds >> m_Volume.VolumeInfoHeader.fDz;

	// original voxel size
	ds >> m_Volume.VolumeInfoHeader.fOriginDx;
	ds >> m_Volume.VolumeInfoHeader.fOriginDy;
	ds >> m_Volume.VolumeInfoHeader.fOriginDz;

	DWORD total = m_Volume.VolumeInfoHeader.viWidth
		* m_Volume.VolumeInfoHeader.viHeight
		* m_Volume.VolumeInfoHeader.viDepth;

	// m_Volume.pVoxel = new VOXEL[ total ];
	m_Volume.wDensity = new WORD[ total ];
	m_Volume.byteFlag = new BYTE[ total ];
	// if ( !m_Volume.pVoxel )
	if ( !m_Volume.wDensity || !m_Volume.byteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for VOXEL..!!" );
		return false;
	}

	QProgressDialog progress( "Opening...", "Abort", 100,
                                g_pMainWindow, "progress", true );
	progress.show();

	int percentage = 0;
	for ( DWORD i=0 ; i<total ; i++ )
	{
		if ( i % (DWORD)(total*0.05) == 0 )
		{
			progress.setProgress( percentage );
			qApp->processEvents();
			percentage += 5;
		}

       		if ( progress.wasCancelled() )
		{
			volumeFile.close();
			return false;
		}
        
		ds >> m_Volume.wDensity[i];
		m_Volume.byteFlag[i] = 0;
	}

	m_pMinThreshold = new WORD[volumeDepth()];
	m_pMaxThreshold = new WORD[volumeDepth()];

	for ( WORD j=0 ; j<volumeDepth() ; j++ )
	{
		ds >> m_pMinThreshold[j];
		ds >> m_pMaxThreshold[j];
	}
	progress.setProgress( 100 );
	progress.hide();

	ds >> m_nWhiteIsBone;

	ds >> m_nDicomFlag;

	ds >> m_wMinImageValue;
	ds >> m_wMaxImageValue;
	// end of opening

	// calculate density level
	// calculateNumVoxel();

	volumeFile.close();

	// if file open is successfully done, set m_bOpenFlag to TRUE
	m_bOpenFlag = true;

	return true;
} // open

bool CVolume::init( KEYWORD key )
{
	// 1. set volume info header
	if ( !setVolumeInfoHeader( key ) )
	{
		/*
		QMessageBox::critical( g_pMainWindow, "Volume Info Error",
			"Can't set Volume Info Header..." );

		*/
		return false;
	}

	// 2. set volume structure
	if ( key.strImageFormat == "DICOM" )
	{
		if ( !setVolumeStructFromDicom( key ) )
			return false;
	}
	else
	{
		if ( !setVolumeStructFromImage( key ) )
		/*
		QMessageBox::critical( g_pMainWindow, "Volume Error",
			"Can't generate Volume structure..." );
		*/
		return false;
	}

	// 3. volume file header
	DWORD total = m_Volume.VolumeInfoHeader.viWidth
		* m_Volume.VolumeInfoHeader.viHeight
		* m_Volume.VolumeInfoHeader.viDepth;

	DWORD dwSize;
	dwSize = total * ( sizeof(BYTE) + sizeof(BYTE) );

	m_Volume.VolumeFileHeader.vfSize =
		dwSize + m_Volume.VolumeInfoHeader.viSize + sizeof( VOLUMEFILEHEADER );

	m_Volume.VolumeFileHeader.vfType = 0x4D56;	// 4D:M 56:V => VM

	// if file open is successfully done, set m_bOpenFlag to TRUE
	m_bOpenFlag = true;

	return true;
} // init

bool CVolume::save( QString fileName )
{
	// start file saving

	// 0. open saving file
	m_strFileName = fileName;

	QFile volumeFile( fileName );
    if ( !volumeFile.open( IO_WriteOnly ) )	// file open error..!!
	{
		QMessageBox::critical( g_pMainWindow, "File Open Error",
			"Can't open volume file..." );
		return false;
	}

	// file opened successfully
	QDataStream ds( &volumeFile );	// use a data stream

	// 1. save volume file header to disk
	ds << (Q_UINT16) m_Volume.VolumeFileHeader.vfVersion;
	ds << (Q_UINT16) m_Volume.VolumeFileHeader.vfType;
	ds << (Q_UINT64) m_Volume.VolumeFileHeader.vfSize;

	// 2. save volume info header to disk
	ds << (Q_UINT16) m_Volume.VolumeInfoHeader.viSize;
	ds << (Q_UINT16) m_Volume.VolumeInfoHeader.viWidth;
	ds << (Q_UINT16) m_Volume.VolumeInfoHeader.viHeight;
	ds << (Q_UINT16) m_Volume.VolumeInfoHeader.viDepth;
	DWORD dwTmp;
	// voxel size = element size
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fDx );
	ds << (Q_UINT64) dwTmp;
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fDy );
	ds << (Q_UINT64) dwTmp;
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fDz );
	ds << (Q_UINT64) dwTmp;
	// original voxel size
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fOriginDx );
	ds << (Q_UINT64) dwTmp;
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fOriginDy );
	ds << (Q_UINT64) dwTmp;
	dwTmp = floatToDWORD( m_Volume.VolumeInfoHeader.fOriginDz );
	ds << (Q_UINT64) dwTmp;

	DWORD total = m_Volume.VolumeInfoHeader.viWidth
		* m_Volume.VolumeInfoHeader.viHeight
		* m_Volume.VolumeInfoHeader.viDepth;

	QProgressDialog progress( "Saving...", "Abort", 100,
                                g_pMainWindow, "progress", true );
	progress.show();

	int percentage = 0;
	for ( DWORD i=0 ; i<total ; i++ )
	{
		if ( i % (DWORD)(total*0.05) == 0 )
		{
			progress.setProgress( percentage );
			qApp->processEvents();
			percentage += 5;
		}

        if ( progress.wasCancelled() )
		{
			volumeFile.close();
			volumeFile.remove();
            return false;
		}

		ds << m_Volume.wDensity[i];
	//	ds << m_Volume.byteFlag[i];
	}

	for ( int j=0 ; j<volumeDepth() ; j++ )
	{
		ds << m_pMinThreshold[j];
		ds << m_pMaxThreshold[j];
	}
	progress.setProgress( 100 );
	progress.hide();

	ds << (Q_INT16) m_nWhiteIsBone;

	ds << (Q_INT16) m_nDicomFlag;

	ds << (Q_INT16) m_wMinImageValue;
	ds << (Q_INT16) m_wMaxImageValue;
	// end of writing

	volumeFile.close();

	return true;
} // save

bool CVolume::isOpen()
{
	return m_bOpenFlag;
} // isOpen

bool CVolume::analysisVolume( int start, int end )
{
	/*
	// check elasped time
	clock_t t_start, t_finish;
	float duration;

	// get the start time
	t_start = clock();
	*/

	/*
	// temporary file open : VoxelCount file
	ofstream OutFile, OutFile2;

	char strTmp[256];
	int nLength;
	WORD wTmp = 0x4356;

	nLength = strlen( string );
	strcpy( strTmp, string );
	strncpy( strTmp + (nLength-4), ".vox", 4 );

	OutFile.open( strTmp, ios::out | ios::binary );

	OutFile.write( (const char *) &wTmp, sizeof(WORD) );

	OutFile2.open( "tmp.vox" );
	OutFile2 << wTmp << endl;
	// end temporary file open
	*/

	QFile f( QDir::homeDirPath() + "/.analysis_result.tmp" );

	if ( !f.open( IO_WriteOnly ) )
		return false;

	QTextStream tmpFile( &f );

	//XXX
	CMultiLineEditDlg *pDlg = new CMultiLineEditDlg( g_pMainWindow, NULL, false );
	pDlg->init( "Analysis Volume" );
	pDlg->show();

	QString strMsg;
	float duration = 0.0;

	/*
	// message file open
	ofstream MsgFile;

	strcpy( strTmp, string );
	strncpy( strTmp + (nLength-4), ".msg", 4 );

	MsgFile.open( strTmp, ios::out );

	MsgFile << endl;
	MsgFile << "MGIBA (R) BM-Soft" << endl << endl;
	MsgFile << endl;
	*/
	// end message file open

	// declare variables
	POSITION_THREED posPosi;
	float fThresholdRgb;
	WORD nNumWidth, nNumHeight, nNumDepth, nNumLayer;
	DWORD dwTotalNumVoxel, m_dwNumVoxel, dwNumSurface[3];
	float fSurfaceArea[3];

	CQueue qBone;
	CQueue qSurrVoxel;
	CQueue qTmpVoxel;		// temporary voxel queue
	DWORD dwId1, dwId2;
	WORD nSize;				// surrounding voxel size
	DWORD dwBoneCount, dwVoxelCount;
	DWORD i;

	DWORD dwTmp;	// temporary variable for temporary voxel file

	// calculate voxel surface area
	fSurfaceArea[0] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDy;

	fSurfaceArea[1] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDz;

	fSurfaceArea[2] = m_Volume.VolumeInfoHeader.fDy
		* m_Volume.VolumeInfoHeader.fDz;

	// set VOLUME information & number of layer
	nNumWidth = m_Volume.VolumeInfoHeader.viWidth;
	nNumHeight = m_Volume.VolumeInfoHeader.viHeight;
	nNumDepth = m_Volume.VolumeInfoHeader.viDepth;
	nNumLayer = nNumWidth * nNumHeight;

	// initialize some variables
	m_dwNumVoxel = 0;
	dwNumSurface[0] = 0;
	dwNumSurface[1] = 0;
	dwNumSurface[2] = 0;
	dwBoneCount = 0;

	dwTotalNumVoxel = totalNumVoxel();

	// save old byte flag
	BYTE *pByteFlag = new BYTE[dwTotalNumVoxel];
	if ( !pByteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for Temporary Flag..!!" );

		return false;
	}

	// set temporary flag
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		m_Volume.byteFlag[i-1] &= 0x7f;		// set 8th bit to 0
		pByteFlag[i-1] = m_Volume.byteFlag[i-1];
	}
	
	// write message to screen
	strMsg.sprintf( " *******[ Analysis Volume ]*******\n" );
	pDlg->IDC_MEDIT_REPORT->append( strMsg );

	strMsg.sprintf( " %-8s  %-15s  %-15s  %-15s  %-15s",
		"Level of", "  Bone Surface ", "   Bone Volume ",
		"   Number of   ", "    Elasped    " );
	pDlg->IDC_MEDIT_REPORT->append( strMsg );
		
	strMsg.sprintf( " %-8s  %-15s  %-15s  %-15s  %-15s\n",
		"Density", "  Area [mm^2]  ",  "     [mm^3]    ",
		"      Bone     ", "  Time [sec]   " );
	pDlg->IDC_MEDIT_REPORT->append( strMsg );
	
	/*
	// write message to message file
	MsgFile << " *******[ Analysis Volume ]*******" << endl << endl;

	sprintf( strTmp, " %-8s  %-15s  %-15s  %-15s  %-15s",
		"Level of", "  Bone Surface ", "   Bone Volume ",
		"   Number of   ", "    Elasped    " );

	MsgFile << strTmp << endl;
	
	sprintf( strTmp," %-8s  %-15s  %-15s  %-15s  %-15s",
		"Density", "  Area [mm^2]  ",  "     [mm^3]    ",
		"      Bone     ", "  Time [sec]   " );

	MsgFile << strTmp << endl << endl;
	*/
	
	// analysis volume from density 0 to density 254, 255 is special case..^^
	int nEndRange;
	if ( end == 255 ) nEndRange = end - 1;
	else nEndRange = end;

	/*
	QProgressDialog progress( "Analysis Volume...", "Abort", nEndRange-start,
                                g_pMainWindow, "progress", true );
	progress.show();
	*/

	for ( WORD j=start ; j<=nEndRange ; j++ )
	{
		/*
		progress.setProgress( j-start+1 );
		qApp->processEvents();

		if ( progress.wasCancelled() )
		{
_FUNC_END:
			// initialize
			for ( j=1 ; j<=dwTotalNumVoxel ; j++ )
				m_Volume.byteFlag[j-1] = pByteFlag[j-1];

			if ( pByteFlag )
			{
				delete pByteFlag;
				pByteFlag = NULL;
			}

			return false;
		}
		*/

		fThresholdRgb = j;

		// initialize voxel byteFlag to calculate number of bone
		for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
		{
				m_Volume.byteFlag[i-1] &= 0xbf;

			//	if( m_Volume.wDensity[i-1] <= fThresholdRgb )
				if( voxelColor( i ) <= fThresholdRgb )
					m_Volume.byteFlag[i-1] |= 0x40;
		}

		for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
		{

			/*********************************************/
			/* calculate bone volume & bone surface area */
			/*********************************************/
		//	if ( m_Volume.wDensity[i-1] <= fThresholdRgb )
			if ( voxelColor( i ) <= fThresholdRgb )
			{
				voxelPosition( i, posPosi );

				m_dwNumVoxel++;

				// 1st direction
				if ( posPosi.z == 1 )
					dwNumSurface[0]++;
				else
				{
				//	if( m_Volume.wDensity[i-nNumLayer-1] > fThresholdRgb )
					if( voxelColor( i-nNumLayer ) > fThresholdRgb )
						dwNumSurface[0]++;
				}

				// 2nd direction
				if ( posPosi.z == nNumDepth )
					dwNumSurface[0]++;
				else
				{
				//	if ( m_Volume.wDensity[i+nNumLayer-1] > fThresholdRgb )
					if ( voxelColor( i+nNumLayer ) > fThresholdRgb )
						dwNumSurface[0]++;
				}

				// 3rd direction
				if ( posPosi.y == 1 )
					dwNumSurface[1]++;
				else
				{
				//	if( m_Volume.wDensity[i-nNumWidth-1] > fThresholdRgb )
					if( voxelColor( i-nNumWidth ) > fThresholdRgb )
						dwNumSurface[1]++;
				}

				// 4th direction
				if ( posPosi.y == nNumHeight )
					dwNumSurface[1]++;
				else
				{
				//	if( m_Volume.wDensity[i+nNumWidth-1] > fThresholdRgb )
					if( voxelColor( i+nNumWidth ) > fThresholdRgb )
						dwNumSurface[1]++;
				}

				// 5th direction
				if ( posPosi.x == nNumWidth )
					dwNumSurface[2]++;
				else
				{
				//	if ( m_Volume.wDensity[i+1-1] > fThresholdRgb )
					if ( voxelColor( i+1 ) > fThresholdRgb )
						dwNumSurface[2]++;
				}

				// 6th direction
				if ( posPosi.x == 1 ) dwNumSurface[2]++;
				else
				{
				//	if( m_Volume.wDensity[i-1-1] > fThresholdRgb )
					if( voxelColor( i-1 ) > fThresholdRgb )
						dwNumSurface[2]++;
				}
			} // end if


			/****************************/
			/* calculate number of bone */
			/****************************/

			// initialize number of voxel in bone to 0
			dwVoxelCount = 0;

			if ( (m_Volume.byteFlag[i-1] & 0x40) == 0x40 )
			{
				m_Volume.byteFlag[i-1] &= 0xbf;				// initialize 7th bit to 0

				dwVoxelCount++;

				dwId1 = i;
BEGIN:
				/*
				if ( progress.wasCancelled() )
					goto _FUNC_END;
				*/

				evaluateSurrVoxel( dwId1, qSurrVoxel );
				/*
				if ( !evaluateSurrVoxel( dwId1, qSurrVoxel ) )
					goto _FUNC_END;
					*/
			
				dwVoxelCount += qSurrVoxel.size();
			
				while ( !qSurrVoxel.is_empty() )
				{
					/*
					if ( progress.wasCancelled() )
						goto _FUNC_END;
					*/

					dwId2 = qSurrVoxel.get_front();
					qBone.insert( dwId2 );
					m_Volume.byteFlag[dwId2-1] &= 0xbf;		// initialize 7th bit to 0
				}

				if ( !qBone.is_empty() )
				{
					dwId1 = qBone.get_front();
					goto BEGIN;
				}

				dwBoneCount++;

				qTmpVoxel.insert( dwVoxelCount );
			}
		}	// for DWORD i

		// write voxel count info. to temporary file
		nSize = qTmpVoxel.size();

		tmpFile << j << " " << nSize << endl;
		if( nSize != 0 )
		{
			while ( !qTmpVoxel.is_empty() )
			{
				dwTmp = qTmpVoxel.get_front();
				tmpFile << dwTmp << " ";
			}
			tmpFile << endl;
		}

		// calculate bone volume
		m_fBoneVolume[j] = voxelVolume() * m_dwNumVoxel;

		// calculate bone surface area
		m_fBoneSurfaceArea[j] = 
				  fSurfaceArea[0] * dwNumSurface[0]
				+ fSurfaceArea[1] * dwNumSurface[1]
				+ fSurfaceArea[2] * dwNumSurface[2];

		// assign number of accumulated feasible voxel
		m_dwNumFeaVoxel[j] = m_dwNumVoxel;

		// assign number of bone
		m_dwNumBone[j] = dwBoneCount;

		/*
		// get the elapsed time
		t_finish = clock();
		duration = (float) (t_finish - t_start) / CLOCKS_PER_SEC;
		*/

		// write result to screen
		strMsg.sprintf( " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			j,
			this->m_fBoneSurfaceArea[j],
			this->m_fBoneVolume[j],
			this->m_dwNumBone[j],
			duration );
		pDlg->IDC_MEDIT_REPORT->append( strMsg );
		
		/*
		// write result to message file
		sprintf( strTmp, " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			j,
			this->m_fBoneSurfaceArea[j],
			this->m_fBoneVolume[j],
			this->m_dwNumBone[j],
			duration );

		MsgFile << strTmp << endl;
		*/

		// initialize some variables
		m_dwNumVoxel = 0;
		dwNumSurface[0] = 0;
		dwNumSurface[1] = 0;
		dwNumSurface[2] = 0;

		// initialize number of bone to 0
		dwBoneCount = 0;
	}	// for int j

	if ( end == 255 )
	{
	// for j = 255, i said before, it is special case
		int nIndex = 255;

		dwBoneCount = 1;
		dwTmp = dwTotalNumVoxel;

		tmpFile << nIndex << " " << dwBoneCount << endl;
		tmpFile << dwTmp << endl;

		// calculate bone volume
		m_fBoneVolume[255] = volumeVolume();

		// calculate bone surface area
		m_fBoneSurfaceArea[255] = volumeSurfaceArea();

		// assign number of accumulated feasible voxel
		m_dwNumFeaVoxel[255] = dwTmp;

		// assign number of bone
		m_dwNumBone[255] = 1;

		/*
		// get the elapsed time
		t_finish = clock();
		duration = (float) (t_finish - t_start) / CLOCKS_PER_SEC;
		*/

		// write result to screen
		strMsg.sprintf( " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			nIndex,
			this->m_fBoneSurfaceArea[255],
			this->m_fBoneVolume[255],
			this->m_dwNumBone[255], duration );
		pDlg->IDC_MEDIT_REPORT->append( strMsg );
		
		/*
		// write result to message file
		sprintf( strTmp, " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			nIndex,
			this->m_fBoneSurfaceArea[255],
			this->m_fBoneVolume[255],
			this->m_dwNumBone[255], duration );

		MsgFile << strTmp << endl;
		*/

	// for j = 255, i said before, it is special case
	}

	// initialize
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		if ( ( m_Volume.byteFlag[i-1] & 0x80 ) == 0x80 )
			pByteFlag[i-1] |= 0x80;

		m_Volume.byteFlag[i-1] = pByteFlag[i-1];
	}

	// close temporary file
	f.close();

	tmpFile.unsetDevice();

	// close message file
//	MsgFile.close();

	if ( pDlg )
		delete pDlg;

	return true;
} // analysisVolume

/*********************/ 
/* VOXEL Information */
/*********************/

void CVolume::setVoxelWidth( float width )
{
	m_Volume.VolumeInfoHeader.fDx = width;
} // setVoxelWidth

void CVolume::setVoxelHeight( float height )
{
	m_Volume.VolumeInfoHeader.fDy = height;
} // setVoxelHeight

void CVolume::setVoxelDepth( float depth )
{
	m_Volume.VolumeInfoHeader.fDz = depth;
} // setVoxelDepth

float CVolume::voxelWidth()
{
	return m_Volume.VolumeInfoHeader.fDx;
} // voxelWidth

float CVolume::voxelHeight()
{
	return m_Volume.VolumeInfoHeader.fDy;
} // voxelHeight

float CVolume::voxelDepth()
{
	return m_Volume.VolumeInfoHeader.fDz;
} // voxelDepth

float CVolume::voxelOriginWidth()
{
	return m_Volume.VolumeInfoHeader.fOriginDx;
} // voxelOriginWidth

float CVolume::voxelOriginHeight()
{
	return m_Volume.VolumeInfoHeader.fOriginDy;
} // voxelOriginHeight

float CVolume::voxelOriginDepth()
{
	return m_Volume.VolumeInfoHeader.fOriginDz;
} // voxelOriginDepth

float CVolume::voxelVolume()
{
	return m_Volume.VolumeInfoHeader.fDx *
		m_Volume.VolumeInfoHeader.fDy *
		m_Volume.VolumeInfoHeader.fDz;
} // voxelVolume

bool CVolume::voxelPosition( DWORD id, POSITION_THREED &posPosi )
{
	DWORD dwNumLayer, dwTmp;
	ldiv_t div_result;

	WORD wWidth = m_Volume.VolumeInfoHeader.viWidth;
	WORD wHeight = m_Volume.VolumeInfoHeader.viHeight;
//	WORD wDepth = m_Volume.VolumeInfoHeader.viDepth;
	dwNumLayer = wWidth * wHeight;
		
	div_result = ldiv( id-1, dwNumLayer );

	posPosi.z = div_result.quot + 1;			// depth position  (z)

	dwTmp = div_result.rem;	// id - dwNumLayer*div_result.quot;
	div_result = ldiv( dwTmp, wWidth );

	if( div_result.rem == 0 )
	{
		posPosi.x = (WORD) wWidth;					// width position  (x)
		posPosi.y = (WORD) div_result.quot;			// height position (y)
	}
	else
	{
		posPosi.x = (WORD) div_result.rem;			// width position  (x)
		posPosi.y = div_result.quot + 1;			// height position (y)
	}

	/*
	QString str;
	bool bError = false;
	if ( posPosi.x < 1 || posPosi.x > wWidth )
	{
		str.sprintf( "Error on Voxel Position X..!!\n\nID : %d  Posi (%d, %d, %d)",
			id, posPosi.x, posPosi.y, posPosi.z );
		bError = true;
	}
	if ( posPosi.y < 1 || posPosi.y > wHeight )
	{
		str.sprintf( "Error on Voxel Position Y..!!\n\nID : %d  Posi (%d, %d, %d)",
			id, posPosi.x, posPosi.y, posPosi.z );
		bError = true;
	}
	if ( posPosi.z < 1 || posPosi.z > wDepth )
	{
		str.sprintf( "Error on Voxel Position Z..!!\n\nID : %d  Posi (%d, %d, %d)",
			id, posPosi.x, posPosi.y, posPosi.z );
		bError = true;
	}
	if ( bError )
	{
		QMessageBox::critical( g_pMainWindow, "Position Error", str );
		return false;
	}
	else
		return true;
	*/

	return true;
} // voxelPosition

/*
VOXEL CVolume::voxel( DWORD id )
{
	return m_Volume.pVoxel[id-1];
} // voxel
*/

WORD CVolume::voxelValue( DWORD id )
{
	WORD value = m_Volume.wDensity[ id-1 ] & 0x7fff;

	return value;
} // voxelValue

BYTE CVolume::voxelColor( DWORD id )
{
	WORD value = voxelValue( id );

// qDebug( "%d %d %d", value, m_wMaxImageValue, m_wMinImageValue );

	if ( m_nDicomFlag )
	{
		value = (WORD) ( value * 255.0 / (m_wMaxImageValue-m_wMinImageValue) );
	}

	return (BYTE) value;
}

BYTE CVolume::voxelFlag( DWORD id )
{
	return m_Volume.byteFlag[ id-1 ];
} // voxelFlag

void CVolume::setMask( DWORD id, bool flag )
{
	if ( flag )
		m_Volume.wDensity[ id-1 ] |= 0x8000;
	else
		m_Volume.wDensity[ id-1 ] &= 0x7fff;
} // setMask

bool CVolume::isMaskOn( DWORD id )
{
	if ( m_Volume.wDensity[ id-1 ] & 0x8000 )
		return true;
	else
		return false;
} // isMaskOn

/*
void CVolume::setVoxelFlag(DWORD id, BYTE flag)
{
	this->m_Volume.byteFlag[ id-1 ] = flag;
} // setVoxelFlag
*/


/*************************/ 
/* THRESHOLD Information */
/*************************/

WORD CVolume::minThreshold( int id )
{
	return m_pMinThreshold[id-1];
} // minThreshold

void CVolume::setMinThreshold( int id, WORD minValue )
{
	m_pMinThreshold[id-1] = minValue;
} // setMinThreshold

void CVolume::setMinThreshold( int id, RGB rgbMin )
{
	m_pMinThreshold[id-1] = (int)
		( 0.299*rgbMin.rgbRed
		+ 0.587*rgbMin.rgbGreen
		+ 0.114*rgbMin.rgbBlue );
} // setMinThreshold

WORD CVolume::maxThreshold( int id)
{
	return m_pMaxThreshold[id-1];
} // maxThreshold

void CVolume::setMaxThreshold( int id, WORD maxValue )
{
	m_pMaxThreshold[id-1] = maxValue;
} // setMaxThreshold

void CVolume::setMaxThreshold( int id, RGB rgbMax )
{
	m_pMaxThreshold[id-1] = (int)
		( 0.299*rgbMax.rgbRed
		+ 0.587*rgbMax.rgbGreen
		+ 0.114*rgbMax.rgbBlue );
} // setMaxThreshold

/*
void CVolume::setThresholdRgb( RGB rgb )
{
	m_rgbThreshold.rgbRed = rgb.rgbRed;
	m_rgbThreshold.rgbGreen = rgb.rgbGreen;
	m_rgbThreshold.rgbBlue = rgb.rgbBlue;
} // setThresholdRgb

float CVolume::thresholdGrayValue()
{
	return (float) ( 0.299*m_rgbThreshold.rgbRed
		+ 0.587*m_rgbThreshold.rgbGreen
		+ 0.114*m_rgbThreshold.rgbBlue );
} // thresholdGrayValue

RGB CVolume::thresholdRgb()
{
	return m_rgbThreshold;
} // thresholdRgb
*/


/**********************/ 
/* VOLUME Information */
/**********************/

WORD CVolume::volumeWidth()
{
	return m_Volume.VolumeInfoHeader.viWidth;
} // volumeWidth

WORD CVolume::volumeHeight()
{
	return m_Volume.VolumeInfoHeader.viHeight;
} // volumeHeight

WORD CVolume::volumeDepth()
{
	return m_Volume.VolumeInfoHeader.viDepth;
} // volumeDepth

float CVolume::volumeVolume()
{
	float fVoxelVolume, fVolumeVolume;

	fVoxelVolume = voxelVolume();

	DWORD dwTotal = m_Volume.VolumeInfoHeader.viWidth
		* m_Volume.VolumeInfoHeader.viHeight
		* m_Volume.VolumeInfoHeader.viDepth;

	fVolumeVolume = fVoxelVolume * dwTotal;

	return fVolumeVolume;
} // volumeVolume

float CVolume::volumeSurfaceArea()
{
	float fVolumeSurfaceArea;
	float fWidth, fHeight, fDepth;

	fWidth = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.viWidth;

	fHeight = m_Volume.VolumeInfoHeader.fDy
		* m_Volume.VolumeInfoHeader.viHeight;

	fDepth = m_Volume.VolumeInfoHeader.fDz
		* m_Volume.VolumeInfoHeader.viDepth;

	fVolumeSurfaceArea = 2 * fWidth * fHeight +
		2 * fHeight * fDepth + 2 * fDepth * fWidth;

	return fVolumeSurfaceArea;
} // volumeSurfaceArea

float CVolume::feaVolume()
{
	float fVoxelVolume, fFeasibleVolume;

	fVoxelVolume = voxelVolume();

	fFeasibleVolume = fVoxelVolume * m_dwNumCurrentFeaVoxel;

	return fFeasibleVolume;
} // feaVolume

float CVolume::feaSurfaceArea()
{
	float fFeasibleSurfaceArea;
	float fSurfaceArea[3];

	fSurfaceArea[0] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDy;

	fSurfaceArea[1] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDz;

	fSurfaceArea[2] = m_Volume.VolumeInfoHeader.fDy
		* m_Volume.VolumeInfoHeader.fDz;

	fFeasibleSurfaceArea = fSurfaceArea[0] * m_dwCurrentFeaSurface[0]
		+ fSurfaceArea[1] * m_dwCurrentFeaSurface[1]
		+ fSurfaceArea[2] * m_dwCurrentFeaSurface[2];

	return fFeasibleSurfaceArea;
} // feaSurfaceArea

DWORD CVolume::totalNumVoxel()
{
	DWORD total = m_Volume.VolumeInfoHeader.viWidth
		* m_Volume.VolumeInfoHeader.viHeight
		* m_Volume.VolumeInfoHeader.viDepth;

	return total;
} // totalNumVoxel


/*******************/
/* etc information */
/*******************/

DWORD CVolume::numCurrentFeaVoxel()
{
	return m_dwNumCurrentFeaVoxel;
} // numCurrentFeaVoxel

DWORD CVolume::numCurrentBone()
{
	return m_dwNumCurrentBone;
} // numCurrentBone

DWORD CVolume::numFilteredBone()
{
	return m_dwNumFilteredBone;
} // numFilteredBone

DWORD CVolume::numVoxel( int id )
{
	return m_dwNumVoxel[id-1];
} // numVoxel

DWORD CVolume::numFeaVoxel( int id )
{
	return m_dwNumFeaVoxel[id-1];
} // numFeaVoxel

float CVolume::boneSurfaceArea( int id )
{
	return m_fBoneSurfaceArea[id-1];
} // boneSurfaceArea

float CVolume::boneVolume( int id )
{
	return m_fBoneVolume[id-1];
} // boneVolume

DWORD CVolume::numBone( int id )
{
	return m_dwNumBone[id-1];
} // numBone

bool CVolume::isDicom()
{
	if ( m_nDicomFlag )
		return true;
	else
		return false;
} // isDicom

WORD *CVolume::density()
{
	if ( m_Volume.wDensity )
		return m_Volume.wDensity; 
	else
		return NULL;
} // density


/***************************/
/* calculation information */
/***************************/
void CVolume::calculateNumVoxel()
{
	for(int j=0 ; j<=255 ; j++)
		m_dwNumVoxel[j] = 0;

	// calculate number of voxel
	// according to density level from 0 to 255
	DWORD dwTotalNumVoxel = totalNumVoxel();
	for ( DWORD i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		m_dwNumVoxel[voxelColor(i)]++;
	}
} // calculateNumVoxel

bool CVolume::calculateNumCurrentBone( bool bfilter )
{
	CQueue qBone;
	CQueue qSurrVoxel;
	CQueue qFilterVoxel;
	DWORD dwTotalNumVoxel = totalNumVoxel();
	DWORD dwBoneCount =0;
	DWORD dwFilterCount =0;
	DWORD dwVoxelCount = 0;
	DWORD dwId1, dwId2;
	DWORD i, j;		// temp variable

	DWORD oldBone = m_dwNumCurrentBone;
	DWORD oldFiltered = m_dwNumFilteredBone;

	QString strID;

	// initialize voxel byteFlag to calculate number of bone
//	WORD tmpLayer = 0;
//	DWORD numLayer = volumeWidth() * volumeHeight();

	/*
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		if ( i % numLayer == 0 )
			tmpLayer++;

		m_Volume.byteFlag[i-1] &= 0xbf;	// initialize 7th bit to 0

		if ( m_Volume.wDensity[i-1] >= m_pMinThreshold[tmpLayer] &&
			 m_Volume.wDensity[i-1] <= m_pMaxThreshold[tmpLayer] )
			m_Volume.byteFlag[i-1] |= 0x40;	// set visible
	}
	*/
	
	// check min & max segmentation value
	// if all min values are MIN
	// and all max vallues are MAX
	// number of bone is 1, so just return 1;
	bool bOne = true;
	for ( i=0 ; i<volumeDepth(); i++ )
	{
		if ( m_pMinThreshold[i] != m_wMinImageValue )
			bOne = false;
		if ( m_pMaxThreshold[i] != m_wMaxImageValue )
			bOne = false;
	}
	if ( bOne )
	{
		m_dwNumCurrentBone = 1;
		m_dwNumFilteredBone = 0;
		return true;
	}


	// open temp file for number of bone
	QFile boneNumFile( QDir::homeDirPath() + "/.mesh_filter.tmp" );

// qDebug( "%s", (QDir::homeDirPath() + "/.mesh_filter.tmp").latin1() );

    if ( !boneNumFile.open( IO_WriteOnly ) )	// file open error..!!
	{
		QMessageBox::critical( g_pMainWindow, "File Open Error",
			"Can't open temp file for number of bone..." );
		return false;
	}

	QTextStream ts( &boneNumFile );	// use a text stream
	ts << "*****************************************************************************" << endl;
	ts << "*                                                                           *" << endl;
	ts << "*                              MGIBA 1.0.0                                  *" << endl;
	ts << "*                                                                           *" << endl;
	ts << "*                      Bio Medical Soft(c) 2000-2002                        *" << endl;
	ts << "*                                                                           *" << endl;
	ts << "*****************************************************************************" << endl << endl << endl;


	BYTE *pByteFlag = new BYTE[dwTotalNumVoxel];
	if ( !pByteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for Temporary Flag..!!" );

		return false;
	}

	// set temporary flag
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		m_Volume.byteFlag[i-1] &= 0x7f;		// set 8th bit to 0
		pByteFlag[i-1] = m_Volume.byteFlag[i-1];
	}

	QString msg;
	msg = "Calculate Number of Bone...\n\nIt takes a few minutes...";
	if ( bfilter )
		msg = "Filtering Voxel Mesh...\n\nIt takes a few minutes...";
	QProgressDialog progress( msg, "Abort", 100,
                                g_pMainWindow, "progress", true );
	progress.show();

	int percentage = 0;
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		if ( i % (DWORD)(dwTotalNumVoxel*0.05) == 0 )
		{
			progress.setProgress( percentage );
			qApp->processEvents();
			percentage += 5;
		}

		if ( progress.wasCancelled() )
		{
_FUNC_END:
			// initialize
			for ( j=1 ; j<=dwTotalNumVoxel ; j++ )
				m_Volume.byteFlag[j-1] = pByteFlag[j-1];

			if ( pByteFlag )
			{
				delete pByteFlag;
				pByteFlag = NULL;
			}

			m_dwNumCurrentBone = oldBone;
			m_dwNumFilteredBone = oldFiltered;

			return false;
		}

		if ( (m_Volume.byteFlag[i-1] & 0x40) == 0x40 )	// if visible
		{
			m_Volume.byteFlag[i-1] &= 0xbf;	// initialize 7th bit to 0

			dwId1 = i;

			dwBoneCount++;
			dwVoxelCount++;

			// write to temp file
			ts << "# No. " << dwBoneCount << endl;
			strID.sprintf( "ID %10ld", dwId1 );
			ts << strID << "   ";
			
			qFilterVoxel.insert( dwId1 );
			
_BEGIN:
			if ( progress.wasCancelled() )
				goto _FUNC_END;

			if ( !evaluateSurrVoxel( dwId1, qSurrVoxel ) )
				goto _FUNC_END;
		
			while ( !qSurrVoxel.is_empty() )
			{
				if ( progress.wasCancelled() )
					goto _FUNC_END;

				dwId2 = qSurrVoxel.get_front();

				// write to temp file
				strID.sprintf( "ID %10ld", dwId2 );
				ts << strID << "   ";

				dwVoxelCount++;

				if ( dwVoxelCount % 5 == 0 )
					ts << endl;

				qBone.insert( dwId2 );
				m_Volume.byteFlag[dwId2-1] &= 0xbf;	// initialize 7th bit to 0

				if ( dwVoxelCount <= m_nNumFilterVoxel )
					qFilterVoxel.insert( dwId2 );
			}

			if ( !qBone.is_empty() )
			{
				dwId1 = qBone.get_front();
				goto _BEGIN;
			}

			ts << endl;		// write to temp file
			ts << "==> Number of voxel : " << dwVoxelCount << endl << endl;	// write to temp file

			// filtering voxel mesh
			if ( dwVoxelCount <= m_nNumFilterVoxel )
			{
				while ( !qFilterVoxel.is_empty() )
				{
					dwId1 = qFilterVoxel.get_front();
					m_Volume.byteFlag[dwId1-1] |= 0x80;	// set 8th bit to 1;
				}

				dwFilterCount++;
			}

			if ( !qFilterVoxel.is_empty() )
				qFilterVoxel.clear();

			dwVoxelCount = 0;
		} // if
	} // for
	progress.setProgress( 100 );
	progress.hide();

	// initialize
	for ( i=1 ; i<=dwTotalNumVoxel ; i++ )
	{
		if ( ( m_Volume.byteFlag[i-1] & 0x80 ) == 0x80 )
			pByteFlag[i-1] |= 0x80;

		m_Volume.byteFlag[i-1] = pByteFlag[i-1];
	}

	ts << endl;								// write to temp file
	ts << "# Total : " << dwBoneCount;		// write to temp file
	boneNumFile.close();

	if ( pByteFlag )
	{
		delete pByteFlag;
		pByteFlag = NULL;
	}

	m_dwNumCurrentBone = dwBoneCount;
	m_dwNumFilteredBone = dwFilterCount;

	m_bMeshFiltered = true;

	return true;
} // calculateNumCurrentBone

bool CVolume::isMeshFiltered()
{
	return m_bMeshFiltered;
} // isMeshFiltered

void CVolume::setMeshFilter( bool filter )
{
	m_bMeshFiltered = filter;
} // setMeshFilter

bool CVolume::findCurrentVisibleSurface( bool reset )
{
	BYTE byteSurface;
	POSITION_THREED posPosi;			// voxel position
	BYTE byteVoxelFlag;					// voxel falg in BYTE
	WORD wNumWidth, wNumHeight, wNumDepth;
	DWORD nNumLayer;
	WORD tmpLayer = 0;
	DWORD i;

	wNumWidth = volumeWidth();
	wNumHeight = volumeHeight();
	wNumDepth = volumeDepth();
	nNumLayer = wNumWidth * wNumHeight;

	m_dwNumCurrentFeaVoxel = 0;
	m_dwCurrentFeaSurface[0] = 0;
	m_dwCurrentFeaSurface[1] = 0;
	m_dwCurrentFeaSurface[2] = 0;

	DWORD total = totalNumVoxel();

	QList<QRegion> listRgnROI = m_pMDIWindow->planView()->planViewArea()->listRgnROI();
	int nCount = listRgnROI.count();

	// set bounding rect for display speed
	QList<QRect> boundingRectROI;
	if ( nCount != 0 )
	{
		for ( int ii=0 ; ii<nCount ; ii++ )
		{
			/*
			QRect *rt = new QRect;
			rt = &listRgnROI.at( ii )->boundingRect();
			*/
			QRect *rt = new QRect( listRgnROI.at( ii )->boundingRect() );
			boundingRectROI.append( rt );
		}
	}

	// check whether point is inside ROI or not
	QProgressDialog progress1( "Check Region Of Interest...", "Abort", 100,
                                g_pMainWindow, "progress1", true );

	if ( nCount != 0 )
		progress1.show();

	// byte flag for region of interest
	BYTE *pByteFlag = new BYTE[total];
	if ( !pByteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for Temporary Flag..!!" );

		return false;
	}

	int percentage = 0;
	for ( i=1 ; i<=total ; i++ )
	{
		if ( nCount != 0 && i % (DWORD)(total*0.05) == 0 )
		{
			progress1.setProgress( percentage );
			qApp->processEvents();
			percentage += 5;
		}

		if ( nCount != 0 && progress1.wasCancelled() )
		{
			if ( pByteFlag )
			{
				delete pByteFlag;
				pByteFlag = NULL;
			}

			return false;
		}

		pByteFlag[i-1] = 0;

		if ( nCount == 0 )
			pByteFlag[i-1] = 0x80;
		else
		{
			if ( !voxelPosition( i, posPosi ) )
				return false;

			QPoint pt( posPosi.x-1, posPosi.y-1 );

			for ( int ii=0 ; ii<nCount ; ii++ )
			{
				if ( boundingRectROI.at( ii )->contains( pt ) )
				{
					QRegion *pRgn = listRgnROI.at( ii );

					if ( pRgn->contains( pt ) )
					{
						pByteFlag[i-1] = 0x80;
						goto _NEXT_RET;
					}
				}
			}
		}

_NEXT_RET:;

	}
	if ( nCount != 0 )
	{
		progress1.setProgress( 100 );
		progress1.hide();
	}


	// find visible surface
	QString msg;
	msg = "Constructing 3D-Model...";
	if ( reset )
		msg = "Reconstructing 3D-Model...";

	QProgressDialog progress2( msg, "Abort", 100,
                                g_pMainWindow, "progress2", true );
	progress2.show();
	percentage = 0;
	for ( i=1 ; i<=total ; i++ )
	{
		if ( i % (DWORD)(total*0.05) == 0 )
		{
			progress2.setProgress( percentage );
			qApp->processEvents();
			percentage += 5;
		}

		if ( progress2.wasCancelled() )
		{
			if ( pByteFlag )
			{
				delete pByteFlag;
				pByteFlag = NULL;
			}

			return false;
		}

		if ( i % nNumLayer == 0 )
			tmpLayer++;

		if ( !voxelPosition( i, posPosi ) )
			return false;

		byteVoxelFlag = pByteFlag[i-1];	// voxelFlag( i );

		byteSurface = 0;

		if ( (byteVoxelFlag & 0x80) && isMaskOn( i ) )
		{
			m_dwNumCurrentFeaVoxel++;

			/*
			// check filtered voxel
			if ( ( voxelFlag( i ) & 0x80 ) == 0x80 )
				byteSurface |= 0x80;
			*/

			// set current voxel to visible
			byteSurface |= 0x40;

			// 1st direction
			if ( posPosi.z == 1 )
			{
				byteSurface |= SURFACE1;
				m_dwCurrentFeaSurface[0]++;
			}
			else
			{
				if ( !isMaskOn( i-nNumLayer ) )
				{
					byteSurface |= SURFACE1;
					m_dwCurrentFeaSurface[0]++;
				}
			}

			// 2nd direction
			if ( posPosi.z == wNumDepth )
			{
				byteSurface |= SURFACE2;
				this->m_dwCurrentFeaSurface[0]++;
			}
			else{
				if ( !isMaskOn( i+nNumLayer ) )
				{
					byteSurface |= SURFACE2;
					m_dwCurrentFeaSurface[0]++;
				}
			}

			// 3rd direction
			if ( posPosi.y == 1 )
			{
				byteSurface |= SURFACE3;
				m_dwCurrentFeaSurface[1]++;
			}
			else
			{
				byteVoxelFlag = pByteFlag[i-wNumWidth-1];	// voxelFlag( i-wNumWidth );
				if ( (byteVoxelFlag & 0x80) != 0x80 )
				{
					byteSurface |= SURFACE3;
					m_dwCurrentFeaSurface[1]++;
				}
				else
				{
					if ( !isMaskOn( i-wNumWidth ) )
					{
						byteSurface |= SURFACE3;
						m_dwCurrentFeaSurface[1]++;
					}
				}
			}

			// 4th direction
			if ( posPosi.y == wNumHeight )
			{
				byteSurface |= SURFACE4;
				m_dwCurrentFeaSurface[1]++;
			}
			else
			{
				byteVoxelFlag = pByteFlag[i+wNumWidth-1];	// voxelFlag( i+wNumWidth );
				if ( (byteVoxelFlag & 0x80) != 0x80 )
				{
					byteSurface |= SURFACE4;
					m_dwCurrentFeaSurface[1]++;
				}
				else
				{
					if ( !isMaskOn( i+wNumWidth ) )
					{
						byteSurface |= SURFACE4;
						m_dwCurrentFeaSurface[1]++;
					}
				}
			}

			// 5th direction
			if ( posPosi.x == wNumWidth )
			{
				byteSurface |= SURFACE5;
				m_dwCurrentFeaSurface[2]++;
			}
			else
			{
				byteVoxelFlag = pByteFlag[i];	// voxelFlag( i+1 );
				if ( (byteVoxelFlag & 0x80) != 0x80 )
				{
					byteSurface |= SURFACE5;
					m_dwCurrentFeaSurface[2]++;
				}
				else
				{
					if ( !isMaskOn( i+1 ) )
					{
						byteSurface |= SURFACE5;
						m_dwCurrentFeaSurface[2]++;
					}
				}
			}

			// 6th direction
			if ( posPosi.x == 1 )
			{
				byteSurface |= SURFACE6;

				m_dwCurrentFeaSurface[2]++;
			}
			else
			{
				byteVoxelFlag = pByteFlag[i-2];		// voxelFlag( i-1 );
				if ( (byteVoxelFlag & 0x80) != 0x80 )
				{
					byteSurface |= SURFACE6;
					m_dwCurrentFeaSurface[2]++;
				}
				else
				{
					if ( !isMaskOn( i-1 ) )
					{
						byteSurface |= SURFACE6;
						m_dwCurrentFeaSurface[2]++;
					}
				}
			} // if - 6th direction
		}

		m_Volume.byteFlag[i-1] = byteSurface;
	}
	progress2.setProgress( 100 );
	progress2.hide();

	if ( pByteFlag )
	{
		delete pByteFlag;
		pByteFlag = NULL;
	}

	return true;
} // findCurrentVisibleSurface


/**********************/
/* filter information */
/**********************/

void CVolume::setNumFilterVoxel( DWORD voxel )
{
	m_nNumFilterVoxel = voxel;
} // setNumFilterVoxel

DWORD CVolume::numFilterVoxel()
{
	return m_nNumFilterVoxel;
} // numFilterVoxel


/********************************/
/* volume file name information */
/********************************/

QString CVolume::volumeFileName()
{
	return m_strFileName;
} // volumeFileName


/*************************/
/* export finite element */
/*************************/

bool CVolume::exportToAnsys( QString fileName )
{
	if( !m_Volume.wDensity || !m_Volume.byteFlag )
		return false;

	QFile ansysFile( fileName );
    if ( !ansysFile.open( IO_WriteOnly ) )	// file open error..!!
	{
		QMessageBox::critical( g_pMainWindow, "File Open Error",
			"Can't open ansys file..." );
		return false;
	}

	// file opened successfully
	QTextStream ts( &ansysFile );	// use a text stream


	/************************/
	/* write finite element */
	/************************/
	DWORD dwTmp;
	DWORD dwWidth = volumeWidth();
	DWORD dwHeight = volumeHeight();
	DWORD dwDepth = volumeDepth();
	DWORD dwNode[8];

	DWORD dwNumnp2d = ( dwWidth+1 ) * ( dwHeight+1 );
	DWORD dwNumel2d = dwWidth * dwHeight;
	WORD wVoxelValue;
	BYTE byteFlag;

	float fDx = voxelWidth();
	float fDy = voxelHeight();
	float fDz = voxelDepth();
	float fTmpX, fTmpY;

	DWORD i, j, index;

	FE_NODE *pNnpLower = new FE_NODE[dwNumnp2d];
	FE_NODE *pNnpUpper = new FE_NODE[dwNumnp2d];
	int *pNnpPreUpper = new int[dwNumnp2d];


	/****************/
	/* element type */
	/****************/

	// save pre data to *.ans file
	// if layer of element is not "1"
	// element type : SOLID45
	ts << "/prep7" << "\n";
	ts << "et,1,solid45" << "\n";
//	ts << "shpp,off" << "\n";


	/*********************/
	/* material property */
	/*********************/

	if ( m_bHomogeneousPossion && m_bHomogeneousDensity && m_bHomogeneousYoung )
	{
		ts << "mat,1" << "\n";
		ts << "mp,nuxy,1," << m_fPossionLUT[0] << "\n";
		ts << "mp,dens,1," << m_fDensityLUT[0] << "\n";
		ts << "mp,ex,1," << m_fYoungLUT[0] << "\n";
	}
	else
	{
		float fPossion, fDensity, fYoung;

		for ( i=0 ; i<256 ; i++ )
		{
			// possion ratio
			if ( m_bHomogeneousPossion )
				fPossion = m_fPossionLUT[0];
			else
				fPossion = m_fPossionLUT[i];
			// density
			if ( m_bHomogeneousDensity )
				fDensity = m_fDensityLUT[0];
			else
				fDensity = m_fDensityLUT[i];
			// young's modulus
			if ( m_bHomogeneousYoung )
				fYoung = m_fYoungLUT[0];
			else
				fYoung = m_fYoungLUT[i];

			ts << "mat," << i+1 << "\n";
			ts << "mp,nuxy," << i+1 << "," << fPossion << "\n";
			ts << "mp,density," << i+1 << "," << fDensity << "\n";
			ts << "mp,ex," << i+1 << "," << fYoung << "\n";
		}
	}


	/**************/
	/* write node */
	/**************/

	DWORD tmpTotal = dwDepth * (3*dwNumnp2d + dwNumel2d);
	DWORD tmp = 0;
	QProgressDialog progressNode( "Writing Node...", "Abort", 100,
                                g_pMainWindow, "progressNode", true );
	progressNode.show();

	int percentage = 0;
	for ( j=1 ; j<=dwDepth ; j++ )	// for z-direction
	{
		fTmpX = fTmpY = 0.0;
		for( i=1 ; i<=dwNumnp2d ; i++)
		{
			if ( tmp % (DWORD)(tmpTotal*0.01) == 0 )
			{
				progressNode.setProgress( percentage );
				qApp->processEvents();
				percentage += 1;
			}
			tmp++;

			if ( progressNode.wasCancelled() )
			{
				ansysFile.close();
				ansysFile.remove();
				return false;
			}

			pNnpLower[i-1].id = i + dwNumnp2d*(j-1);
			pNnpLower[i-1].x = fTmpX;
			pNnpLower[i-1].y = fTmpY;
			pNnpLower[i-1].z = (j-1)*fDz;
			pNnpLower[i-1].count = 0;
			pNnpLower[i-1].bDupFlag = false;

			pNnpUpper[i-1].id = i + dwNumnp2d*j;
			pNnpUpper[i-1].x = fTmpX;
			pNnpUpper[i-1].y = fTmpY;
			pNnpUpper[i-1].z = j*fDz;
			pNnpUpper[i-1].count = 0;
			pNnpUpper[i-1].bDupFlag = false;

			if( i % ( dwWidth + 1 ) == 0 )
			{
				fTmpX = 0.;
				fTmpY += fDy;
			}
			else
			{
				fTmpX += fDx;
			}
		}

		dwTmp = 0;
		for( i=1 ; i<=dwNumel2d ; i++)
		{
			if ( tmp % (DWORD)(tmpTotal*0.01) == 0 )
			{
				progressNode.setProgress( percentage );
				qApp->processEvents();
				percentage += 1;
			}
			tmp++;

			if ( progressNode.wasCancelled() )
			{
				ansysFile.close();
				ansysFile.remove();
				return false;
			}

			// set element ID
			index = i + dwNumel2d*(j-1);

			dwNode[0] = i + dwTmp + dwNumnp2d*(j-1);
			dwNode[1] = i + 1 + dwTmp + dwNumnp2d*(j-1);
			dwNode[2] = i + dwWidth + 2 + dwTmp + dwNumnp2d*(j-1);
			dwNode[3] = i + dwWidth + 1 + dwTmp + dwNumnp2d*(j-1);
				
			dwNode[4] = i + dwTmp + dwNumnp2d*j;
			dwNode[5] = i + 1 + dwTmp + dwNumnp2d*j;
			dwNode[6] = i + dwWidth + 2 + dwTmp + dwNumnp2d*j;
			dwNode[7] = i + dwWidth + 1 + dwTmp + dwNumnp2d*j;

			if( i % dwWidth == 0 ) dwTmp++;

			wVoxelValue = voxelValue( index );
			byteFlag = voxelFlag( index );
			if ( ( byteFlag & 0x40 ) == 0x40 &&		// if visible
				 ( byteFlag & 0x80 ) != 0x80 )		// if not filtered
			{
				// ts << index << "   " << wVoxelValue << "\n";
				for ( int ii=0 ; ii<8 ; ii++ )
				{
					DWORD idx;

					idx = dwNode[ii] % (dwNumnp2d*2);
					if ( idx == 0 ) idx = dwNumnp2d*2;

					if ( idx > dwNumnp2d )
					{
						idx = idx - dwNumnp2d - 1;
						pNnpUpper[idx].count++;
					}
					else
					{
						idx--;
						pNnpLower[idx].count++;
					}
				} // for
			} // if
		}

		for ( i=0 ; i<dwNumnp2d ; i++ )
		{
			if ( tmp % (DWORD)(tmpTotal*0.01) == 0 )
			{
				progressNode.setProgress( percentage );
				qApp->processEvents();
				percentage += 1;
			}
			tmp++;

			if ( progressNode.wasCancelled() )
			{
				ansysFile.close();
				ansysFile.remove();
				return false;
			}

			// ̸   带 Ѵ.
			if ( j == 1 )
			{
				if ( pNnpLower[i].count != 0 )
					ts << "n," << pNnpLower[i].id
						<< "," << pNnpLower[i].x
						<< "," << pNnpLower[i].y
						<< "," << pNnpLower[i].z << "\n";
			}
			//  ƴϸ  尡     ߺǸ,
			//  ʴ´.
			else
			{
				if ( pNnpLower[i].count != 0 && pNnpPreUpper[i] != 1 )
					ts << "n," << pNnpLower[i].id
						<< "," << pNnpLower[i].x
						<< "," << pNnpLower[i].y
						<< "," << pNnpLower[i].z << "\n";
			}
		}
		for ( i=0 ; i<dwNumnp2d ; i++ )
		{
			if ( tmp % (DWORD)(tmpTotal*0.01) == 0 )
			{
				progressNode.setProgress( percentage );
				qApp->processEvents();
				percentage += 1;
			}
			tmp++;

			if ( progressNode.wasCancelled() )
			{
				ansysFile.close();
				ansysFile.remove();
				return false;
			}

			pNnpPreUpper[i] = 0;	//    ʱȭ : 0 ->  X
			if ( pNnpUpper[i].count != 0 )
			{
				ts << "n," << pNnpUpper[i].id
					<< "," << pNnpUpper[i].x
					<< "," << pNnpUpper[i].y
					<< "," << pNnpUpper[i].z << "\n";
			
				pNnpPreUpper[i] = 1;	//  尡 Ǿ. : 1
			}
		}

	}
	progressNode.setProgress( 100 );
	progressNode.hide();


	/*****************/
	/* write element */
	/*****************/

	QProgressDialog progressElement( "Writing Element...", "Abort", 100,
                                g_pMainWindow, "progressElement", true );
	progressElement.show();

	DWORD total = totalNumVoxel();
	int matIndex;
//	float fVal, fYoung, fPossion;

	percentage = 0;
	for ( j=1 ; j<=dwDepth ; j++ )
	{
		dwTmp = 0;
		for ( i=1 ; i<=dwNumel2d ; i++ )
		{
			index = i + dwNumel2d*(j-1);

			if ( index % (DWORD)(total*0.01) == 0 )
			{
				progressElement.setProgress( percentage );
				qApp->processEvents();
				percentage += 1;
			}

			if ( progressElement.wasCancelled() )
			{
				ansysFile.close();
				ansysFile.remove();
				return false;
			}

			wVoxelValue = voxelValue( index );
			byteFlag = voxelFlag( index );
			if ( ( byteFlag & 0x40 ) == 0x40 &&		// if visible
				 ( byteFlag & 0x80 ) != 0x80 )		// if not filtered
			{
				dwNode[0] = i + dwTmp + dwNumnp2d*(j-1);
				dwNode[1] = i + 1 + dwTmp + dwNumnp2d*(j-1);
				dwNode[2] = i + dwWidth + 2 + dwTmp + dwNumnp2d*(j-1);
				dwNode[3] = i + dwWidth + 1 + dwTmp + dwNumnp2d*(j-1);

				dwNode[4] = i + dwTmp + dwNumnp2d*j;
				dwNode[5] = i + 1 + dwTmp + dwNumnp2d*j;
				dwNode[6] = i + dwWidth + 2 + dwTmp + dwNumnp2d*j;
				dwNode[7] = i + dwWidth + 1 + dwTmp + dwNumnp2d*j;

				if ( m_bHomogeneousPossion && m_bHomogeneousDensity && m_bHomogeneousYoung )
					matIndex = 1;
				else
					matIndex = (int) (wVoxelValue+1);

				// write element
				ts << "en,"
					<< matIndex << ","
					<< dwNode[0] << ","
					<< dwNode[1] << ","
					<< dwNode[2] << ","
					<< dwNode[3] << ","
					<< dwNode[4] << ","
					<< dwNode[5] << ","
					<< dwNode[6] << ","
					<< dwNode[7] << "\n";
			}

			if ( i % dwWidth == 0 ) dwTmp++;
		} // for i
	} // for j
	progressElement.setProgress( 100 );
	progressElement.hide();

	ansysFile.close();

	return true;
} // exportToAnsys

void CVolume::setHomogeneousPossion( bool homo )
{
	m_bHomogeneousPossion = homo;
} // setHomogeneousPossion

void CVolume::setHomogeneousDensity( bool homo )
{
	m_bHomogeneousDensity = homo;
} // setHomogeneousDensity

void CVolume::setHomogeneousYoung( bool homo )
{
	m_bHomogeneousYoung = homo;
} // setHomogeneousYoung

void CVolume::setPossionLUT( int idx, float value )
{
	m_fPossionLUT[idx] = value;
} // setPossionLUT

void CVolume::setDensityLUT( int idx, float value )
{
	m_fDensityLUT[idx] = value;
} // setDensityLUT

void CVolume::setYoungLUT( int idx, float value )
{
	m_fYoungLUT[idx] = value;
} // setYoungLUT

bool CVolume::isSurfaceContact()
{
	if ( m_bSurfaceContact )
		return true;
	else
		return false;
} // isSurfaceContact

void CVolume::setSurfaceContact( bool flag )
{
	m_bSurfaceContact = flag;
} // setSurfaceContact



/******************/
/* export surface */
/******************/

bool CVolume::exportToSurface( QString fileName, int type )
{
	DWORD total = totalNumVoxel();
	BYTE byteFlag;
	WORD wVoxelValue;
	float fDx, fDy, fDz;
	float fPoint1[3], fPoint2[3];
	POSITION_THREED posPosition;
	SRF_NODE node[4];

	QFile srfFile( fileName );
	if ( !srfFile.open( IO_WriteOnly ) )
	{
		QMessageBox::critical( g_pMainWindow, "File Open Error",
			"Can't open file..." );
		return false;
	}

	fDx = voxelWidth();
	fDy = voxelHeight();
	fDz = voxelDepth();

	QTextStream ts;
	QDataStream ds;

	if ( type == SRF_STL_ASCII || type == SRF_ACAD_SCRIPT || type == SRF_ACAD_DXF )
		ts.setDevice( &srfFile );
	else if ( type == SRF_STL_BINARY )
		ds.setDevice( &srfFile );

	QProgressDialog progress( "Writing Surface...", "Abort", 100,
                                g_pMainWindow, "progress", true );
	progress.show();

	int percentage = 0;

	if ( type == SRF_STL_ASCII )
		ts << "solid mgiba" << endl;
	else if ( type == SRF_ACAD_DXF )
	{
		ts << "999" << endl;
		ts << "mgiba" << endl;
		ts << "0" << endl;
		ts << "SECTION" << endl;
		ts << "2" << endl;
		ts << "HEADER" << endl;
		ts << "0" << endl;
		ts << "ENDSEC" << endl;

		ts << "0" << endl;
		ts << "SECTION" << endl;
		ts << "2" << endl;
		ts << "TABLES" << endl;
		ts << "0" << endl;
		ts << "TABLE" << endl;
		ts << "2" << endl;
		ts << "LAYER" << endl;
		ts << "70" << endl;
		ts << "1" << endl;
		ts << "0" << endl;
		ts << "LAYER" << endl;
		ts << "2" << endl;
		ts << "0" << endl;
		ts << "70" << endl;
		ts << "0" << endl;
		ts << "62" << endl;
		ts << "7" << endl;
		ts << "6" << endl;
		ts << "CONTINUOUS" << endl;
		ts << "0" << endl;
		ts << "ENDTAB" << endl;
		ts << "0" << endl;
		ts << "ENDSEC" << endl;
		
		ts << "0" << endl;
		ts << "SECTION" << endl;
		ts << "2" << endl;
		ts << "BLOCKS" << endl;
		ts << "0" << endl;
		ts << "ENDSEC" << endl;

		ts << "0" << endl;
		ts << "SECTION" << endl;
		ts << "2" << endl;
		ts << "ENTITIES" << endl;
	}

	for ( DWORD i=1 ; i<=total ; i++ )
	{
		if ( i % (DWORD)(total*0.01) == 0 )
		{
			progress.setProgress( percentage );
			qApp->processEvents();
			percentage += 1;
		}

		if ( progress.wasCancelled() )
		{
			srfFile.close();
			srfFile.remove();
			return false;
		}

		byteFlag = voxelFlag( i );
		wVoxelValue = voxelValue( i );

		if ( ( byteFlag & 0x40 ) == 0x40 &&		// if visible
			 ( byteFlag & 0x80 ) != 0x80 )		// if not filtered
		{
			if ( !voxelPosition( i, posPosition ) )
				return false;
			
			fPoint1[0] = fDx * (posPosition.x-1);
			fPoint1[1] = fDy * (posPosition.y-1);
			fPoint1[2] = fDz * (posPosition.z-1);

			fPoint2[0] = fDx * posPosition.x;
			fPoint2[1] = fDy * posPosition.y;
			fPoint2[2] = fDz * posPosition.z;

			if ( (byteFlag & SURFACE1) == SURFACE1 )
			{
				node[0].x = fPoint1[0];
				node[0].y = fPoint1[1];
				node[0].z = fPoint1[2];

				node[1].x = fPoint1[0];
				node[1].y = fPoint2[1];
				node[1].z = fPoint1[2];

				node[2].x = fPoint2[0];
				node[2].y = fPoint2[1];
				node[2].z = fPoint1[2];

				node[3].x = fPoint2[0];
				node[3].y = fPoint1[1];
				node[3].z = fPoint1[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 1 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 1 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}

			if ( (byteFlag & SURFACE2) == SURFACE2 )
			{
				node[0].x = fPoint1[0];
				node[0].y = fPoint1[1];
				node[0].z = fPoint2[2];

				node[1].x = fPoint2[0];
				node[1].y = fPoint1[1];
				node[1].z = fPoint2[2];

				node[2].x = fPoint2[0];
				node[2].y = fPoint2[1];
				node[2].z = fPoint2[2];

				node[3].x = fPoint1[0];
				node[3].y = fPoint2[1];
				node[3].z = fPoint2[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 2 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 2 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}

			if ( (byteFlag & SURFACE3) == SURFACE3 )
			{
				node[0].x = fPoint1[0];
				node[0].y = fPoint1[1];
				node[0].z = fPoint1[2];

				node[1].x = fPoint2[0];
				node[1].y = fPoint1[1];
				node[1].z = fPoint1[2];

				node[2].x = fPoint2[0];
				node[2].y = fPoint1[1];
				node[2].z = fPoint2[2];

				node[3].x = fPoint1[0];
				node[3].y = fPoint1[1];
				node[3].z = fPoint2[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 3 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 3 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}

			if ( (byteFlag & SURFACE4) == SURFACE4 )
			{
				node[0].x = fPoint1[0];
				node[0].y = fPoint2[1];
				node[0].z = fPoint1[2];

				node[1].x = fPoint1[0];
				node[1].y = fPoint2[1];
				node[1].z = fPoint2[2];

				node[2].x = fPoint2[0];
				node[2].y = fPoint2[1];
				node[2].z = fPoint2[2];

				node[3].x = fPoint2[0];
				node[3].y = fPoint2[1];
				node[3].z = fPoint1[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 4 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 4 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}

			if ( (byteFlag & SURFACE5) == SURFACE5 )
			{
				node[0].x = fPoint2[0];
				node[0].y = fPoint1[1];
				node[0].z = fPoint1[2];

				node[1].x = fPoint2[0];
				node[1].y = fPoint2[1];
				node[1].z = fPoint1[2];

				node[2].x = fPoint2[0];
				node[2].y = fPoint2[1];
				node[2].z = fPoint2[2];

				node[3].x = fPoint2[0];
				node[3].y = fPoint1[1];
				node[3].z = fPoint2[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 5 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 5 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}

			if ( (byteFlag & SURFACE6) == SURFACE6 )
			{
				node[0].x = fPoint1[0];
				node[0].y = fPoint1[1];
				node[0].z = fPoint1[2];

				node[1].x = fPoint1[0];
				node[1].y = fPoint1[1];
				node[1].z = fPoint2[2];

				node[2].x = fPoint1[0];
				node[2].y = fPoint2[1];
				node[2].z = fPoint2[2];

				node[3].x = fPoint1[0];
				node[3].y = fPoint2[1];
				node[3].z = fPoint1[2];

				if ( type == SRF_STL_ASCII )
					writeToSTLAsciiFormat( ts, node, 6 );
				else if ( type == SRF_STL_BINARY )
					writeToSTLBinaryFormat( ds, node, 6 );
				else if ( type == SRF_ACAD_SCRIPT )
					writeToAcadScriptFormat( ts, node, wVoxelValue );
				else if ( type == SRF_ACAD_DXF )
					writeToAcadDxfFormat( ts, node );
			}
		}
	}

	if ( type == SRF_STL_ASCII )
		ts << "end solid mgiba" << endl;
	else if ( type == SRF_ACAD_DXF )
	{
		ts << "0" << endl;
		ts << "ENDSEC" << endl;
		ts << "0" << endl;
		ts << "EOF" << endl;
	}

	progress.setProgress( 100 );
	progress.hide();

	srfFile.close();

	return true;
} // exportToSurface


/********************/
/* private function */
/********************/

// private
bool CVolume::evaluateSurrVoxel( DWORD id, CQueue &queue )
{
//	CQueue qSurrVoxel;
	DWORD dwVoxel[26];
	int nSize;
	POSITION_THREED posPosi;
	WORD wWidth, wHeight, wDepth;
	DWORD nNumLayer;
	BYTE byteFlag;

	if ( !voxelPosition( id, posPosi ) )
		return false;

	wWidth = m_Volume.VolumeInfoHeader.viWidth;
	wHeight = m_Volume.VolumeInfoHeader.viHeight;
	wDepth = m_Volume.VolumeInfoHeader.viDepth;

	nNumLayer = wWidth * wHeight;

	nSize = 0;
	byteFlag = 0;

	/* 8   3   7
	   4  id   2
	   5   1   6 */

	// case id
	if( posPosi.z != 1 )
		dwVoxel[nSize++] = id - nNumLayer;
	if( posPosi.z != wDepth )
		dwVoxel[nSize++] = id + nNumLayer;

	// case 1
	if ( posPosi.y != 1 )
	{
		dwVoxel[nSize++] = id - wWidth;
		byteFlag |= 0x01;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer - wWidth;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer - wWidth;
	}

	// case 2
	if ( posPosi.x != wWidth )
	{
		dwVoxel[nSize++] = id + 1;
		byteFlag |= 0x02;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer + 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer + 1;
	}

	// case 3
	if ( posPosi.y != wWidth )
	{
		dwVoxel[nSize++] = id + wWidth;
		byteFlag |= 0x04;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer + wWidth;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer + wWidth;
	}

	// case 4
	if ( posPosi.x != 1)
	{
		dwVoxel[nSize++] = id - 1;
		byteFlag |= 0x08;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer - 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer - 1;
	}

	// case 5
	if ( (byteFlag & 0x01) == 0x01 && (byteFlag & 0x08) == 0x08 && !m_bSurfaceContact )
	{
		dwVoxel[nSize++] = id - wWidth - 1;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer - wWidth - 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer - wWidth - 1;
	}

	// case 6
	if ( (byteFlag & 0x01) == 0x01 && (byteFlag & 0x02) == 0x02 && !m_bSurfaceContact )
	{
		dwVoxel[nSize++] = id - wWidth + 1;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer - wWidth + 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer - wWidth + 1;
	}

	// case 7
	if ( (byteFlag & 0x02) == 0x02 && (byteFlag & 0x04) == 0x04 && !m_bSurfaceContact )
	{
		dwVoxel[nSize++] = id + wWidth + 1;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer + wWidth + 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer + wWidth + 1;
	}

	// case 8
	if ( (byteFlag & 0x04) == 0x04 && (byteFlag & 0x08) == 0x08 && !m_bSurfaceContact )
	{
		dwVoxel[nSize++] = id + wWidth - 1;

		if( posPosi.z != 1 )
			dwVoxel[nSize++] = id - nNumLayer + wWidth - 1;
		if( posPosi.z != wDepth )
			dwVoxel[nSize++] = id + nNumLayer + wWidth - 1;
	}

	DWORD dwId;
	DWORD dwTotalVoxel = totalNumVoxel();
	for ( int i=0 ; i<nSize ; i++ )
	{
		dwId = dwVoxel[i];

#ifdef MGIBA_DEBUG
if ( dwId > totalNumVoxel() )
{
	qDebug( "[%d] ID = %ld dwId = %ld (x, y, z) = (%d %d %d)",
		i+1, id, dwId, posPosi.x, posPosi.y, posPosi.z );
}
#endif // MGIBA_DEBUG

		if ( dwId <= dwTotalVoxel && (m_Volume.byteFlag[dwId-1] & 0x40) == 0x40 )
			queue.insert( dwId );
	}

	return true;
} // evaluateSurrVoxel

// private
bool CVolume::setVolumeInfoHeader( KEYWORD key )
{
	// set volume info header structure -- start
	m_Volume.VolumeInfoHeader.viSize = sizeof( VOLUMEINFOHEADER );

	m_Volume.VolumeInfoHeader.viWidth = key.regRegion.width;
	m_Volume.VolumeInfoHeader.viHeight = key.regRegion.height;
	m_Volume.VolumeInfoHeader.viDepth =
		key.regRegion.depth_up - key.regRegion.depth_down + 1;

	// voxel size = element size
	m_Volume.VolumeInfoHeader.fDx = key.fVoxelWidth;
	m_Volume.VolumeInfoHeader.fDy = key.fVoxelHeight;
	m_Volume.VolumeInfoHeader.fDz = key.fVoxelDepth;

	// original voxel size
	m_Volume.VolumeInfoHeader.fOriginDx = key.fVoxelWidth;
	m_Volume.VolumeInfoHeader.fOriginDy = key.fVoxelHeight;
	m_Volume.VolumeInfoHeader.fOriginDz = key.fVoxelDepth;

	// set volume info header structure -- end

	return true;
} // setVolumeInfoHeader

// private
bool CVolume::setVolumeStructFromImage( KEYWORD key )
{
	DWORD dwCount = 0;
	WORD i, j, k;
	BYTE tmpByte;
	QRgb tmpRgb;
	QString m_strFileName;

	// temporary variables
	DWORD dwTmpPosiX, dwTmpPosiY;
	DWORD dwTmpWidth, dwTmpHeight;

	m_nWhiteIsBone = key.bWhiteIsBone;

	m_wMinImageValue = 0;
	m_wMaxImageValue = 255;

	// set segmentation value for each layer
	m_pMinThreshold = new WORD[ volumeDepth() ];
	m_pMaxThreshold = new WORD[ volumeDepth() ];
	for ( i=0 ; i<volumeDepth() ; i++ )
	{
		m_pMinThreshold[i] = ( m_wMaxImageValue - m_wMinImageValue ) / 2;
		m_pMaxThreshold[i] = m_wMaxImageValue;
		if ( !m_nWhiteIsBone )
		{
			m_pMinThreshold[i] = m_wMinImageValue;
			m_pMaxThreshold[i] = ( m_wMaxImageValue - m_wMinImageValue ) / 2;
		}
	}

	m_Volume.wDensity = new WORD[ totalNumVoxel() ];
	m_Volume.byteFlag = new BYTE[ totalNumVoxel() ];
	if( !m_Volume.wDensity || !m_Volume.byteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for VOXEL..!!" );

		return false;
	}

	int numFiles = key.regRegion.depth_up - key.regRegion.depth_down + 1;
	int count = 0;
	QProgressDialog progress( "Importing files...", "Abort", numFiles,
                                g_pMainWindow, "progress", true );
	progress.show();

	for( i=key.regRegion.depth_down-1 ; i<key.regRegion.depth_up ; i++ )
	{
		progress.setProgress( count );
		qApp->processEvents();

		if ( progress.wasCancelled() )
			return false;

		m_strFileName = key.strList[i];
		QImage image;
		
		if ( !image.load( m_strFileName, key.strImageFormat ) ) // file open error..!!
		{
			QMessageBox::critical( g_pMainWindow, "File Open Error",
				"Can't open volume file..." );
			return false;
		}

		dwTmpPosiX = key.regRegion.start_ele.x - 1;
		dwTmpPosiY = key.regRegion.start_ele.y - 1;
		dwTmpWidth = dwTmpPosiX + key.regRegion.width;
		dwTmpHeight = dwTmpPosiY + key.regRegion.height;


	//	for ( WORD k=dwTmpPosiY ; k<dwTmpHeight; k++ )
		for ( k=dwTmpHeight-1 ; k>=dwTmpPosiY ; k-- )
		{
			for ( j=dwTmpPosiX ; j<dwTmpWidth ; j++ )
			{
				tmpRgb = image.pixel( j, k );
				tmpByte = (BYTE) rgbToGray( tmpRgb );

				// set voxel value
				m_Volume.wDensity[dwCount] = tmpByte;

				// set voxel mask on
				if ( m_pMinThreshold[count] <= tmpByte &&
					 m_pMaxThreshold[count] >= tmpByte )
					 m_Volume.wDensity[dwCount] |= 0x8000;

				// initialize visible surface to 0
				// means all surfaces are invisible in initial state.
				m_Volume.byteFlag[dwCount] = 0;

				dwCount++;
			}
		}

		count++;
	}
    progress.setProgress( numFiles );
	progress.hide();

	// set file version
	m_Volume.VolumeFileHeader.vfVersion = MGIBA_100;	// Ver 1.0.0

	// calculate level of density
	// it takes long time.. so comment this
	// calculateNumVoxel();

	return true;
} // setVolumeStructFromImage

bool CVolume::setVolumeStructFromDicom( KEYWORD key )
{
	DWORD dwCount = 0;
	WORD i;	//, j, k;
	WORD tmpWord;
	QString m_strFileName;

	// temporary variables
	WORD dwTmpPosiX, dwTmpPosiY;
	WORD dwTmpWidth, dwTmpHeight;

	m_nWhiteIsBone = key.bWhiteIsBone;

	m_nDicomFlag = true;
	m_wMinImageValue = 65535;
	m_wMaxImageValue = 0;

	m_Volume.wDensity = new WORD[ totalNumVoxel() ];
	m_Volume.byteFlag = new BYTE[ totalNumVoxel() ];
	if( !m_Volume.wDensity || !m_Volume.byteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for VOXEL..!!" );

		return false;
	}

	int numFiles = key.regRegion.depth_up - key.regRegion.depth_down + 1;
	int count = 0;
	QProgressDialog progress( "Importing files...", "Abort", numFiles,
                                g_pMainWindow, "progress", true );
	progress.show();

	for( i=key.regRegion.depth_down-1 ; i<key.regRegion.depth_up ; i++ )
	{
		progress.setProgress( count );
		qApp->processEvents();

		if ( progress.wasCancelled() )
			return false;

		m_strFileName = key.strList[i];

		QImage image;

		CDicomFile df;
		BYTE *pData = NULL;
		int w, h;

		if ( !df.init( m_strFileName ) )
		{
			QMessageBox::critical( g_pMainWindow, "File Open Error",
				"Can't open volume file..." );
			return false;
		}

		w = df.width();
		h = df.height();

		pData = new BYTE[w*h*2];
		if ( !df.data( pData ) )
		{
			QMessageBox::critical( g_pMainWindow, "File Open Error",
				"Can't read data from DICOM image..." );
			return false;
		}

		if ( m_wMaxImageValue < df.maxValue() )
			m_wMaxImageValue = df.maxValue();
		if ( m_wMinImageValue > df.minValue() )
			m_wMinImageValue = df.minValue();

		dwTmpPosiX = key.regRegion.start_ele.x - 1;
		dwTmpPosiY = key.regRegion.start_ele.y - 1;
		dwTmpWidth = dwTmpPosiX + key.regRegion.width;
		dwTmpHeight = dwTmpPosiY + key.regRegion.height;

	//	for ( k=dwTmpPosiY ; k<dwTmpHeight; k++ )
		for ( WORD k=dwTmpHeight-1 ; k>=dwTmpPosiY ; k-- )
		{
			for ( WORD j=dwTmpPosiX ; j<dwTmpWidth ; j++ )
			{
				DWORD idx = w*k + j;
				WORD *pix = (WORD *) &pData[idx*2];
				tmpWord = (WORD) pix[0];

				// set voxel value
				m_Volume.wDensity[dwCount] = tmpWord;

				// initialize visible surface to 0
				// means all surfaces are invisible in initial state.
				m_Volume.byteFlag[dwCount] = 0;

				dwCount++;
			}
		}

		count++;

		if ( pData )
		{
			delete pData;
			pData = NULL;
		}

		df.close();
	}

	// set segmentation value for each layer
	m_pMinThreshold = new WORD[ volumeDepth() ];
	m_pMaxThreshold = new WORD[ volumeDepth() ];

	for ( i=0 ; i<volumeDepth() ; i++ )
	{
		m_pMinThreshold[i] = ( m_wMaxImageValue - m_wMinImageValue ) / 2;
		m_pMaxThreshold[i] = m_wMaxImageValue;
		if ( !m_nWhiteIsBone )
		{
			m_pMinThreshold[i] = m_wMinImageValue;
			m_pMaxThreshold[i] = ( m_wMaxImageValue - m_wMinImageValue ) / 2;
		}
	}

	// set voxel mask on
	dwCount = 0;
	count = 0;
	for ( i=0 ; i<volumeDepth() ; i++ )
	{
		for ( WORD j=0 ; j<volumeWidth() ; j++ )
		{
			for ( WORD k=0 ; k<volumeHeight() ; k++ )
			{
				// set voxel mask on
				tmpWord = m_Volume.wDensity[dwCount];
				if ( m_pMinThreshold[count] <= tmpWord &&
					 m_pMaxThreshold[count] >= tmpWord )
					m_Volume.wDensity[dwCount] |= 0x8000;

				dwCount++;
			}
		}

		count++;
	}

	// set file version
	m_Volume.VolumeFileHeader.vfVersion = MGIBA_100;	// Ver 1.0.0

	// calculate level of density
	// it takes long time.. so comment this
	// calculateNumVoxel();


	// set progress to 100%
	progress.setProgress( numFiles );
	progress.hide();

	return true;
} // setVolumeStructFromDicom

// private
bool CVolume::writeToSTLAsciiFormat( QTextStream &ts, SRF_NODE node[4], int face )
{
	/*
	float fVectorA[3];
	float fVectorB[3];
	float fVectorN[3];
	float fXc, fYc, fZc;		// center of gravity for each surface

	// for triangle 1
	fXc = ( node[0].x + node[1].x + node[2].x ) / 3.0;
	fYc = ( node[0].y + node[1].y + node[2].y ) / 3.0;
	fZc = ( node[0].z + node[1].z + node[2].z ) / 3.0;

	fVectorA[0] = node[0].x - fXc;
	fVectorA[1] = node[0].y - fYc;
	fVectorA[2] = node[0].z - fZc;

	fVectorB[0] = node[1].x - fXc;
	fVectorB[1] = node[1].y - fYc;
	fVectorB[2] = node[1].z - fZc;

	fVectorN[0] = fVectorA[1]*fVectorB[2] - fVectorA[2]*fVectorB[1];
	fVectorN[1] = -1*fVectorA[0]*fVectorB[2] + fVectorA[2]*fVectorB[0];
	fVectorN[2] = fVectorA[0]*fVectorB[1] - fVectorA[1]*fVectorB[0];

	for ( int i=0 ; i<3 ; i++ )
		if ( fVectorN[i] < 1e-7 ) fVectorN[i] = 0.0;
	*/
	float fVectorN[3];

	if ( face == 1 )
	{
		fVectorN[0] = 0;
		fVectorN[1] = 0;
		fVectorN[2] = -1;
	}
	else if ( face == 2 )
	{
		fVectorN[0] = 0;
		fVectorN[1] = 0;
		fVectorN[2] = 1;
	}
	else if ( face == 3 )
	{
		fVectorN[0] = 0;
		fVectorN[1] = -1;
		fVectorN[2] = 0;
	}
	else if ( face == 4 )
	{
		fVectorN[0] = 0;
		fVectorN[1] = 1;
		fVectorN[2] = 0;
	}
	else if ( face == 5 )
	{
		fVectorN[0] = 1;
		fVectorN[1] = 0;
		fVectorN[2] = 0;
	}
	else if ( face == 6 )
	{
		fVectorN[0] = -1;
		fVectorN[1] = 0;
		fVectorN[2] = 0;
	}

	ts << "  facet normal"
		<< " " << fVectorN[0]
		<< " " << fVectorN[1]
		<< " " << fVectorN[2] << endl;
	ts << "    outer loop" << endl;
	ts << "      vertex"
		<< " " << node[0].x
		<< " " << node[0].y
		<< " " << node[0].z << endl;
	ts << "      vertex"
		<< " " << node[1].x
		<< " " << node[1].y
		<< " " << node[1].z << endl;
	ts << "      vertex"
		<< " " << node[2].x
		<< " " << node[2].y
		<< " " << node[2].z << endl;
	ts << "    endloop" << endl;
	ts << "  endfacet" << endl;

	/*
	// for trangle 2
	fXc = ( node[0].x + node[2].x + node[3].x ) / 3.0;
	fYc = ( node[0].y + node[2].y + node[3].y ) / 3.0;
	fZc = ( node[0].z + node[2].z + node[3].z ) / 3.0;

	fVectorA[0] = node[0].x - fXc;
	fVectorA[1] = node[0].y - fYc;
	fVectorA[2] = node[0].z - fZc;

	fVectorB[0] = node[2].x - fXc;
	fVectorB[1] = node[2].y - fYc;
	fVectorB[2] = node[2].z - fZc;

	fVectorN[0] = fVectorA[1]*fVectorB[2] - fVectorA[2]*fVectorB[1];
	fVectorN[1] = -1*fVectorA[0]*fVectorB[2] + fVectorA[2]*fVectorB[0];
	fVectorN[2] = fVectorA[0]*fVectorB[1] - fVectorA[1]*fVectorB[0];

	for(int j=0 ; i<3 ; i++)
		if( fVectorN[j] < 1e-7 ) fVectorN[j] = 0.0;
	*/

	ts << "  facet normal"
		<< " " << fVectorN[0]
		<< " " << fVectorN[1]
		<< " " << fVectorN[2] << endl;
	ts << "    outer loop" << endl;
	ts << "      vertex"
		<< " " << node[0].x
		<< " " << node[0].y
		<< " " << node[0].z << endl;
	ts << "      vertex"
		<< " " << node[2].x
		<< " " << node[2].y
		<< " " << node[2].z << endl;
	ts << "      vertex"
		<< " " << node[3].x
		<< " " << node[3].y
		<< " " << node[3].z << endl;
	ts << "    endloop" << endl;
	ts << "  endfacet" << endl;

	return true;
} // writeToSTLAsciiFormat

// private
bool CVolume::writeToSTLBinaryFormat( QDataStream &ds, SRF_NODE node[4], int face )
{
	return true;
} // writeToSTLBinaryFormat

// private
bool CVolume::writeToAcadScriptFormat( QTextStream &ts, SRF_NODE node[4], WORD value )
{
//	static bool bFlag = true;

	float fCap;
	float fTvalue;

	fCap = 254;	//XXX

	/*
	if ( bFlag )
	{
		ts << "color" << endl;
		bFlag = false;
	}
	else
		ts << " color" << endl;
	*/
	ts << "color" << endl;
	
	fTvalue = value / fCap * 16;
	if( fmod( fTvalue, 1 ) < 0.5 )
		fTvalue = floor( fTvalue );
	else
		fTvalue = ceil( fTvalue );

	fTvalue = (fTvalue + 1) * 10;

	ts << fTvalue << endl;
	ts << "3dface" << endl;

	for ( int i=0 ; i<4 ; i++ )
	{
		ts << node[i].x << "," << node[i].y << "," << node[i].z << endl;
	}
	ts << endl;

	return true;
} // writeToAcadScriptFormat

// private
bool CVolume::writeToAcadDxfFormat( QTextStream &ts, SRF_NODE node[4] )
{
	ts << "0" << endl;
	ts << "3DFACE" << endl;
	ts << "8" << endl;
	ts << "0" << endl;

	ts <<"10" << endl;
	ts << node[0].x << endl;
	ts << "20" << endl;
	ts << node[0].y << endl;
	ts << "30" << endl;
	ts << node[0].z << endl;

	ts <<"11" << endl;
	ts << node[1].x << endl;
	ts << "21" << endl;
	ts << node[1].y << endl;
	ts << "31" << endl;
	ts << node[1].z << endl;

	ts <<"12" << endl;
	ts << node[2].x << endl;
	ts << "22" << endl;
	ts << node[2].y << endl;
	ts << "32" << endl;
	ts << node[2].z << endl;

	ts <<"13" << endl;
	ts << node[3].x << endl;
	ts << "23" << endl;
	ts << node[3].y << endl;
	ts << "33" << endl;
	ts << node[3].z << endl;

	return true;
} // writeToAcadDxfFormat


/*******************/
/* static function */
/*******************/

// static
DWORD CVolume::floatToDWORD( float value_in )
{
	DWORD new_value;
	union
	{
		float float_value;
		char  char_value[4];
	} value;

	value.float_value = value_in;

	new_value  = value.char_value[0] & 0xFF;
	new_value |= (value.char_value[1] & 0xFF) << 0x08;
	new_value |= (value.char_value[2] & 0xFF) << 0x10;
	new_value |= (value.char_value[3] & 0xFF) << 0x18;

	return new_value;
} // floatToDWORD

// static
float CVolume::rgbToGray( QRgb rgb )
{
// qDebug( "%d %d %d", qRed(rgb), qGreen(rgb), qBlue(rgb) );
	return (float) ( 0.299*qRed(rgb) + 0.587*qGreen(rgb) + 0.114*qBlue(rgb) );
} // rgbToGray

/*
// static
int CVolume::checkFileExistence( char string[256] )
{
	struct _stat buf;

	if( _stat( string, &buf) == -1 ) return false;
	else return true;
} // checkFileExistence
*/

/*
// static
bool CVolume::storeToBMP( unsigned char *image, int width, int height, char *filename )
{
	QImage img( width, height, 32 );

	int t;
	int gray;
	for ( int i=0 ; i<height ; i++ )
	{
		for ( int j=0 ; j<width ; j++ )
		{
			t = i*width + j;
			gray = (int) image[t];
			img.setPixel( i, j, qRgb( gray, gray, gray ) );
		}
	}

	if ( img.save( filename, "BMP" ) == false )
		return false;

	return true;
} // storeToBMP
*/
