#include "VolumeInfoDlg.h"
#include "VolumeFile.h"
#include "MultiLineEditDlg.h"

#include <qlabel.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

CVolumeInfoDlg::CVolumeInfoDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CVolumeInfoDlg_base( parent, name, modal, f )
{
	connect( IDC_BTN_OK, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
	connect( IDC_BTN_SAVEAS, SIGNAL( clicked() ), this, SLOT( slotSaveAs() ) );
} // CVolumeInfoDlg

CVolumeInfoDlg::~CVolumeInfoDlg()
{
} // ~CVolumeInfoDlg

bool CVolumeInfoDlg::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	QString str;

	// volume size
	str.sprintf( "%d", m_pVolume->volumeWidth() );
	IDC_LBL_WIDTH->setText( str );

	str.sprintf( "%d", m_pVolume->volumeHeight() );
	IDC_LBL_HEIGHT->setText( str );

	str.sprintf( "%d", m_pVolume->volumeDepth() );
	IDC_LBL_DEPTH->setText( str );

	// volume information
	str.sprintf( "%f", m_pVolume->volumeVolume() );
	IDC_LBL_TOTAL_VOLUME->setText( str );

	str.sprintf( "%f", m_pVolume->volumeSurfaceArea() );
	IDC_LBL_TOTAL_SURFACE->setText( str );

	str.sprintf( "%ld", m_pVolume->totalNumVoxel() );
	IDC_LBL_TOTAL_VOXEL->setText( str );

	str.sprintf( "%f", m_pVolume->feaVolume() );
	IDC_LBL_VISIBLE_VOLUME->setText( str );

	str.sprintf( "%f", m_pVolume->feaSurfaceArea() );
	IDC_LBL_VISIBLE_SURFACE->setText( str );

	str.sprintf( "%ld", m_pVolume->numCurrentFeaVoxel() );
	IDC_LBL_VISIBLE_VOXEL->setText( str );

	// fragment information
	DWORD count = m_pVolume->numCurrentBone();
	str.sprintf( "%ld", count );
	IDC_LBL_FRAGMENT->setText( str );

	DWORD filter = m_pVolume->numFilteredBone();
	str.sprintf( "%ld", filter );
	IDC_LBL_FILTERED->setText( str );

	DWORD remain = count - filter;
	str.sprintf( "%ld", remain );
	IDC_LBL_REMAINED->setText( str );

	return true;
} // init

// slot
void CVolumeInfoDlg::slotOk()
{
	QDialog::accept();
} // slotOk

// slot
void CVolumeInfoDlg::slotSaveAs()
{
	QString fn = QFileDialog::getSaveFileName( NULL, "Volume Info File (*.inf)", this );
    if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".inf", 0 ) != -1 || fn.find( ".INF", 0 ) != -1 )
			;
		else
			fn.append( ".inf" );

		QFile infoFile( fn );
		if ( !infoFile.open( IO_WriteOnly ) )	// file open error..!!
		{
			QMessageBox::critical( this, "File Open Error",
				"Can't open temp file for volume information..." );
			return;
		}

		QTextStream ts( &infoFile );			// use a text stream

		ts << "*****************************************************************************" << endl;
		ts << "*                                                                           *" << endl;
		ts << "*                              MGIBA 1.0.0                                  *" << endl;
		ts << "*                                                                           *" << endl;
		ts << "*                       Bio Medical Soft(c) 2000-2002                       *" << endl;
		ts << "*                                                                           *" << endl;
		ts << "*****************************************************************************" << endl << endl << endl;

		ts << "# Volume Size" << endl;
		ts << "  Width  = " << IDC_LBL_WIDTH->text() << endl;
		ts << "  Height = " << IDC_LBL_HEIGHT->text() << endl;
		ts << "  Depth  = " << IDC_LBL_DEPTH->text() << endl << endl << endl;

		ts << "# Volume Information" << endl;
		ts << "  Total volume            = " << IDC_LBL_TOTAL_VOLUME->text() << endl;
		ts << "  Total surface area      = " << IDC_LBL_TOTAL_SURFACE->text() << endl;
		ts << "  Total number of voxel   = " << IDC_LBL_TOTAL_VOXEL->text() << endl << endl;

		ts << "  Visible volume          = " << IDC_LBL_VISIBLE_VOLUME->text() << endl;
		ts << "  Visible surface area    = " << IDC_LBL_VISIBLE_SURFACE->text() << endl;
		ts << "  Number of visible voxel = " << IDC_LBL_VISIBLE_VOXEL->text() << endl << endl << endl;

		ts << "# Fragmentation Information" << endl;
		ts << "  Number of tatal fragment    = " << IDC_LBL_FRAGMENT->text() << endl;
		ts << "  Number of filtered fragment = " << IDC_LBL_FILTERED->text() << endl;
		ts << "  Number of remained fragment = " << IDC_LBL_REMAINED->text() << endl;

		infoFile.close();


		WFlags f = Qt::WType_TopLevel | Qt::WStyle_MinMax;
#ifdef Q_WS_WIN
		f |= Qt::WStyle_SysMenu;
#endif
		
		CMultiLineEditDlg *pDlg = new CMultiLineEditDlg( this, NULL, true, f );
		if ( pDlg->init( fn, "Volume Information", false ) )
			pDlg->show();
	}
} // slotSaveAs