#include "vtkQtRenderWindow.h"

vtkQtRenderWindow::vtkQtRenderWindow( QGLFormat glf, QWidget * parent, const char * name )
  : QGLWidget(glf, parent, name)
{
	init();
}

vtkQtRenderWindow::vtkQtRenderWindow( QWidget * parent, const char * name )
  : QGLWidget(parent, name )
{
	init();
} // init

void vtkQtRenderWindow::init() 
{
	m_bInitNeeded = true;
	m_pVtkRenderWindow = vtkRenderWindow::New();
	m_pVtkRenderer = vtkRenderer::New();
	m_pVtkRenderWindow->AddRenderer( m_pVtkRenderer );

	m_pVtkLight = vtkLight::New();
	m_pVtkRenderer->AddLight( m_pVtkLight );

#ifdef QT_DOUBLEBUFFER
	// use the double buffering of QT
	m_pVtkRenderWindow->SetSwapBuffers( 0 );
#else
	// use the double buffering of vtk
	setAutoBufferSwap( false );
#endif
} // initRenderWindow

vtkQtRenderWindow::~vtkQtRenderWindow()
{
	if ( m_pVtkRenderer )
	{
		m_pVtkRenderer->Delete();
		m_pVtkRenderer = NULL;
	}
	if ( m_pVtkRenderWindow )
	{
		m_pVtkRenderWindow->Delete();
		m_pVtkRenderWindow = NULL;
	}
} // ~vtkQtRenderWindow

vtkRenderer *vtkQtRenderWindow::getRenderer()
{
	if ( m_pVtkRenderer )
		return m_pVtkRenderer;
	else
		return NULL;
} // getRenderer

vtkRenderWindow *vtkQtRenderWindow::getRenderWindow()
{
   if ( m_pVtkRenderWindow )
	   return m_pVtkRenderWindow;
   else
	   return NULL;
} // getRenderWindow;

void vtkQtRenderWindow::initializeGL()
{
	glClearColor( 1.0, 1.0, 1.0, 1.0 );
	glColor3f(1.0, 0.0, 0.0 );
} // initializeGL

void vtkQtRenderWindow::resizeGL( int w, int h )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluOrtho2D( 0.0, w, 0.0, h );
	glMatrixMode( GL_MODELVIEW );

	updateGL();
} // resizeGL

void vtkQtRenderWindow::paintGL()
{
	if ( m_bInitNeeded)
	{
		WId nId = winId();
		m_pVtkRenderWindow->SetWindowId( (void*) nId );
		m_bInitNeeded = false;
	}
	else
	{
		m_pVtkRenderWindow->Render();
	}
} // paingGL