#include "vtkQtRenderWindowInteractor.h"

#include <vtkMath.h>
#include <vtkLightCollection.h>
#include <vtkSphereSource.h>
#include <vtkPolyDataMapper.h>
#include <vtkMath.h>

vtkQtRenderWindowInteractor::vtkQtRenderWindowInteractor( QWidget *parent, const char *name )
	: vtkQtRenderWindow( parent, name )
{
	init();
} // vtkQtRenderWindowInteractor

vtkQtRenderWindowInteractor::vtkQtRenderWindowInteractor( QGLFormat glf, QWidget *parent, const char *name )
	: vtkQtRenderWindow( glf, parent, name )
{
	init();
} // vtkQtRenderWindowInteractor

vtkQtRenderWindowInteractor::~vtkQtRenderWindowInteractor()
{
   delete m_StepTimer;
} // ~vtkQtRenderWindowInteractor

void vtkQtRenderWindowInteractor::init()
{
	/*
	setLeftButtonFunction( SLOT( cameraRotate() ) );
	setMiddleButtonFunction( SLOT( cameraTranslate() ) );
	setRightButtonFunction( SLOT( cameraZoom() ) );
	setShiftLeftButtonFunction( SLOT( cameraRoll() ) );
	*/
	setLeftButtonFunction( SLOT( cameraZoom() ) );
	setMiddleButtonFunction( SLOT( cameraTranslate() ) );
	setRightButtonFunction( SLOT( cameraRotate() ) );

	m_bMouseEventActive = false;
	setInteractionTime( 50 );
	setSensitivity( 10.0 );

	m_StepTimer = new QTimer();
	CHECK_PTR( m_StepTimer );

	m_pVtkCurrentCamera = getRenderer()->GetActiveCamera();
	CHECK_PTR( m_pVtkCurrentCamera );

	vtkLightCollection *lightCollection = getRenderer()->GetLights();
	lightCollection->InitTraversal();
	m_pVtkCurrentLight = lightCollection->GetNextItem();
	CHECK_PTR( m_pVtkCurrentLight );

	m_pVtkInteractionActor = NULL;
	m_pVtkCollisionActor = NULL;
} // init

const char *vtkQtRenderWindowInteractor::getClassName()
{
	return ( "RenderWindowInteractor" );
} // getClassName

void vtkQtRenderWindowInteractor::setPlane( float *ebene )
{
	m_fNormale[0] = ebene[0];
	m_fNormale[1] = ebene[1];
	m_fNormale[2] = ebene[2];
	m_fEbenekonstante = ebene[3];
} // setPlane

void vtkQtRenderWindowInteractor::setSensitivity( float t )
{
	m_fTrackballFactor = t;
} // setSensitivity

float vtkQtRenderWindowInteractor::getSensitivity()
{
   return m_fTrackballFactor;
} // getSensitivity

void vtkQtRenderWindowInteractor::setLeftButtonFunction( const char* function )
{
	if ( function == NULL )
	{
		strncpy( m_chLeftButtonFunction, SLOT( doNothing() ), 100 );
	}
	else
	{
		strncpy( m_chLeftButtonFunction, function, 100 );
	}
} // setLeftButtonFunction

void vtkQtRenderWindowInteractor::setMiddleButtonFunction( const char* function )
{
	if ( function == NULL )
	{
		strncpy( m_chMiddleButtonFunction, SLOT( doNothing() ), 100 );
	}
	else
	{
		strncpy( m_chMiddleButtonFunction, function, 100 );
	}
} // setMiddleButtonFunction

void vtkQtRenderWindowInteractor::setRightButtonFunction( const char* function )
{
	if ( function == NULL )
	{
		strncpy( m_chRightButtonFunction, SLOT( doNothing() ), 100 );
	}
	else
	{
		strncpy( m_chRightButtonFunction, function, 100 );
	}
} // setRightButtonFunction

void vtkQtRenderWindowInteractor::setShiftLeftButtonFunction( const char* function ) 
{
	if ( function == NULL )
	{
		strncpy( m_chShiftLeftButtonFunction, SLOT( doNothing() ), 100 );
	}
	else
	{
		strncpy( m_chShiftLeftButtonFunction, function, 100 );
	}
} // setShiftLeftButtonFunction

void vtkQtRenderWindowInteractor::setInteractionActor( vtkActor* actor )
{
	m_pVtkInteractionActor = actor;
} // setInteractionActor

vtkActor *vtkQtRenderWindowInteractor::getInteractionActor( void )
{
	if ( m_pVtkInteractionActor )
		return m_pVtkInteractionActor;
	else
		return NULL;
} // getInteractionActor

void vtkQtRenderWindowInteractor::setActorRotation( float x, float y, float z )
{
	ASSERT( m_pVtkInteractionActor != NULL );

	m_pVtkInteractionActor->SetOrigin( x, y, z );
} // setActorRotation

void vtkQtRenderWindowInteractor::mousePressEvent( QMouseEvent *event )
{
	if ( m_bMouseEventActive )
	{
		return;
	}

	if ( event->button() & LeftButton )
	{
		if ( event->state() & ShiftButton )
		{
			// left mouse mouse and shift button
			m_bMouseEventActive = true;
			QObject::connect( m_StepTimer, SIGNAL( timeout() ), this, m_chShiftLeftButtonFunction );
		}
		else
		{
			// left mouse button without shift button
			m_bMouseEventActive = true;
			QObject::connect( m_StepTimer, SIGNAL( timeout() ), this, m_chLeftButtonFunction );
		}
	}
	else if ( event->button() & MidButton )
	{
		// middle mouse button
		m_bMouseEventActive = true;
		QObject::connect( m_StepTimer, SIGNAL( timeout() ), this, m_chMiddleButtonFunction );
	}
	else if ( event->button() & RightButton )
	{
		// right mouse button
		m_bMouseEventActive = true;
		QObject::connect( m_StepTimer, SIGNAL( timeout() ), this, m_chRightButtonFunction );
	}

	if ( m_bMouseEventActive ) 
	{
		m_nCurrentMousePos[0] = event->x();
		m_nCurrentMousePos[1] = event->y();
		m_nLastMousePos[0] = event->x();
		m_nLastMousePos[1] = event->y();
		m_bFirstCall = true;
		m_StepTimer->start( m_nInteractionTime );
	}
} // mousePressEvent

void vtkQtRenderWindowInteractor::mouseReleaseEvent( QMouseEvent *event )
{
	if ( m_bMouseEventActive )
	{
		m_StepTimer->stop();
		m_bMouseEventActive = false;
		QObject::disconnect( m_StepTimer, 0, this, 0 );
	}
	
	emit( released( event->x(), this->height() - event->y() ) );
} // mouseReleaseEvent

void vtkQtRenderWindowInteractor::mouseMoveEvent( QMouseEvent *event )
{
	m_nCurrentMousePos[0] = event->x();
	m_nCurrentMousePos[1] = event->y();
} // mouseMoveEvent

void vtkQtRenderWindowInteractor::setInteractionTime( int time )
{
	m_nInteractionTime = time;
} // setInteractionTime

int vtkQtRenderWindowInteractor::getInteractionTime( void )
{
	return m_nInteractionTime;
} // getInteractionTime

void vtkQtRenderWindowInteractor::actorRotate()
{
	float xf;
	float yf;

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
	     m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	// basic idea: the change in x of the mouse is used to rotate around the
	// view-up vector. the change in y of the mouse is used to rotate around the
	// cross of the viewplane-normal and the view-up vector,

	// first time we do some preprocessing
	if ( m_bFirstCall )
	{
		int *size = getRenderWindow()->GetSize();
		float *vp = getRenderer()->GetViewport();

		m_fDeltaAzimuth = -20.0/((vp[2] - vp[0])*size[0]);
		m_fDeltaElevation = -20.0/((vp[3] - vp[1])*size[1]);

		// get the rotation axis: cross of view-up and the normal in the viewplane
		m_pVtkCurrentCamera->GetViewUp( m_fViewUp );
		m_pVtkCurrentCamera->GetViewPlaneNormal( m_fVpNormal );
		vtkMath::Cross( m_fVpNormal, m_fViewUp, m_fRotateY );

		m_bFirstCall = false;
	}

	// how much to rotate
	xf = (m_nCurrentMousePos[0] - m_nLastMousePos[0]) * m_fTrackballFactor * m_fDeltaAzimuth * (-1);
	yf = (m_nCurrentMousePos[1] - m_nLastMousePos[1]) * m_fTrackballFactor * m_fDeltaElevation;

	m_pVtkInteractionActor->RotateWXYZ( xf, m_fViewUp[0], m_fViewUp[1], m_fViewUp[2] );
	m_pVtkInteractionActor->RotateWXYZ( yf, m_fRotateY[0], m_fRotateY[1], m_fRotateY[2] );

	emit actorRotated( xf, yf );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
} // actorRotate

void vtkQtRenderWindowInteractor::actorTranslate()
{
	float *center;
	float dispCenter[4];
	float newP[4];
	float oldP[4];

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	center = m_pVtkInteractionActor->GetCenter();
	WorldToDisplay( center[0], center[1], center[2], dispCenter );

	DisplayToWorld( float( m_nCurrentMousePos[0] ), float( height() - m_nCurrentMousePos[1] ),
                         dispCenter[2], newP );

	DisplayToWorld( float( m_nLastMousePos[0] ), float( height() - m_nLastMousePos[1] ),
                         dispCenter[2], oldP);

	m_pVtkInteractionActor->AddPosition( newP[0]-oldP[0], newP[1]-oldP[1], newP[2]-oldP[2] );
   
	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::actorZoom()
{
	float yf;

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	   // first time we do some preprocessing
	if( m_bFirstCall )
	{
		m_pVtkCurrentCamera->GetViewPlaneNormal( m_fVpNormal );
		m_bFirstCall = false;
	}

	yf = float( m_nLastMousePos[1] - m_nCurrentMousePos[1] ) / float( height() ) * m_fTrackballFactor * -10.0;

   m_pVtkInteractionActor->AddPosition( m_fVpNormal[0] * yf, m_fVpNormal[1] * yf, m_fVpNormal[2] * yf );

   // update
   updateGL();

   // save mouse pos
   m_nLastMousePos[0] = m_nCurrentMousePos[0];
   m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::actorRoll()
{
	float angle;

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	// first time we do some preprocessing
	if( m_bFirstCall )
	{
		m_pVtkCurrentCamera->GetViewPlaneNormal( m_fVpNormal );
		m_fRenCenter[0] = getRenderer()->GetCenter()[0];
	    m_fRenCenter[1] = getRenderer()->GetCenter()[1];
		m_fRenCenter[2] = getRenderer()->GetCenter()[2];
		m_bFirstCall = false;
	}

	int diffX1 = m_nCurrentMousePos[0] - int( m_fRenCenter[0] );
	int diffY1 = m_nCurrentMousePos[1] - int( m_fRenCenter[1] );
	int diffX2 = m_nLastMousePos[0] - int( m_fRenCenter[0] );
	int diffY2 = m_nLastMousePos[1] - int( m_fRenCenter[1] );

	double a1 = atan2( double(diffY1), double(diffX1) );
	double a2 = atan2( double(diffY2), double(diffX2) );
	angle = (a2 - a1) / (2.0 * 3.1415926535) * 360.0 / 10.0 * m_fTrackballFactor;
	m_pVtkInteractionActor->RotateWXYZ( angle, m_fVpNormal[0], m_fVpNormal[1],  m_fVpNormal[2] );

	emit actorRolled( angle );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::cameraRotate()
{
	float xf;
	float yf;

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	// first time we do some preprocessing
	if( m_bFirstCall )
	{
		int *size = getRenderWindow()->GetSize();
		float *vp = getRenderer()->GetViewport();

		m_fDeltaAzimuth = -20.0/((vp[2] - vp[0])*size[0]);
		m_fDeltaElevation = -20.0/((vp[3] - vp[1])*size[1]);
		m_bFirstCall = false;
	}

	xf = (m_nCurrentMousePos[0] - m_nLastMousePos[0]) * m_fDeltaAzimuth * m_fTrackballFactor;
	yf = (m_nLastMousePos[1] - m_nCurrentMousePos[1]) * m_fDeltaElevation * m_fTrackballFactor;

	m_pVtkCurrentCamera->Azimuth( xf );
	m_pVtkCurrentCamera->Elevation( yf );
	m_pVtkCurrentCamera->OrthogonalizeViewUp();
	getRenderer()->ResetCameraClippingRange();

	// light follows the camera
	m_pVtkCurrentLight->SetPosition( m_pVtkCurrentCamera->GetPosition() );
	m_pVtkCurrentLight->SetFocalPoint( m_pVtkCurrentCamera->GetFocalPoint() );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::cameraTranslate()
{
	float newP[4];
	float viewFocus[4];
	float viewPoint[3];
	double motionVector[3];

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	m_pVtkCurrentCamera->GetFocalPoint( viewFocus );
	WorldToDisplay( viewFocus[0], viewFocus[1], viewFocus[2], viewFocus );
	double focalDepth = viewFocus[2];

	DisplayToWorld( float( width()/2.0 + (m_nCurrentMousePos[0] - m_nLastMousePos[0]) ),
                         float( height()/2.0 - (m_nCurrentMousePos[1] - m_nLastMousePos[1])),
                         focalDepth, newP );

	m_pVtkCurrentCamera->GetFocalPoint( viewFocus );
	m_pVtkCurrentCamera->GetPosition( viewPoint );

	motionVector[0] = m_fTrackballFactor / 10.0 * ( viewFocus[0] - newP[0] );
	motionVector[1] = m_fTrackballFactor / 10.0 * ( viewFocus[1] - newP[1] );
	motionVector[2] = m_fTrackballFactor / 10.0 * ( viewFocus[2] - newP[2] );

	m_pVtkCurrentCamera->SetFocalPoint( motionVector[0] + viewFocus[0],
                                       motionVector[1] + viewFocus[1],
                                       motionVector[2] + viewFocus[2]);
	m_pVtkCurrentCamera->SetPosition( motionVector[0] + viewPoint[0],
                                     motionVector[1] + viewPoint[1],
                                     motionVector[2] + viewPoint[2]);

	// light follows the camera
	m_pVtkCurrentLight->SetPosition( m_pVtkCurrentCamera->GetPosition() );
	m_pVtkCurrentLight->SetFocalPoint( m_pVtkCurrentCamera->GetFocalPoint() );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::cameraZoom()
{
	float yf;

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}
   
	yf = float( m_nCurrentMousePos[1] - m_nLastMousePos[1] ) / float( height() ) * m_fTrackballFactor;

	// if yf < 0, we need to be between 0..1
	yf = pow( 1.1, yf );

	m_pVtkCurrentCamera->Dolly( yf );
	getRenderer()->ResetCameraClippingRange();

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::cameraRoll()
{
	float angle;
	
	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	// first time we do some preprocessing
	if ( m_bFirstCall )
	{
		m_fRenCenter[0] = getRenderer()->GetCenter()[0];
		m_fRenCenter[1] = getRenderer()->GetCenter()[1];
		m_fRenCenter[2] = getRenderer()->GetCenter()[2];
		m_bFirstCall = false;
	}

	// calculate the angle in radians and roll the camera
	int diffX1 = m_nCurrentMousePos[0] - int(m_fRenCenter[0]);
	int diffY1 = m_nCurrentMousePos[1] - int(m_fRenCenter[1]);
	int diffX2 = m_nLastMousePos[0] - int(m_fRenCenter[0]);
	int diffY2 = m_nLastMousePos[1] - int(m_fRenCenter[1]);

	double a1 = atan2( double(diffY1), double(diffX1) );
	double a2 = atan2( double(diffY2), double(diffX2) );
	angle = (a2 - a1) / (2.0 * 3.1415926535) * 360.0 / 10.0 * m_fTrackballFactor;

	m_pVtkCurrentCamera->Roll( angle );

	// light follows the camera
	m_pVtkCurrentLight->SetPosition( m_pVtkCurrentCamera->GetPosition() );
	m_pVtkCurrentLight->SetFocalPoint( m_pVtkCurrentCamera->GetFocalPoint() );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::actorPlaneMove() 
{
	float *center;
	float dispCenter[4];
	float newP[4];
	float oldP[4];
	float p0[3],p1[3],t;
	float tkonstante = m_fNormale[0] * m_fNormale[0] + m_fNormale[1] * m_fNormale[1] + m_fNormale[2] * m_fNormale[2];

	// do nothing if mouse is still on the same pos
	if ( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
		 m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	center = m_pVtkInteractionActor->GetCenter();
	WorldToDisplay( center[0], center[1], center[2], dispCenter );

	DisplayToWorld( float( m_nCurrentMousePos[0]), float( height() - m_nCurrentMousePos[1] ),
                         dispCenter[2], newP );

	DisplayToWorld( float( m_nLastMousePos[0]), float( height() - m_nLastMousePos[1]),
                         dispCenter[2], oldP );

	oldP[3] = oldP[0] * m_fNormale[0] + oldP[1] * m_fNormale[1] + oldP[2] * m_fNormale[2];

	t = ( m_fEbenekonstante - oldP[3] ) / tkonstante;

	p0[0] = oldP[0] + t*m_fNormale[0];
	p0[1] = oldP[1] + t*m_fNormale[1];
	p0[2] = oldP[2] + t*m_fNormale[2];

	newP[3] = newP[0]*(this->m_fNormale[0]) +
              newP[1]*(this->m_fNormale[1]) +
              newP[2]*(this->m_fNormale[2]);

	t = (m_fEbenekonstante - newP[3])/tkonstante;

	p1[0] = newP[0] + t*(this->m_fNormale[0]);
	p1[1] = newP[1] + t*(this->m_fNormale[1]);
	p1[2] = newP[2] + t*(this->m_fNormale[2]);

	m_pVtkInteractionActor->AddPosition( p1[0]-p0[0], p1[1]-p0[1], p1[2]-p0[2] );
	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
}

void vtkQtRenderWindowInteractor::actorPlaneRoll() 
{
	float angle;

	// do nothing if mouse is still on the same pos
	if( m_nCurrentMousePos[0] == m_nLastMousePos[0] &&
        m_nCurrentMousePos[1] == m_nLastMousePos[1] )
	{
		return;
	}

	// first time we do some preprocessing
	if ( m_bFirstCall )
	{
		m_fRenCenter[0] = getRenderer()->GetCenter()[0];
		m_fRenCenter[1] = getRenderer()->GetCenter()[1];
		m_fRenCenter[2] = getRenderer()->GetCenter()[2];
		m_bFirstCall = false;
	}

	int diffX1 = this->m_nCurrentMousePos[0] - int(m_fRenCenter[0]);
	int diffY1 = this->m_nCurrentMousePos[1] - int(m_fRenCenter[1]);
	int diffX2 = this->m_nLastMousePos[0] - int(m_fRenCenter[0]);
	int diffY2 = this->m_nLastMousePos[1] - int(m_fRenCenter[1]);

	double a1 = atan2( double(diffY1), double(diffX1) );
	double a2 = atan2( double(diffY2), double(diffX2) );
	angle = (a2 - a1) / (2.0 * 3.1415926535) * 360.0 / 10.0 * m_fTrackballFactor;
	m_pVtkInteractionActor->RotateWXYZ( -angle, m_fNormale[0], m_fNormale[1], m_fNormale[2] );

	// update
	updateGL();

	// save mouse pos
	m_nLastMousePos[0] = m_nCurrentMousePos[0];
	m_nLastMousePos[1] = m_nCurrentMousePos[1];
} // actorPlaneRoll

void vtkQtRenderWindowInteractor::doNothing()
{
} // doNothing

void vtkQtRenderWindowInteractor::DisplayToWorld( float x, float y, float z, float *worldPoint )
{
	getRenderer()->SetDisplayPoint( x, y, z );
	getRenderer()->DisplayToWorld();
	getRenderer()->GetWorldPoint( worldPoint );

	// normalize
	if ( worldPoint[3] )
	{
		worldPoint[0] = worldPoint[0] / worldPoint[3];
		worldPoint[1] = worldPoint[1] / worldPoint[3];
		worldPoint[2] = worldPoint[2] / worldPoint[3];
		worldPoint[3] = 1.0;
	}
} // DisplayToWorld

void vtkQtRenderWindowInteractor::WorldToDisplay( float x, float y, float z, float *displayPoint )
{
	getRenderer()->SetWorldPoint(x, y, z, 1.0 );
	getRenderer()->WorldToDisplay();
	getRenderer()->GetDisplayPoint( displayPoint );
} // WorldToDisplay
