/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _GLVIEW_H_
#define _GLVIEW_H_

#include "global_define.h"
#include "define.h"
#include <qgl.h>
#include <qpoint.h>
#include <qpixmap.h>


class CVolume;


class CGLView : public QGLWidget
{
    Q_OBJECT

public:
    CGLView( QWidget *parent=0, const char *name=0, const QGLWidget *shareWidget = 0, WFlags f=0 );
    ~CGLView();

	bool init( CVolume *pVolume );

	void setVolume( CVolume *pVolume );
	CVolume *volume() { return m_pVolume; }

	void updateView( bool bUpdate = true );

	QPixmap imagePixmap() { return m_Pixmap; }
	int imagePalette();

	bool isTitleShow();
	bool isGridLineShow();
	bool isBoundingBoxShow();

	void setShowTitle( bool flag, bool update=true );
	void setShowGridLine( bool flag, bool update=true );
	void setShowBoundingBox( bool flag, bool update=true );

	bool isFilteredShow();
	void setShowFilteredVoxel( bool flag, bool update=true );

protected:
	virtual void paintGL();
	virtual void resizeGL( int w, int h );

	virtual void paintEvent( QPaintEvent *e );
	virtual void keyPressEvent( QKeyEvent *e );
	virtual void keyReleaseEvent( QKeyEvent *e );
	virtual void mousePressEvent( QMouseEvent *e );
	virtual void mouseReleaseEvent( QMouseEvent *e );
	virtual void mouseMoveEvent( QMouseEvent *e );

public slots:
	void slotZoomAll( bool bDefault=false );
	void slotTopView();
	void slotBottomView();
	void slotLeftView();
	void slotRightView();
	void slotFrontView();
	void slotBackView();

	void slotWhiteToBlackColor();
	void slotBlackToWhiteColor();
	void slotSingleColor();
	void slotIndexColor();
	void slotSelectSingleColor();
	void slotSelectIndexRange();

	void slotImageChanged( bool );

signals:
	void imageChanged( bool );

private:
	void renderScene();

	void drawVolume();
	void drawBoundingBox();
	void drawWorkspace( int gridNum, float gridSpace, float fZ=0.0f );
	void drawTitle();

	void setupViewingFrustum();
	void setupViewingTransform();
	void setupOpenGLState();

	void setViewingDistance( float dist );

	void pointTo3DVector( QPoint point, GLfloat *pVec );
	RGB  getRenderingColor( DWORD id );
	void initIndexColor1();	// for white color is bone
	void initIndexColor2(); // for black color is bone

	BYTE checkVisibleSurface();

private:
	CVolume *m_pVolume;
	QPixmap m_Pixmap;

	enum ManipulateMode { NONE=0, TRACKBALL, ZOOMING, PANNING } m_eManipulationMode;

//	GLuint m_nObject;

	GLfloat m_fFovy;
	GLfloat m_fNearFactor;
	GLfloat m_fFarFactor;

	GLfloat m_fZoomScale;
	GLfloat m_fPanOffsetX;
	GLfloat m_fPanOffsetY;

	QPoint m_ptPrev;
	GLfloat m_fViewingDistance;

	GLfloat m_vAxis[3]; 
	GLfloat m_fTrackingAngle;
	GLfloat m_mxTransform[4][4];

	GLfloat m_vViewport[4];

	GLfloat m_vPrevVec[3];
	GLfloat m_vCurVec[3];

	bool m_bResizeFlag;
	bool m_bIsTracking;

	bool m_bControlKey;

	bool m_bDefaultView;
	bool m_bZoomAll;

	bool m_bIndexColor;
	bool m_bSingleColor;
	bool m_bWhiteBlack;

	int m_nIndexRange[2];
	RGB m_rgbSingleColor;
	RGB m_rgbIndexColor[25];

	bool m_bShowTitle;
	bool m_bShowGridLine;
	bool m_bShowBoundingBox;

	bool m_bShowFilteredVoxel;

//	static int m_nListCount;
};

#endif // _GLVIEW_H_
