/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _IMPORTSTACK_H_
#define _IMPORTSTACK_H_

#include "global_define.h"
#include "define.h"
#include "../BaseDialog/ImportTool/ImportFileSelectionDlg_base.h"
#include "../BaseDialog/ImportTool/ImportImageSizeDlg_base.h"
#include "../BaseDialog/ImportTool/ImportVoxelInfoDlg_base.h"
#include "../BaseDialog/ImportTool/ImportROIDlg_base.h"
#include "../BaseDialog/ImportTool/ImportROIPreviewDlg_base.h"
#include "../BaseDialog/ImportTool/ImportBoneColorDlg_base.h"

#include <qwizard.h>
#include <qimage.h>

// class QImage;
class QScrollView;
class QPoint;
class QSize;
class CImportFileSelectionDlg;
class CImportImageSizeDlg;
class CImportVoxelInfoDlg;
class CImportROIDlg;
class CImportROIPreviewDlg;
class CImportBoneColorDlg;


class CImportStackWizard : public QWizard
{
	Q_OBJECT

public:
	CImportStackWizard( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportStackWizard();

	void showPage(QWidget* page);

	QImage *image();

	QString imageFileName();
	void setImageFileName( QString str );

	QString imageFormat();
	void setImageFormat( QString str );

	int numSlice() { return m_nNumSlice; }
	void setNumSlice( int slice ) { m_nNumSlice = slice; }

	KEYWORD keyword() { return m_Keyword; }

	QStringList fileList();

protected:
	void setupSelectionPage();
	void setupImageSizePage();
	void setupVoxelInfoPage();
	void setupROIPage();
	void setupBoneColorPage();

public slots:
	void slotFinishWizard();

protected slots:
	void reject();

private:
	CImportFileSelectionDlg *m_pFileSelectionDlg;
	CImportImageSizeDlg *m_pImageSizeDlg;
	CImportVoxelInfoDlg *m_pVoxelInfoDlg;
	CImportROIDlg *m_pROIDlg;
	CImportBoneColorDlg *m_pBoneColorDlg;

	KEYWORD m_Keyword;

	QImage *m_pImage;

	QString m_strImageFileName;
	QString m_strImageFormat;

	int m_nNumSlice;
}; // end of class CImportStackWizard


class CImportFileSelectionDlg : public CImportFileSelectionDlg_base
{
	Q_OBJECT

public:
	CImportFileSelectionDlg( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportFileSelectionDlg();

	void init();

	QStringList fileList();
	QListBox *fileListBox() { return IDC_LIST_FILE; }

public slots:
	void slotFileList();
	void slotFileFormat( const QString & );
	void slotAddFiles();
	void slotRemoveFiles();
	void slotClearAll();
	void slotSortList( int index );

private:
	CImportStackWizard *m_pParent;

	int m_nOldSortIndex;
}; // end of clas CImportFileSelectionDlg


class CImportImageSizeDlg : public CImportImageSizeDlg_base
{
	Q_OBJECT

public:
	CImportImageSizeDlg( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportImageSizeDlg();

	void init();

	int imageWidth() { return m_nWidth; }
	void setImageWidth( int width ) { m_nWidth = width; }

	int imageHeight() { return m_nHeight; }
	void setImageHeight( int height ) { m_nHeight = height; }

public slots:
	void slotSetDefault();
	void slotWidthChanged( int value );
	void slotHeightChanged( int value );

private:
	CImportStackWizard *m_pParent;

	int m_nWidth;
	int m_nHeight;
}; // end of class CImportImageSizeDlg


class CImportVoxelInfoDlg : public CImportVoxelInfoDlg_base
{
	Q_OBJECT

public:
	CImportVoxelInfoDlg( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportVoxelInfoDlg();

	void init();

	double voxelWidth() { return m_fWidth; }
	void setVoxelWidth( double width ) { m_fWidth = width; }

	double voxelHeight() { return m_fHeight; }
	void setVoxelHeight( double height ) { m_fHeight = height; }

	double voxelDepth() { return m_fDepth; }
	void setVoxelDepth( double depth ) { m_fDepth = depth; }

public slots:
	void slotValueChanged();

private:
	CImportStackWizard *m_pParent;

	double m_fWidth;
	double m_fHeight;
	double m_fDepth;
}; // end of class CImportVoxelInfoDlg


class CImportROIDlg : public CImportROIDlg_base
{
	Q_OBJECT

public:
	CImportROIDlg( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportROIDlg();

	void init();

	int widthUp() { return m_nWidthUp; }
	int widthDown() { return m_nWidthDown; }

	int heightUp() { return m_nHeightUp; }
	int heightDown() { return m_nHeightDown; }

	int depthUp() { return m_nDepthUp; }
	int depthDown() { return m_nDepthDown; }

	void setWidthRange( int up, int down );
	void setHeightRange( int up, int down );
	void setDepthRange( int up, int down );

public slots:
	void slotSetDefault();
	void slotSelectROI();
	void slotWidthUp( int value );
	void slotWidthDown( int value );
	void slotHeightUp( int value );
	void slotHeightDown( int value );
	void slotDepthUp( int value );
	void slotDepthDown( int value );

private:
	void setRequiredMemory();

private:
	CImportStackWizard *m_pParent;

	int m_nWidthUp;
	int m_nWidthDown;

	int m_nHeightUp;
	int m_nHeightDown;

	int m_nDepthUp;
	int m_nDepthDown;
}; // end of class CImportROIDlg


class CROIPreviewArea;

class CImportROIPreviewDlg : public CImportROIPreviewDlg_base
{
	Q_OBJECT

public:
	CImportROIPreviewDlg( CImportStackWizard *pWizard,
		QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=WType_Modal );
	~CImportROIPreviewDlg();

	void init();

protected:
	virtual void resizeEvent( QResizeEvent * );

public slots:
	void slotSliceChanged( int );
	void slotEndROI();
	void slotMoveImageToCenter();

signals:
	void viewResized();

private:
	CImportStackWizard *m_pWizard;
	CImportROIDlg *m_pParent;

	QStringList m_StringList;
	QString m_ImageFormat;

	QScrollView *m_pScrollView;
	CROIPreviewArea *m_pROIPreviewArea;
}; // end of class CImportROIPreviewDlg

class CROIPreviewArea : public QWidget
{
	Q_OBJECT

public:
	CROIPreviewArea( QWidget *parent=0, const char *name=0, WFlags fl=0 );
	~CROIPreviewArea();

	bool init( QString fileName, QString imageFormat );

protected:
	virtual void mousePressEvent( QMouseEvent * );
	virtual void mouseMoveEvent( QMouseEvent * );
	virtual void mouseReleaseEvent( QMouseEvent * );
	virtual void paintEvent( QPaintEvent * );
	virtual void resizeEvent( QResizeEvent * );

private:
	CImportROIPreviewDlg *m_pParent;

	QPoint m_ptFirst;
	QPoint m_ptLast;
	QImage *m_pBuffer;
	QSize m_szImage;

	bool m_bChangeFlag;
}; // end of class CROIPreviewArea


class CImportBoneColorDlg : public CImportBoneColorDlg_base
{
	Q_OBJECT

public:
	CImportBoneColorDlg( QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags fl=0 );
	~CImportBoneColorDlg();

	void init();

	bool whiteIsBone() { return m_bWhiteIsBone; }
	void setwhiteIsBone( bool bone ) { m_bWhiteIsBone = bone; }

public slots:
	void slotWhiteButtonClicked();
	void slotBlackButtonClicked();

private:
	bool m_bWhiteIsBone;
}; // end of class CImportBoneColorDlg

#endif // _IMPORTSTACK_H_
