/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MultiLineEditDlg.h"
#include <qmultilineedit.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qfiledialog.h>

CMultiLineEditDlg::CMultiLineEditDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CMultiLineEditDlg_base( parent, name, modal, f )
{
#ifdef Q_WS_WIN
	setWFlags( f | Qt::WStyle_MinMax | Qt::WStyle_SysMenu );
#endif

#ifdef Q_WS_X11
	setWFlags( f | Qt::WStyle_MinMax );
#endif

	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( IDC_BTN_SAVE, SIGNAL( clicked() ),
		this, SLOT( slotSave() ) );
} /* CMultiLineEditDlg */

CMultiLineEditDlg::~CMultiLineEditDlg()
{
} /* ~CMultiLineEditDlg */

bool CMultiLineEditDlg::init( 
		QString fname, QString caption, bool bSave, bool bEdit, bool bShowFrame )
{
	resize( 800, 600 );

	m_bShowFrame = bShowFrame;
	if ( bShowFrame )
	{
		IDC_MEDIT_REPORT->setGeometry( 0, 0, width(), height()-40 );
		IDC_FRAME_BOTTOM->setGeometry( 0, height()-40, width(), 40 );
	}
	else
	{
		IDC_MEDIT_REPORT->setGeometry( 0, 0, width(), height() );
		IDC_FRAME_BOTTOM->hide();
	}

	if ( !bSave )
		IDC_BTN_SAVE->hide();

	setCaption( caption );

	// load file to multi line edit
	QFile f( fname );
	if ( !f.open( IO_ReadOnly ) )
		return false;

	QTextStream t( &f );

    QString s = t.read();
	IDC_MEDIT_REPORT->setText( s );

	f.close();

	if ( !bEdit )
		IDC_MEDIT_REPORT->setReadOnly( true );

	IDC_MEDIT_REPORT->repaint();

	return true;
} /* init */

bool CMultiLineEditDlg::init( QString caption )
{
	m_bShowFrame = false;

	resize( 800, 600 );

	IDC_MEDIT_REPORT->setGeometry( 0, 0, width(), height() );
	IDC_FRAME_BOTTOM->hide();

	IDC_MEDIT_REPORT->setAutoUpdate( true );

	setCaption( caption );

	return true;
} /* init */

// slot
void CMultiLineEditDlg::slotOk()
{
	QDialog::accept();
} /* slotOk */

// slot
void CMultiLineEditDlg::slotSave()
{
	QString fn = QFileDialog::getSaveFileName( NULL, "*.inf", this );
    if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".inf", 0 ) != -1 || fn.find( ".INF", 0 ) != -1 )
			;
		else
			fn.append( ".inf" );

		QFile f( fn );

		if ( !f.open( IO_WriteOnly ) )
			return;

		QTextStream t( &f );
		t << IDC_MEDIT_REPORT->text();

		f.close();
	}

} /* slotSave */

// virtual
void CMultiLineEditDlg::resizeEvent( QResizeEvent * /*e*/ )
{
	if ( m_bShowFrame )
	{
		IDC_MEDIT_REPORT->setGeometry( 0, 0, width(), height()-40 );
		IDC_FRAME_BOTTOM->setGeometry( 0, height()-40, width(), 40 );
	}
	else
		IDC_MEDIT_REPORT->setGeometry( 0, 0, width(), height() );
} /* resizeEvent */
