/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _PLANVIEW_H_
#define _PLANVIEW_H_

#include "global_define.h"
#include "../BaseDialog/PlanView/PlanView_base.h"
#include <qimage.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qlist.h>


class QTimer;
class QSize;
class QPoint;
class QRegion;
class QScrollView;
class QPopupMenu;

class MDIWindow;
class CVolume;
class CPlanViewArea;


enum { NONE_MEASURE = 0, MEASURE_MEASURE, MEASURE_ANGLE, MEASURE_REGION };
enum { NONE_ROI = 0, ROI_FREE_REGION };
enum { FILL_SEGMENTATION = 0, ERASE_SEGMENTATION };


class CPlanView : public CPlanView_base
{
	Q_OBJECT

public:
	CPlanView( QWidget *parent=0, const char *name=0, WFlags f=0 );
	~CPlanView();

	void setVolume( CVolume *pVolume );
	MDIWindow *mdiWindow();
	CVolume *volume();
	CPlanViewArea *planViewArea();

	int  currentSlice() { return m_nCurrentSlice; }

	bool isSegmentationShow();
	void setShowSegmentation( bool flag );

	bool isMeasureLineShow();
	void setShowMeasureLine( bool flag );

	int  viewType() { return m_nType; }
	bool checkViewType( int type );

	bool isTimerOn();

	void enterDrawMode( /*int mode*/ );
	void leaveDrawMode();

	void clearROI();
	void clearMeasureLine();

	bool resampling( QString &fileType, QString &directory, QString &keyword,
		int viewType, WORD wStart, WORD wEnd );

// private:
	bool init( MDIWindow *pMDIWindow );
	DWORD voxelIndex( WORD x, WORD y, int slice );
	void viewTypeChanged( int type );

protected:
	virtual void resizeEvent( QResizeEvent * );

signals:
	void sliceChanged( int );
	void viewResized();

public slots:
	void slotSetSlice( int slice );			// for external use
	void slotSliceChanged( int slice );		// for internal use

	void slotZoomIn();
	void slotZoomOut();
	void slotZoomAll();

	void slotXView();
	void slotYView();
	void slotZView();

	void slotNextSlice();
	void slotPreviousSlice();

	void slotTypeChanged( int type );

	void slotAnimationButton();
	void slotAnimation();

private slots:
	void slotMoveImageToCenter();

private:
	MDIWindow *m_pMDIWindow;
	CVolume *m_pVolume;
	int m_nType;	// 1: x-view  2: y-view  3: z-view (default)

	WORD m_nWidth;
	WORD m_nHeight;
	WORD m_nDepth;

	QScrollView *m_pScrollView;
	CPlanViewArea *m_pPlanViewArea;

	QTimer *m_pAnimationTimer;
	double m_fRatio;
	int m_nCurrentSlice;

	bool m_bShowSegmentation;
	bool m_bShowMeasureLine;
}; // end of class CPlanView


class CPlanViewArea : public QWidget
{
	Q_OBJECT

public:
	CPlanViewArea( QWidget *parent=0, const char *name = 0, WFlags f=0 );
	~CPlanViewArea();

	void init( QImage image, int slice, int type );
	void scaleImage( double fRatio );

	QList<QRegion> listRgnROI();

	void enterDrawMode( /*int mode*/ );
	void leaveDrawMode();

	void clearROI();
	void clearMeasureLine();

	bool isMeasure() { return m_bMeasure; }
	QPoint lastPoint() { return m_ptLast; }

private:
	void setPopupMenu();

	void calculateMeasureLine();
	void calculateAngle();

	float lineLength( QPoint pt1, QPoint pt2 );
	bool isMouseInsideROI( QPoint pt );

public slots:
	void slotFillEraseSizePopupAboutToShow();
	void slotFillEraseSize( int id );

protected:
	virtual void paintEvent( QPaintEvent * );
	virtual void resizeEvent( QResizeEvent * );
	virtual void mousePressEvent( QMouseEvent * );
	virtual void mouseReleaseEvent( QMouseEvent * );
	virtual void mouseMoveEvent( QMouseEvent * );
	virtual void keyPressEvent( QKeyEvent * );
	virtual void focusInEvent( QFocusEvent * );
	virtual void focusOutEvent( QFocusEvent * );

private:
	CPlanView *m_pPlanView;
	CVolume *m_pVolume;

	QImage m_OriginalImage;		// original image
	QImage m_Image;				// view image
	QPixmap m_Pixmap;			// display purpose only
	QSize m_szImage;

	int m_nType;
	double m_fRatio;

	int m_nWidth;
	int m_nHeight;
	int m_nSlice;

//	int m_nMeasureMode;

	// measure
	bool m_bMeasure;
	int m_nPtCount;				// temporary : number of point
	QPoint m_ptMeasureFirst;	// to measure first point to last point
	// 1. for measure
	QList<QPoint> m_listPtMeasure;
	QList<float> m_listLineValue;
	QList<float> m_listAreaValue;
	// 2. for angle
	QList<QPoint> m_listPtAngle;
	QList<float> m_listAngleValue;

	// ROI
	QList<QPoint> m_listPtROI;	// for display
	QList<QRegion> m_listRgnROI;
	QPointArray *m_pPtArray;

	QPoint m_ptFirst;
	QPoint m_ptLast;

	// fill & erase segmentation
	bool m_bFillEraseSegmentation;

	// popup menu
	QPopupMenu *m_pFillEraseSizePopup;
	int m_nFillEraseSize;
	int m_nFillEraseSizeId;
}; // end of CPlanViewArea

#endif // _PLANVIEW_H_
