/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include <assert.h>

#include "link.h"

size_t list_length( MY_NODE * head_ptr )
{
	MY_NODE *cursor;
	size_t answer;

	answer = 0;
	for( cursor = head_ptr ; cursor != NULL ; cursor = cursor->link )
		answer++;

	return answer;
} /* list_length */

void list_head_insert( MY_NODE *& head_ptr, const MY_NODE::Item & entry )
{
	MY_NODE *insert_ptr;

	insert_ptr = new MY_NODE;
	insert_ptr->data = entry;
	insert_ptr->link = head_ptr;
	head_ptr = insert_ptr;
} /* list_head_insert */

void list_insert( MY_NODE * previous_ptr, const MY_NODE::Item & entry )
{
	MY_NODE *insert_ptr;
	
	insert_ptr = new MY_NODE;
	insert_ptr->data = entry;
	insert_ptr->link = previous_ptr->link;
	previous_ptr->link = insert_ptr;
} /* list_insert */

MY_NODE * list_search( MY_NODE * head_ptr, const MY_NODE::Item & target )
{
	MY_NODE *cursor;

	for( cursor = head_ptr ; cursor != NULL ; cursor = cursor->link )
		if( target == cursor->data )
			return cursor;

	return NULL;
} /* list_search */

MY_NODE * list_locate( MY_NODE * head_ptr, size_t position )
{
	MY_NODE *cursor;
	size_t i;

	assert( 0 < position );
	cursor = head_ptr;
	for( i = 1 ; (i <position) && (cursor != NULL) ; i++ )
		cursor = cursor->link;

	return cursor;
} /* list_locate */

void list_copy(	MY_NODE * source_ptr, MY_NODE *& head_ptr, MY_NODE *& tail_ptr )
{
	head_ptr = NULL;
	tail_ptr = NULL;

	if( source_ptr == NULL )
		return;

	list_head_insert( head_ptr, source_ptr->data );
	tail_ptr = head_ptr;

	for( source_ptr = source_ptr->link ; source_ptr != NULL ; source_ptr = source_ptr->link )
	{
		list_insert( tail_ptr, source_ptr->data );
		tail_ptr = tail_ptr->link;
	}
} /* list_copy */

void list_piece( 
		MY_NODE * start_ptr, MY_NODE * end_ptr,
		MY_NODE *& head_ptr, MY_NODE *& tail_ptr )
{
} /* list_piece */

void list_head_remove( MY_NODE *& head_ptr )
{
	MY_NODE *remove_ptr;

	remove_ptr = head_ptr;
	head_ptr = head_ptr->link;

	delete remove_ptr;
	remove_ptr = NULL;
} /* list_head_remove */

void list_remove( MY_NODE * previous_ptr )
{
	MY_NODE *remove_ptr;

	remove_ptr = previous_ptr->link;
	previous_ptr->link = remove_ptr->link;

	delete remove_ptr;
	remove_ptr = NULL;
} /* list_remove */

void list_clear( MY_NODE *& head_ptr )
{
	while( head_ptr != NULL )
		list_head_remove( head_ptr );
} /* list_clear */
