/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include <assert.h>		// assert()
#include <stdlib.h>		// size_t

#include "queue.h"

CQueue::CQueue()
{
	count = 0;
	front_ptr = NULL;
} /* CQueue */

CQueue::CQueue( const CQueue & source )
{
	count = source.count;
	list_copy( source.front_ptr, front_ptr, rear_ptr );
} /* CQueue */

CQueue::~CQueue()
{
	list_clear( front_ptr );
} /* ~CQueue */

void CQueue::operator = ( const CQueue & source )
{
	if( source.front_ptr == front_ptr )
		return;

	list_clear( front_ptr );
	count = source.count;
	list_copy( source.front_ptr, front_ptr, rear_ptr );
} /* operator = */

void CQueue::insert( const MY_NODE::Item & entry )
{
	if( is_empty() ){
		list_head_insert( front_ptr, entry );
		rear_ptr = front_ptr;
	}
	else{
		list_insert( rear_ptr, entry );
		rear_ptr = rear_ptr->link;
	}

	count++;
} /* insert */

MY_NODE::Item CQueue::get_front()
{
	MY_NODE::Item answer;

	assert( !is_empty() );
	answer = front_ptr->data;
	list_head_remove( front_ptr );
	count--;

	return answer;
} /* get_front */

void CQueue::clear()
{
	list_clear( front_ptr );
	count = 0;
} /* clear */
